@@ -10,7 +10,7 @@ m.Template = m.Class({
 		this._template = template;
 		this._tpCache = new Map();
 	},
-	
+
 	// helper function to return a template value, optionally adjusting for tech.
 	// TODO: there's no support for "_string" values here.
 	get: function(string)
@@ -44,7 +44,7 @@ m.Template = m.Class({
 			return undefined;
 		return this.get("Identity/GenericName");
 	},
-						  
+
 	rank: function() {
 		if (!this.get("Identity"))
 			return undefined;
@@ -61,13 +61,13 @@ m.Template = m.Class({
 	requiredTech: function() {
 		return this.get("Identity/RequiredTechnology");
 	},
-						  
+
 	available: function(gameState) {
 		if (this.requiredTech() === undefined)
 			return true;
 		return gameState.isResearched(this.get("Identity/RequiredTechnology"));
 	},
-						  
+
 	// specifically
 	phase: function() {
 		if (!this.get("Identity/RequiredTechnology"))
@@ -82,17 +82,21 @@ m.Template = m.Class({
 	},
 
 	hasClass: function(name) {
-		var classes = this.classes();
+		if (!this._classes)
+			this._classes = this.classes();
+		var classes = this._classes;
 		return (classes && classes.indexOf(name) != -1);
 	},
-	
+
 	hasClasses: function(array) {
-		var classes = this.classes();
+		if (!this._classes)
+			this._classes = this.classes();
+		let classes = this._classes;
 		if (!classes)
 			return false;
-		
-		for (var i in array)
-			if (classes.indexOf(array[i]) === -1)
+
+		for (let cls of array)
+			if (classes.indexOf(cls) === -1)
 				return false;
 		return true;
 	},
@@ -110,11 +114,11 @@ m.Template = m.Class({
 			ret[type] = +this.get("Cost/Resources/" + type);
 		return ret;
 	},
-	
+
 	costSum: function() {
 		if (!this.get("Cost"))
 			return undefined;
-		
+
 		var ret = 0;
 		for (var type in this.get("Cost/Resources"))
 			ret += +this.get("Cost/Resources/" + type);
@@ -141,7 +145,7 @@ m.Template = m.Class({
 
 		return 0; // this should never happen
 	},
-						  
+
 	/**
 	 * Returns the radius of a circle surrounding this entity's
 	 * footprint.
@@ -149,17 +153,17 @@ m.Template = m.Class({
 	footprintRadius: function() {
 		if (!this.get("Footprint"))
 			return undefined;
-		
+
 		if (this.get("Footprint/Square"))
 		{
 			var w = +this.get("Footprint/Square/@width");
 			var h = +this.get("Footprint/Square/@depth");
 			return Math.sqrt(w*w + h*h) / 2;
 		}
-		
+
 		if (this.get("Footprint/Circle"))
 			return +this.get("Footprint/Circle/@radius");
-		
+
 		return 0; // this should never happen
 	},
 
@@ -230,7 +234,7 @@ m.Template = m.Class({
 			crush: +(this.get("Attack/" + type + "/Crush") || 0)
 		};
 	},
-	
+
 	attackTimes: function(type) {
 		if (!this.get("Attack/" + type +""))
 			return undefined;
@@ -246,9 +250,10 @@ m.Template = m.Class({
 	getCounteredClasses: function() {
 		if (!this.get("Attack"))
 			return undefined;
-		
+
 		var Classes = [];
-		for (var i in this.get("Attack")) {
+		for (var i in this.get("Attack"))
+		{
 			if (!this.get("Attack/" + i + "/Bonuses"))
 				continue;
 			for (var o in this.get("Attack/" + i + "/Bonuses"))
@@ -264,7 +269,8 @@ m.Template = m.Class({
 		if (!this.get("Attack"))
 			return false;
 		var mcounter = [];
-		for (var i in this.get("Attack")) {
+		for (var i in this.get("Attack"))
+		{
 			if (!this.get("Attack/" + i + "/Bonuses"))
 				continue;
 			for (var o in this.get("Attack/" + i + "/Bonuses"))
@@ -285,7 +291,8 @@ m.Template = m.Class({
 			return undefined;
 
 		if (this.get("Attack/" + type + "/Bonuses"))
-			for (var o in this.get("Attack/" + type + "/Bonuses")) {
+			for (var o in this.get("Attack/" + type + "/Bonuses"))
+			{
 				if (!this.get("Attack/" + type + "/Bonuses/" + o + "/Classes"))
 					continue;
 				var total = this.get("Attack/" + type + "/Bonuses/" + o + "/Classes").split(" ");
@@ -300,7 +307,7 @@ m.Template = m.Class({
 	canAttackClass: function(saidClass) {
 		if (!this.get("Attack"))
 			return false;
-		
+
 		for (var i in this.get("Attack")) {
 			if (!this.get("Attack/" + i + "/RestrictedClasses") || !this.get("Attack/" + i + "/RestrictedClasses/_string"))
 				continue;
@@ -376,7 +383,9 @@ m.Template = m.Class({
 	resourceDropsiteTypes: function() {
 		if (!this.get("ResourceDropsite"))
 			return undefined;
-		return this.get("ResourceDropsite/Types").split(/\s+/);
+
+		let types = this.get("ResourceDropsite/Types");
+		return types ? types.split(/\s+/) : [];
 	},
 
 
@@ -404,6 +413,12 @@ m.Template = m.Class({
 		return +this.get("BuildingAI/GarrisonArrowMultiplier");
 	},
 
+	getGarrisonArrowClasses: function() {
+		if (!this.get("BuildingAI"))
+			return undefined;
+		return this.get("BuildingAI/GarrisonArrowClasses").split(/\s+/);
+	},
+
 	buffHeal: function() {
 		if (!this.get("GarrisonHolder"))
 			return undefined;
@@ -415,7 +430,7 @@ m.Template = m.Class({
 			return undefined;
 		return this.get("Promotion/Entity");
 	},
-	
+
 	/**
 	 * Returns whether this is an animal that is too difficult to hunt.
 	 * (Any non domestic currently.)
@@ -441,13 +456,19 @@ m.Template = m.Class({
 			this.get("UnitAI/NaturalBehaviour") === "aggressive" ||
 			this.get("UnitAI/NaturalBehaviour") === "defensive");
 	},
-						  
+
 	walkSpeed: function() {
 		if (!this.get("UnitMotion") || !this.get("UnitMotion/WalkSpeed"))
 			 return undefined;
 		return this.get("UnitMotion/WalkSpeed");
 	},
 
+	trainingCategory: function() {
+		if (!this.get("TrainingRestrictions") || !this.get("TrainingRestrictions/Category"))
+			return undefined;
+		return this.get("TrainingRestrictions/Category");
+	},
+
 	buildCategory: function() {
 		if (!this.get("BuildRestrictions") || !this.get("BuildRestrictions/Category"))
 			return undefined;
@@ -552,11 +573,11 @@ m.Entity = m.Class({
 	setMetadata: function(player, key, value) {
 		this._ai.setMetadata(player, this, key, value);
 	},
-	
+
 	deleteAllMetadata: function(player) {
 		delete this._ai._entityMetadata[player][this.id()];
 	},
-				  
+
 	deleteMetadata: function(player, key) {
 		this._ai.deleteMetadata(player, this, key);
 	},
@@ -568,10 +589,10 @@ m.Entity = m.Class({
 			return undefined;
 		return this._entity.idle;
 	},
-	
-	unitAIState: function() { return this._entity.unitAIState; },
+
+	unitAIState: function() { if (this._entity.unitAIState) return this._entity.unitAIState; return undefined; },
 	unitAIOrderData: function() { return this._entity.unitAIOrderData; },
-	
+
 	// TODO  understand why we have sometimes rounding problems with maxHitpoints ? making wrongly isHurt=true
 	// problem seems to be with hele civs (i.e. spart)
 	hitpoints: function() { if (this._entity.hitpoints !== undefined) return this._entity.hitpoints; return undefined; },
@@ -605,7 +626,7 @@ m.Entity = m.Class({
 			return undefined;
 		return this._entity.foundationProgress;
 	},
-	
+
 	getBuilders: function() {
 		if (this._entity.foundationProgress === undefined)
 			return undefined;
@@ -613,7 +634,7 @@ m.Entity = m.Class({
 			return [];
 		return this._entity.foundationBuilders;
 	},
-	
+
 	getBuildersNb: function() {
 		if (this._entity.foundationProgress === undefined)
 			return undefined;
@@ -631,11 +652,11 @@ m.Entity = m.Class({
 			return false;
 		return this._entity.owner === player;
 	},
-	
+
 	isFriendly: function(player) {
 		return this.isOwn(player); // TODO: diplomacy
 	},
-	
+
 	isEnemy: function(player) {
 		return !this.isOwn(player); // TODO: diplomacy
 	},
@@ -645,14 +666,14 @@ m.Entity = m.Class({
 			return undefined;
 		return this._entity.resourceSupplyAmount;
 	},
-				  
+
 	resourceSupplyGatherers: function()
 	{
 		if (this._entity.resourceSupplyGatherers !== undefined)
 			return this._entity.resourceSupplyGatherers;
 		return [];
 	},
-				  
+
 	isFull: function()
 	{
 		if (this._entity.resourceSupplyGatherers !== undefined)
@@ -666,34 +687,34 @@ m.Entity = m.Class({
 			return undefined;
 		return this._entity.resourceCarrying; 
 	},
-				  
+
 	currentGatherRate: function() {
 		// returns the gather rate for the current target if applicable.
 		if (!this.get("ResourceGatherer"))
 			return undefined;
-		
+
 		if (this.unitAIOrderData().length &&
 			(this.unitAIState().split(".")[1] === "GATHER" || this.unitAIState().split(".")[1] === "RETURNRESOURCE"))
 		{
 			var ress = undefined;
 			// this is an abuse of "_ai" but it works.
 			if (this.unitAIState().split(".")[1] === "GATHER" && this.unitAIOrderData()[0]["target"] !== undefined)
-				ress = this._ai._entities[this.unitAIOrderData()[0]["target"]];
+				ress = this._ai._entities.get(this.unitAIOrderData()[0]["target"]);
 			else if (this.unitAIOrderData()[1] !== undefined && this.unitAIOrderData()[1]["target"] !== undefined)
-				ress = this._ai._entities[this.unitAIOrderData()[1]["target"]];
-			
+				ress = this._ai._entities.get(this.unitAIOrderData()[1]["target"]);
+
 			if (ress == undefined)
 				return undefined;
-			
+
 			var type = ress.resourceSupplyType();
 			var tstring = type.generic + "." + type.specific;
-				  
+
 			if (type.generic == "treasure")
 				return 1000;
-				
+
 			var speed = +this.get("ResourceGatherer/BaseSpeed");
 			speed *= +this.get("ResourceGatherer/Rates/" +tstring);
-				  
+
 			if (speed)
 				return speed;
 			return 0;
@@ -703,8 +724,8 @@ m.Entity = m.Class({
 
 	isGarrisonHolder: function() { return this.get("GarrisonHolder") },
 
-	garrisoned: function() { return new m.EntityCollection(this._ai, this._entity.garrisoned); },
-	
+	garrisoned: function() { return this._entity.garrisoned; },
+
 	canGarrisonInside: function() { return this._entity.garrisoned.length < this.garrisonMax(); },
 
 	// TODO: visibility
@@ -720,7 +741,7 @@ m.Entity = m.Class({
 		Engine.PostCommand(PlayerID,{"type": "walk-to-range", "entities": [this.id()], "x": x, "z": z, "min": min, "max": max, "queued": queued });
 		return this;
 	},
-	
+
 	attackMove: function(x, z, targetClasses, queued) {
 		queued = queued || false;
 		Engine.PostCommand(PlayerID,{"type": "attack-walk", "entities": [this.id()], "x": x, "z": z, "targetClasses": targetClasses, "queued": queued });
@@ -762,7 +783,7 @@ m.Entity = m.Class({
 		Engine.PostCommand(PlayerID,{"type": "attack", "entities": [this.id()], "target": unitId, "queued": false});
 		return this;
 	},
-	
+
 	// moveApart from a point in the opposite direction with a distance dist
 	moveApart: function(point, dist) {
 		if (this.position() !== undefined) {
@@ -774,7 +795,7 @@ m.Entity = m.Class({
 			{
 				direction[0] /= norm;
 				direction[1] /= norm;
-			}			
+			}
 			Engine.PostCommand(PlayerID,{"type": "walk", "entities": [this.id()], "x": this.position()[0] + direction[0]*dist, "z": this.position()[1] + direction[1]*dist, "queued": false});
 		}
 		return this;
@@ -787,7 +808,7 @@ m.Entity = m.Class({
 			var dist = m.VectorDistance(unitToFleeFrom.position(), this.position() );
 			FleeDirection[0] = (FleeDirection[0]/dist) * 8;
 			FleeDirection[1] = (FleeDirection[1]/dist) * 8;
-			
+
 			Engine.PostCommand(PlayerID,{"type": "walk", "entities": [this.id()], "x": this.position()[0] + FleeDirection[0]*5, "z": this.position()[1] + FleeDirection[1]*5, "queued": false});
 		}
 		return this;
@@ -804,7 +825,7 @@ m.Entity = m.Class({
 		Engine.PostCommand(PlayerID,{"type": "repair", "entities": [this.id()], "target": target.id(), "autocontinue": false, "queued": queued});
 		return this;
 	},
-	
+
 	returnResources: function(target, queued) {
 		queued = queued || false;
 		Engine.PostCommand(PlayerID,{"type": "returnresource", "entities": [this.id()], "target": target.id(), "queued": queued});
@@ -815,7 +836,7 @@ m.Entity = m.Class({
 		Engine.PostCommand(PlayerID,{"type": "delete-entities", "entities": [this.id()] });
 		return this;
 	},
-	
+
 	barter: function(buyType, sellType, amount) {
 		Engine.PostCommand(PlayerID,{"type": "barter", "sell" : sellType, "buy" : buyType, "amount" : amount });
 		return this;
@@ -825,7 +846,7 @@ m.Entity = m.Class({
 		Engine.PostCommand(PlayerID,{"type": "setup-trade-route", "entities": [this.id()], "target": target.id(), "source": source.id(), "route": undefined, "queued": false });
 		return this;
 	},
-	
+
 	setRallyPoint: function(target, command) {
 		var data = {"command": command, "target": target.id()};
 		Engine.PostCommand(PlayerID, {"type": "set-rallypoint", "entities": [this.id()], "x": target.position()[0], "z": target.position()[1], "data": data});
@@ -880,7 +901,7 @@ m.Entity = m.Class({
 		});
 		return this;
 	},
-				  
+
 	 research: function(template) {
 		Engine.PostCommand(PlayerID,{ "type": "research", "entity": this.id(), "template": template });
 		return this;
@@ -890,16 +911,14 @@ m.Entity = m.Class({
 		Engine.PostCommand(PlayerID,{ "type": "stop-production", "entity": this.id(), "id": id });
 		return this;
 	},
-	
+
 	stopAllProduction: function(percentToStopAt) {
 		var queue = this._entity.trainingQueue;
 		if (!queue)
 			return true;	// no queue, so technically we stopped all production.
-		for (var i in queue)
-		{
-			if (queue[i].progress < percentToStopAt)
-				Engine.PostCommand(PlayerID,{ "type": "stop-production", "entity": this.id(), "id": queue[i].id });
-		}
+		for (var item of queue)
+			if (item.progress < percentToStopAt)
+				Engine.PostCommand(PlayerID,{ "type": "stop-production", "entity": this.id(), "id": item.id });
 		return this;
 	}
 });
