@@ -17,14 +17,13 @@ m.GameState = function() {
 m.GameState.prototype.init = function(SharedScript, state, player) {
 	this.sharedScript = SharedScript;
 	this.EntCollecNames = SharedScript._entityCollectionsName;
-	this.EntCollec = SharedScript._entityCollections;
 	this.timeElapsed = SharedScript.timeElapsed;
 	this.circularMap = SharedScript.circularMap;
 	this.templates = SharedScript._templates;
 	this.techTemplates = SharedScript._techTemplates;
 	this.entities = SharedScript.entities;
 	this.player = player;
-	this.playerData = this.sharedScript.playersData[this.player];
+	this.playerData = SharedScript.playersData[this.player];
 	this.techModifications = SharedScript._techModifications[this.player];
 	this.barterPrices = SharedScript.barterPrices;
 	this.gameType = SharedScript.gameType;
@@ -33,7 +32,6 @@ m.GameState.prototype.init = function(SharedScript, state, player) {
 m.GameState.prototype.update = function(SharedScript, state) {
 	this.sharedScript = SharedScript;
 	this.EntCollecNames = SharedScript._entityCollectionsName;
-	this.EntCollec = SharedScript._entityCollections;
 	this.timeElapsed = SharedScript.timeElapsed;
 	this.templates = SharedScript._templates;
 	this.techTemplates = SharedScript._techTemplates;
@@ -47,67 +45,52 @@ m.GameState.prototype.update = function(SharedScript, state) {
 	this.turnCache = {};
 };
 
-m.GameState.prototype.updatingCollection = function(id, filter, collection, allowQuick){
-	// automatically add the player ID in front.
-	id = this.player + "-" + id;
-	if (!this.EntCollecNames[id])  {
-		if (collection !== undefined)
-			this.EntCollecNames[id] = collection.filter(filter);
-		else {
-			this.EntCollecNames[id] = this.entities.filter(filter);
-		}
-		if (allowQuick)
-			this.EntCollecNames[id].allowQuickIter();
-		this.EntCollecNames[id].registerUpdates();
-		//	warn ("New Collection named " +id);
-	}
-	
-	return this.EntCollecNames[id];
-};
-m.GameState.prototype.destroyCollection = function(id){
-	// automatically add the player ID
-	id = this.player + "-" + id;
-	
-	if (this.EntCollecNames[id] !== undefined){
-		this.sharedScript.removeUpdatingEntityCollection(this.EntCollecNames[id]);
-		delete this.EntCollecNames[id];
-	}
-};
-m.GameState.prototype.getEC = function(id){
-	// automatically add the player ID
-	id = this.player + "-" + id;
-
-	if (this.EntCollecNames[id] !== undefined)
-		return this.EntCollecNames[id];
-	return undefined;
+m.GameState.prototype.updatingCollection = function(id, filter, collection)
+{
+	let gid = this.player + "-" + id;	// automatically add the player ID
+	return this.updatingGlobalCollection(gid, filter, collection);
 };
 
-m.GameState.prototype.updatingGlobalCollection = function(id, filter, collection, allowQuick) {
-	if (!this.EntCollecNames[id]){
-		if (collection !== undefined)
-			this.EntCollecNames[id] = collection.filter(filter);
-		else
-			this.EntCollecNames[id] = this.entities.filter(filter);
-		if (allowQuick)
-			this.EntCollecNames[id].allowQuickIter();
-		this.EntCollecNames[id].registerUpdates();
-		//warn ("New Global Collection named " +id);
-	}
-	
-	return this.EntCollecNames[id];
+m.GameState.prototype.destroyCollection = function(id)
+{
+	let gid = this.player + "-" + id;	// automatically add the player ID
+	return this.destroyGlobalCollection(gid);
 };
+
+m.GameState.prototype.getEC = function(id)
+{
+	let gid = this.player + "-" + id;	// automatically add the player ID
+	return this.getGEC(gid);
+};
+
+m.GameState.prototype.updatingGlobalCollection = function(id, filter, collection)
+{
+	if (this.EntCollecNames.has(id))
+		return this.EntCollecNames.get(id);
+
+	if (collection !== undefined)
+		var newCollection = collection.filter(filter);
+	else
+		var newCollection = this.entities.filter(filter);
+	newCollection.registerUpdates();
+	this.EntCollecNames.set(id, newCollection);	
+	return newCollection;
+};
+
 m.GameState.prototype.destroyGlobalCollection = function(id)
 {
-	if (this.EntCollecNames[id] !== undefined){
-		this.sharedScript.removeUpdatingEntityCollection(this.EntCollecNames[id]);
-		delete this.EntCollecNames[id];
-	}
+	if (!this.EntCollecNames.has(id))
+		return;
+
+	this.sharedScript.removeUpdatingEntityCollection(this.EntCollecNames.get(id));
+	this.EntCollecNames.delete(id);
 };
+
 m.GameState.prototype.getGEC = function(id)
 {
-	if (this.EntCollecNames[id] !== undefined)
-		return this.EntCollecNames[id];
-	return undefined;
+	if (!this.EntCollecNames.has(id))
+		return undefined;
+	return this.EntCollecNames.get(id);
 };
 
 m.GameState.prototype.getTimeElapsed = function()
@@ -314,62 +297,77 @@ m.GameState.prototype.isPlayerEnemy = function(id) {
 	return this.playerData.isEnemy[id];
 };
 
-m.GameState.prototype.getEnemies = function(){
-	var ret = [];
-	for (var i in this.playerData.isEnemy){
-		if (this.playerData.isEnemy[i]){
-			ret.push(i);
-		}
-	}
+m.GameState.prototype.getEnemies = function()
+{
+	let ret = [];
+	for (let i in this.playerData.isEnemy)
+		if (this.playerData.isEnemy[i])
+			ret.push(+i);
 	return ret;
 };
 
-m.GameState.prototype.getAllies = function(){
-	var ret = [];
-	for (var i in this.playerData.isAlly)
+m.GameState.prototype.getAllies = function()
+{
+	let ret = [];
+	for (let i in this.playerData.isAlly)
 		if (this.playerData.isAlly[i])
-			ret.push(i);
+			ret.push(+i);
 	return ret;
 };
 
-m.GameState.prototype.getExclusiveAllies = function(){  // Player is not included
-	var ret = [];
-	for (var i in this.playerData.isAlly)
+m.GameState.prototype.getExclusiveAllies = function()
+{	// Player is not included
+	let ret = [];
+	for (let i in this.playerData.isAlly)
 		if (this.playerData.isAlly[i] && +i !== this.player)
-			ret.push(i);
+			ret.push(+i);
 	return ret;
 };
 
-m.GameState.prototype.isEntityAlly = function(ent) {
-	if (ent && ent.owner && (typeof ent.owner) === "function"){
+m.GameState.prototype.isEntityAlly = function(ent)
+{
+	if (!ent || !ent.owner)
+		return false;
+	if ((typeof ent.owner) === "function")
 		return this.playerData.isAlly[ent.owner()];
-	} else if (ent && ent.owner){
+	else
 		return this.playerData.isAlly[ent.owner];
-	}
-	return false;
 };
 
-m.GameState.prototype.isEntityEnemy = function(ent) {
-	if (ent && ent.owner && (typeof ent.owner) === "function"){
+m.GameState.prototype.isEntityExclusiveAlly = function(ent)
+{
+	if (!ent || !ent.owner)
+		return false;
+	if ((typeof ent.owner) === "function")
+		return (this.playerData.isAlly[ent.owner()] && ent.owner() !== this.player);
+	else
+		return (this.playerData.isAlly[ent.owner] && ent.owner !== this.player);
+};
+
+m.GameState.prototype.isEntityEnemy = function(ent)
+{
+	if (!ent || !ent.owner)
+		return false;
+	if ((typeof ent.owner) === "function")
 		return this.playerData.isEnemy[ent.owner()];
-	} else if (ent && ent.owner){
+	else
 		return this.playerData.isEnemy[ent.owner];
-	}
-	return false;
 };
  
-m.GameState.prototype.isEntityOwn = function(ent) {
-	if (ent && ent.owner && (typeof ent.owner) === "function"){
-		return ent.owner() == this.player;
-	} else if (ent && ent.owner){
-		return ent.owner == this.player;
-	}
-	return false;
+m.GameState.prototype.isEntityOwn = function(ent)
+{
+	if (!ent || !ent.owner)
+		return false;
+	if ((typeof ent.owner) === "function")
+		return ent.owner() === this.player;
+	else
+		return ent.owner === this.player;
 };
 
-m.GameState.prototype.getEntityById = function(id){
-	if (this.entities._entities[id])
-		return this.entities._entities[id];
+m.GameState.prototype.getEntityById = function(id)
+{
+	if (this.entities._entities.has(+id))
+		return this.entities._entities.get(+id);
 
 	return undefined;
 };
@@ -398,6 +396,10 @@ m.GameState.prototype.getExclusiveAllyEntities = function() {
 	return this.entities.filter(m.Filters.byOwners(this.getExclusiveAllies()));
 };
 
+m.GameState.prototype.getAllyStructures = function() {
+	return this.updatingCollection("ally-structures", m.Filters.byClass("Structure"), this.getAllyEntities());
+};
+
 // Try to use a parameter for those three, it'll be a lot faster.
 
 m.GameState.prototype.getEnemyEntities = function(enemyID) {
@@ -409,7 +411,7 @@ m.GameState.prototype.getEnemyEntities = function(enemyID) {
 
 m.GameState.prototype.getEnemyStructures = function(enemyID) {
 	if (enemyID === undefined)
-		return this.getEnemyEntities().filter(m.Filters.byClass("Structure"));
+		return this.updatingCollection("enemy-structures", m.Filters.byClass("Structure"), this.getEnemyEntities());
 
 	return this.updatingGlobalCollection("" + enemyID + "-structures", m.Filters.byClass("Structure"), this.getEnemyEntities(enemyID));
 };
@@ -441,11 +443,11 @@ m.GameState.prototype.getOwnEntitiesByType = function(type, maintain){
 };
 
 m.GameState.prototype.getOwnTrainingFacilities = function(){
-	return this.updatingGlobalCollection("" + this.player + "-training-facilities", m.Filters.byTrainingQueue(), this.getOwnEntities(), true);
+	return this.updatingGlobalCollection("" + this.player + "-training-facilities", m.Filters.byTrainingQueue(), this.getOwnEntities());
 };
 
 m.GameState.prototype.getOwnResearchFacilities = function(){
-	return this.updatingGlobalCollection("" + this.player + "-research-facilities", m.Filters.byResearchAvailable(), this.getOwnEntities(), true);
+	return this.updatingGlobalCollection("" + this.player + "-research-facilities", m.Filters.byResearchAvailable(), this.getOwnEntities());
 };
 
 
@@ -530,20 +532,20 @@ m.GameState.prototype.getOwnFoundations = function() {
 
 m.GameState.prototype.getOwnDropsites = function(resource){
 	if (resource !== undefined)
-		return this.updatingCollection("dropsite-" + resource, m.Filters.isDropsite(resource), this.getOwnEntities(), true);
-	return this.updatingCollection("dropsite-all", m.Filters.isDropsite(), this.getOwnEntities(), true);
+		return this.updatingCollection("dropsite-" + resource, m.Filters.isDropsite(resource), this.getOwnEntities());
+	return this.updatingCollection("dropsite-all", m.Filters.isDropsite(), this.getOwnEntities());
 };
 
 m.GameState.prototype.getResourceSupplies = function(resource){
-	return this.updatingGlobalCollection("resource-" + resource, m.Filters.byResource(resource), this.getEntities(), true);
+	return this.updatingGlobalCollection("resource-" + resource, m.Filters.byResource(resource), this.getEntities());
 };
 
 m.GameState.prototype.getHuntableSupplies = function(){
-	return this.updatingGlobalCollection("resource-hunt", m.Filters.isHuntable(), this.getEntities(), true);
+	return this.updatingGlobalCollection("resource-hunt", m.Filters.isHuntable(), this.getEntities());
 };
 
 m.GameState.prototype.getFishableSupplies = function(){
-	return this.updatingGlobalCollection("resource-fish", m.Filters.isFishable(), this.getEntities(), true);
+	return this.updatingGlobalCollection("resource-fish", m.Filters.isFishable(), this.getEntities());
 };
 
 // This returns only units from buildings.
@@ -561,15 +563,17 @@ m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
 	var ret = [];
 	var limits = this.getEntityLimits();
 	var current = this.getEntityCounts();
-	for (var i in allTrainable)
+	for (let trainable of allTrainable)
 	{
-		var template = this.getTemplate(allTrainable[i]);
+		let template = this.getTemplate(trainable);
 
 		if (!template.available(this))
 			continue;
+		if (this.isDisabledTemplates(trainable))
+			continue;
 		
-		var okay = true;
-		for (var clas of classes)
+		let okay = true;
+		for (let clas of classes)
 		{
 			if (template.hasClass(clas))
 				continue;
@@ -579,7 +583,7 @@ m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
 		if (!okay)
 			continue;
 
-		for (var clas of anticlasses)
+		for (let clas of anticlasses)
 		{
 			if (!template.hasClass(clas))
 				continue;
@@ -589,17 +593,11 @@ m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
 		if (!okay)
 			continue;
 
-		for (var limitedClass in limits)
-		{
-			if (!template.hasClass(limitedClass) || current[limitedClass] < limits[limitedClass])
-				continue;
-			okay = false;
-			break;
-		}
-		if (!okay)
+		let category = template.trainingCategory();
+		if (category && limits[category] && current[category] >= limits[category])
 			continue;
 
-		ret.push( [allTrainable[i], template] );
+		ret.push( [trainable, template] );
 	}
 	return ret;
 };
@@ -607,33 +605,33 @@ m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
 // Return all techs which can currently be researched
 // Does not factor cost.
 // If there are pairs, both techs are returned.
-m.GameState.prototype.findAvailableTech = function() {
-	
-	var allResearchable = [];
+m.GameState.prototype.findAvailableTech = function()
+{	
+	let allResearchable = [];
 	this.getOwnEntities().forEach(function(ent) {
-		var searchable = ent.researchableTechs();
-		for (var i in searchable) {
-			if (allResearchable.indexOf(searchable[i]) === -1) {
-				allResearchable.push(searchable[i]);
-			}
-		}
+		let searchable = ent.researchableTechs();
+		if (!searchable)
+			return;
+		for (let tech of searchable)
+			if (allResearchable.indexOf(tech) === -1)
+				allResearchable.push(tech);
 	});
 	
-	var ret = [];
-	for (var i in allResearchable) {
-		var template = this.getTemplate(allResearchable[i]);
-
+	let ret = [];
+	for (let tech of allResearchable)
+	{
+		let template = this.getTemplate(tech);
 		if (template.pairDef())
 		{
-			var techs = template.getPairedTechs();
+			let techs = template.getPairedTechs();
 			if (this.canResearch(techs[0]._templateName))
 				ret.push([techs[0]._templateName, techs[0]] );
 			if (this.canResearch(techs[1]._templateName))
 				ret.push([techs[1]._templateName, techs[1]] );
-		} else {
-			if (this.canResearch(allResearchable[i]) && template._templateName != this.townPhase() && template._templateName != this.cityPhase())
-				ret.push( [allResearchable[i], template] );
 		}
+		else
+			if (this.canResearch(tech) && template._templateName != this.townPhase() && template._templateName != this.cityPhase())
+				ret.push( [tech, template] );
 	}
 	return ret;
 };
@@ -642,9 +640,10 @@ m.GameState.prototype.findAvailableTech = function() {
  * Find buildings that are capable of training said template.
  * Getting the best is up to the AI.
  */
-m.GameState.prototype.findTrainers = function(template) {	
+m.GameState.prototype.findTrainers = function(template)
+{	
 	return this.getOwnTrainingFacilities().filter(function(ent) {
-		var trainable = ent.trainableEntities();
+		let trainable = ent.trainableEntities();
 		if (!trainable || trainable.indexOf(template) == -1)
 			return false;
 		return true;
@@ -654,18 +653,19 @@ m.GameState.prototype.findTrainers = function(template) {
 /**
  * Find units that are capable of constructing the given building type.
  */
-m.GameState.prototype.findBuilders = function(template) {
+m.GameState.prototype.findBuilders = function(template)
+{
 	return this.getOwnUnits().filter(function(ent) {
-		var buildable = ent.buildableEntities();
+		let buildable = ent.buildableEntities();
 		if (!buildable || buildable.indexOf(template) == -1)
 			return false;
-
 		return true;
 	});
 };
 
 // Find buildings that are capable of researching the given tech
-m.GameState.prototype.findResearchers = function(templateName, noRequirementCheck) {
+m.GameState.prototype.findResearchers = function(templateName, noRequirementCheck)
+{
 	// let's check we can research the tech.
 	if (!this.canResearch(templateName, noRequirementCheck))
 		return [];
@@ -675,52 +675,48 @@ m.GameState.prototype.findResearchers = function(templateName, noRequirementChec
 	
 	return this.getOwnResearchFacilities().filter(function(ent) {
 		var techs = ent.researchableTechs();
-		for (var i in techs)
+		for (let tech of techs)
 		{
-			var thisTemp = self.getTemplate(techs[i]);
+			let thisTemp = self.getTemplate(tech);
 			if (thisTemp.pairDef())
 			{
 				var pairedTechs = thisTemp.getPairedTechs();
 				if (pairedTechs[0]._templateName == templateName || pairedTechs[1]._templateName == templateName)
 					return true;
-			} else {
-				if (techs[i] == templateName)
-					return true;
 			}
+			else
+				if (tech == templateName)
+					return true;
 		}
 		return false;
 	});
 };
 
-m.GameState.prototype.getEntityLimits = function() {
+m.GameState.prototype.getEntityLimits = function()
+{
 	return this.playerData.entityLimits;
 };
 
-m.GameState.prototype.getEntityCounts = function() {
+m.GameState.prototype.getEntityCounts = function()
+{
 	return this.playerData.entityCounts;
 };
 
+m.GameState.prototype.isDisabledTemplates = function(template)
+{
+	if (!this.playerData.disabledTemplates[template])
+		return false;
+	return this.playerData.disabledTemplates[template];
+};
+
 // Checks whether the maximum number of buildings have been cnstructed for a certain catergory
-m.GameState.prototype.isEntityLimitReached = function(category) {
+m.GameState.prototype.isEntityLimitReached = function(category)
+{
 	if(this.playerData.entityLimits[category] === undefined || this.playerData.entityCounts[category] === undefined)
 		return false;
 	return (this.playerData.entityCounts[category] >= this.playerData.entityLimits[category]);
 };
 
-// defcon utilities
-m.GameState.prototype.timeSinceDefconChange = function() {
-	return this.getTimeElapsed()-this.ai.defconChangeTime;
-};
-m.GameState.prototype.setDefcon = function(level,force) {
-	if (this.ai.defcon >= level || force) {
-		this.ai.defcon = level;
-		this.ai.defconChangeTime = this.getTimeElapsed();
-	}
-};
-m.GameState.prototype.defcon = function() {
-	return this.ai.defcon;
-};
-
 return m;
 
 }(API3);
