@@ -20,12 +20,17 @@ m.SharedScript = function(settings)
 	this._techModifications = { 0 : {}, 1 : {}, 2 : {}, 3 : {}, 4 : {}, 5 : {}, 6 : {}, 7 : {}, 8 : {} };
 	
 	// array of entity collections
-	this._entityCollections = [];
+	this._entityCollections = new Map();
 	// each name is a reference to the actual one.
-	this._entityCollectionsName = {};
+	this._entityCollectionsName = new Map();
 	this._entityCollectionsByDynProp = {};
 	this._entityCollectionsUID = 0;
 	
+	// size of the map  
+	// TODO should be set in settings, for the time being recompute it assuming the passability cell size   
+	this.sizeMap = undefined;
+	this.passabilityCell = 4;
+
 	// A few notes about these maps. They're updated by checking for "create" and "destroy" events for all resources
 	// TODO: change the map when the resource amounts change for at least stone and metal mines.
 	this.resourceMaps = {}; // Contains maps showing the density of wood, stone and metal
@@ -33,8 +38,6 @@ m.SharedScript = function(settings)
 	// Resource maps data.
 	// By how much to divide the resource amount for plotting (ie a tree having 200 wood is "4").
 	this.decreaseFactor = {'wood': 50.0, 'stone': 90.0, 'metal': 90.0, 'food': 40.0};
-
-	this.turn = 0;
 }
 
 //Return a simple object (using no classes etc) that will be serialized
@@ -42,7 +45,19 @@ m.SharedScript = function(settings)
 //TODO: that
 m.SharedScript.prototype.Serialize = function()
 {
-	return { "players" : this._players, "templates" : this._templates, "techTp" : this._techTemplates };
+	// serializing entities without using the class.
+	var entities = []; 
+	for (let ent of this._entities.values()) 
+		entities.push( ent._entity );
+
+	return {
+		"players" : this._players,
+		"templates" : this._templates,
+		"techTp" : this._techTemplates,
+		"techModifications": this._techModifications,
+		"metadata": this._entityMetadata,
+		"entities": entities
+	};
 };
 
 // Called after the constructor when loading a saved game, with 'data' being
@@ -52,6 +67,14 @@ m.SharedScript.prototype.Deserialize = function(data)
 	this._players = data.players;
 	this._templates = data.templates;
 	this._techTemplates = data.techTp;
+	this._techModifications = data.techModifications;
+	this._entityMetadata = data.metadata;
+	this._derivedTemplates = {};
+
+	this._entities = new Map();
+	for (let ent of data.entities)
+		this._entities.set(ent.id, new m.Entity(this, ent)); 
+
 	this.isDeserialized = true;
 };
 
@@ -118,26 +141,47 @@ m.SharedScript.prototype.GetTemplate = function(name)
 // Initialize the shared component.
 // We need to now the initial state of the game for this, as we will use it.
 // This is called right at the end of the map generation.
-m.SharedScript.prototype.init = function(state) {
+m.SharedScript.prototype.init = function(state, deserialization)
+{
 	this.ApplyTemplatesDelta(state);
 
 	this.passabilityClasses = state.passabilityClasses;
-	this.passabilityMap = state.passabilityMap;
 	this.players = this._players;
 	this.playersData = state.players;
-	this.territoryMap = state.territoryMap;
 	this.timeElapsed = state.timeElapsed;
 	this.circularMap = state.circularMap;
 	this.gameType = state.gameType;
 	this.barterPrices = state.barterPrices;
 
-	this._entities = {};
-	for (var id in state.entities)
+	this.passabilityMap = state.passabilityMap;
+	this.passabilityMap.cellSize = this.passabilityCell;
+	this.sizeMap = this.passabilityMap.width * this.passabilityMap.cellSize;
+	this.territoryMap = state.territoryMap;
+	this.territoryMap.cellSize = this.sizeMap / this.territoryMap.width;
+
+/*
+	var landPassMap = new Uint8Array(this.passabilityMap.data.length);
+	var waterPassMap = new Uint8Array(this.passabilityMap.data.length);
+	var obstructionMap = new Uint8Array(this.passabilityMap.data.length);
+	var obstructionMaskLand = this.passabilityClasses["default"];
+	var obstructionMaskWater = this.passabilityClasses["ship"];
+	var obstructionMask = this.passabilityClasses["pathfinderObstruction"];
+	for (var i = 0; i < this.passabilityMap.data.length; ++i)
 	{
-		// entropy generator
-		for (var p = 0; p < id; ++p)
-			Math.random();
-		this._entities[id] = new m.Entity(this, state.entities[id]);
+		landPassMap[i] = (this.passabilityMap.data[i] & obstructionMaskLand) ? 0 : 255;
+		waterPassMap[i] = (this.passabilityMap.data[i] & obstructionMaskWater) ? 0 : 255;
+		obstructionMap[i] = (this.passabilityMap.data[i] & obstructionMask) ? 0 : 255;
+	}
+	Engine.DumpImage("LandPassMap.png", landPassMap, this.passabilityMap.width, this.passabilityMap.height, 255);
+	Engine.DumpImage("WaterPassMap.png", waterPassMap, this.passabilityMap.width, this.passabilityMap.height, 255);
+	Engine.DumpImage("ObstrctionMap.png", obstructionMap, this.passabilityMap.width, this.passabilityMap.height, 255);
+*/
+
+	if (!deserialization)
+	{
+		this._entities = new Map();
+		for (var id in state.entities)
+			this._entities.set(+id, new m.Entity(this, state.entities[id]));
 	}
 	// entity collection updated on create/destroy event.
 	this.entities = new m.EntityCollection(this, this._entities);
@@ -163,12 +207,12 @@ m.SharedScript.prototype.init = function(state) {
 // applies entity deltas, and each gamestate.
 m.SharedScript.prototype.onUpdate = function(state)
 {
-	if (this.isDeserialized && this.turn !== 0)
+	if (this.isDeserialized)
 	{
+		this.init(state, true);
 		this.isDeserialized = false;
-		this.init(state);
-	} else if (this.isDeserialized)
-		return;
+	}
+
 	// deals with updating based on create and destroy messages.
 	this.ApplyEntitiesDelta(state);
 	this.ApplyTemplatesDelta(state);
@@ -178,11 +222,14 @@ m.SharedScript.prototype.onUpdate = function(state)
 	// those are dynamic and need to be reset as the "state" object moves in memory.
 	this.events = state.events;
 	this.passabilityClasses = state.passabilityClasses;
-	this.passabilityMap = state.passabilityMap;
 	this.playersData = state.players;
-	this.territoryMap = state.territoryMap;
 	this.timeElapsed = state.timeElapsed;
 	this.barterPrices = state.barterPrices;
+
+	this.passabilityMap = state.passabilityMap;
+	this.passabilityMap.cellSize = this.sizeMap / this.passabilityMap.width;
+	this.territoryMap = state.territoryMap;
+	this.territoryMap.cellSize = this.sizeMap / this.territoryMap.width;
 	
 	for (var i in this.gameState)
 		this.gameState[i].update(this,state);
@@ -191,8 +238,6 @@ m.SharedScript.prototype.onUpdate = function(state)
 	this.updateResourceMaps(this, this.events);
 	this.terrainAnalyzer.updateMapWithEvents(this);
 	
-	this.turn++;
-	
 	Engine.ProfileStop();
 };
 
@@ -212,66 +257,56 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 	var MetadataEvents = state.events["AIMetadata"];
 	var ownershipChangeEvents = state.events["OwnershipChanged"];
 
-	for (var i = 0; i < CreateEvents.length; ++i)
+	for (let i = 0; i < CreateEvents.length; ++i)
 	{
-		var evt = CreateEvents[i];
+		let evt = CreateEvents[i];
 		if (!state.entities[evt.entity])
 			continue; // Sometimes there are things like foundations which get destroyed too fast
 
-		this._entities[evt.entity] = new m.Entity(this, state.entities[evt.entity]);
-		this.entities.addEnt(this._entities[evt.entity]);
+		let entity = new m.Entity(this, state.entities[evt.entity]);
+		this._entities.set(evt.entity, entity);
+		this.entities.addEnt(entity);
 		
 		// Update all the entity collections since the create operation affects static properties as well as dynamic
-		for (var entCollection in this._entityCollections)
-			this._entityCollections[entCollection].updateEnt(this._entities[evt.entity]);
+		for (let entCol of this._entityCollections.values())
+			entCol.updateEnt(entity);
 	}
-	for (var i in RenamingEvents)
-	{
-		var evt = RenamingEvents[i];
-		// Switch the metadata
-		for (var p in this._players)
+
+	for (let evt of RenamingEvents)
+	{	// Switch the metadata: TODO entityCollections are updated only because of the owner change. Should be done properly
+		for (let p in this._players)
 		{
 			this._entityMetadata[this._players[p]][evt.newentity] = this._entityMetadata[this._players[p]][evt.entity];
 			this._entityMetadata[this._players[p]][evt.entity] = {};
 		}
 	}
-	for (var i in TrainingEvents)
-	{
-		var evt = TrainingEvents[i];
-		// Apply metadata stored in training queues
-		for each (var ent in evt.entities)
-		{
-			for (var key in evt.metadata)
-			{
-				this.setMetadata(evt.owner, this._entities[ent], key, evt.metadata[key])
-			}
-		}
+
+	for (let evt of TrainingEvents)
+	{	// Apply metadata stored in training queues
+		for each (let entId in evt.entities)
+			for (let key in evt.metadata)
+				this.setMetadata(evt.owner, this._entities.get(entId), key, evt.metadata[key])
 	}
-	for (var i in ConstructionEvents)
-	{
-		var evt = ConstructionEvents[i];
-		// we'll move metadata.
-		if (!this._entities[evt.entity])
+
+	for (let evt of ConstructionEvents)
+	{	// we'll move metadata.
+		if (!this._entities.has(evt.entity))
 			continue;
-		var ent = this._entities[evt.entity];
-		var newEnt = this._entities[evt.newentity];
+		let ent = this._entities.get(evt.entity);
+		let newEnt = this._entities.get(evt.newentity);
 		if (this._entityMetadata[ent.owner()] && this._entityMetadata[ent.owner()][evt.entity] !== undefined)
-			for (var key in this._entityMetadata[ent.owner()][evt.entity])
-			{
+			for (let key in this._entityMetadata[ent.owner()][evt.entity])
 				this.setMetadata(ent.owner(), newEnt, key, this._entityMetadata[ent.owner()][evt.entity][key])
-			}
 		foundationFinished[evt.entity] = true;
 	}
-	for (var i in MetadataEvents)
+
+	for (let evt of MetadataEvents)
 	{
-		var evt = MetadataEvents[i];
-		if (!this._entities[evt.id])
+		if (!this._entities.has(evt.id))
 			continue;	// might happen in some rare cases of foundations getting destroyed, perhaps.
 						// Apply metadata (here for buildings for example)
-		for (var key in evt.metadata)
-		{
-			this.setMetadata(evt.owner, this._entities[evt.id], key, evt.metadata[key])
-		}
+		for (let key in evt.metadata)
+			this.setMetadata(evt.owner, this._entities.get(evt.id), key, evt.metadata[key])
 	}
 	
 	for (var i = 0; i < DestroyEvents.length; ++i)
@@ -281,7 +316,7 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 		// the "deleted" object remains in memory, and any older reference to it will still reference it as if it were not "deleted".
 		// Worse, they might prevent it from being garbage collected, thus making it stay alive and consuming ram needlessly.
 		// So take care, and if you encounter a weird bug with deletion not appearing to work correctly, this is probably why.
-		if (!this._entities[evt.entity])
+		if (!this._entities.has(evt.entity))
 			continue;// probably should remove the event.
 
 		if (foundationFinished[evt.entity])
@@ -290,30 +325,28 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 		// The entity was destroyed but its data may still be useful, so
 		// remember the entity and this AI's metadata concerning it
 		evt.metadata = {};
-		evt.entityObj = this._entities[evt.entity];
+		evt.entityObj = this._entities.get(evt.entity);
 		for (var j in this._players)
 			evt.metadata[this._players[j]] = this._entityMetadata[this._players[j]][evt.entity];
 		
-		for each (var entCol in this._entityCollections)
-		{
-			entCol.removeEnt(this._entities[evt.entity]);
-		}
-		this.entities.removeEnt(this._entities[evt.entity]);
+		let entity = this._entities.get(evt.entity);
+		for (let entCol of this._entityCollections.values())
+			entCol.removeEnt(entity);
+		this.entities.removeEnt(entity);
 		
-		delete this._entities[evt.entity];
+		this._entities.delete(evt.entity);
 		for (var j in this._players)
 			delete this._entityMetadata[this._players[j]][evt.entity];
 	}
 
-	for (var id in state.entities)
+	for (let id in state.entities)
 	{
-		var changes = state.entities[id];
-
-		for (var prop in changes)
+		let changes = state.entities[id];
+		let entity = this._entities.get(+id);
+		for (let prop in changes)
 		{
-			this._entities[id]._entity[prop] = changes[prop];
-			
-			this.updateEntityCollections(prop, this._entities[id]);
+			entity._entity[prop] = changes[prop];
+			this.updateEntityCollections(prop, entity);
 		}
 	}
 
@@ -321,11 +354,12 @@ m.SharedScript.prototype.ApplyEntitiesDelta = function(state)
 	// this supersedes tech-related changes.
 	for (var id in state.changedEntityTemplateInfo)
 	{
-		if (!this._entities[id])
+		if (!this._entities.has(+id))
 			continue;	// dead, presumably.
 		var changes = state.changedEntityTemplateInfo[id];
+		let entity = this._entities.get(+id);
 		for each (var change in changes)
-			this._entities[id]._auraTemplateModif.set(change.variable, change.value);
+			entity._auraTemplateModif.set(change.variable, change.value);
 	}
 	Engine.ProfileStop();
 };
@@ -349,51 +383,38 @@ m.SharedScript.prototype.ApplyTemplatesDelta = function(state)
 	Engine.ProfileStop();
 };
 
-m.SharedScript.prototype.registerUpdatingEntityCollection = function(entCollection, noPush)
+m.SharedScript.prototype.registerUpdatingEntityCollection = function(entCollection)
 {
-	if (!noPush) {
-		this._entityCollections.push(entCollection);
-	}
 	entCollection.setUID(this._entityCollectionsUID);
+	this._entityCollections.set(this._entityCollectionsUID, entCollection);
 	for each (var prop in entCollection.dynamicProperties())
 	{
-		this._entityCollectionsByDynProp[prop] = this._entityCollectionsByDynProp[prop] || [];
-		this._entityCollectionsByDynProp[prop].push(entCollection);
+		if (!this._entityCollectionsByDynProp[prop])
+			this._entityCollectionsByDynProp[prop] = new Map();
+		this._entityCollectionsByDynProp[prop].set(this._entityCollectionsUID, entCollection);
 	}
 	this._entityCollectionsUID++;
 };
 
 m.SharedScript.prototype.removeUpdatingEntityCollection = function(entCollection)
 {
-	for (var i in this._entityCollections)
-	{
-		if (this._entityCollections[i].getUID() === entCollection.getUID())
-		{
-			this._entityCollections.splice(i, 1);
-		}
-	}
-	
-	for each (var prop in entCollection.dynamicProperties())
-	{
-		for (var i in this._entityCollectionsByDynProp[prop])
-		{
-			if (this._entityCollectionsByDynProp[prop][i].getUID() === entCollection.getUID())
-			{
-				this._entityCollectionsByDynProp[prop].splice(i, 1);
-			}
-		}
-	}
+	let uid = entCollection.getUID();
+
+	if (this._entityCollections.has(uid))
+		this._entityCollections.delete(uid);
+
+	for each (let prop in entCollection.dynamicProperties())
+		if (this._entityCollectionsByDynProp[prop].has(uid))
+			this._entityCollectionsByDynProp[prop].delete(uid);
 };
 
 m.SharedScript.prototype.updateEntityCollections = function(property, ent)
 {
-	if (this._entityCollectionsByDynProp[property] !== undefined)
-	{
-		for (var entCollectionid in this._entityCollectionsByDynProp[property])
-		{
-			this._entityCollectionsByDynProp[property][entCollectionid].updateEnt(ent);
-		}
-	}
+	if (this._entityCollectionsByDynProp[property] === undefined)
+		return;
+
+	for (let entCol of this._entityCollectionsByDynProp[property].values())
+		entCol.updateEnt(ent);
 }
 
 m.SharedScript.prototype.setMetadata = function(player, ent, key, value)
