@@ -15,23 +15,23 @@ var API3 = function(m)
  * But truly separating optimizes.
  */
 
-m.TerrainAnalysis = function() {
-	this.cellSize = 4;
-}
+m.TerrainAnalysis = function()
+{
+};
 
 m.copyPrototype(m.TerrainAnalysis, m.Map);
 
-m.TerrainAnalysis.prototype.init = function(sharedScript,rawState) {
-	var self = this;
-
+m.TerrainAnalysis.prototype.init = function(sharedScript,rawState)
+{
 	var passabilityMap = rawState.passabilityMap;
 	this.width = passabilityMap.width;
 	this.height = passabilityMap.height;
+	this.cellSize = passabilityMap.cellSize;
 	
 	// the first two won't change, the third is a reference to a value updated by C++
-	this.obstructionMaskLand = rawState.passabilityClasses["default"];
-	this.obstructionMaskWater = rawState.passabilityClasses["ship"];
-	this.obstructionMask = rawState.passabilityClasses["pathfinderObstruction"];
+	var obstructionMaskLand = rawState.passabilityClasses["default"];
+	var obstructionMaskWater = rawState.passabilityClasses["ship"];
+	var obstructionMask = rawState.passabilityClasses["pathfinderObstruction"];
 
 	var obstructionTiles = new Uint8Array(passabilityMap.data.length);
 	
@@ -48,11 +48,11 @@ m.TerrainAnalysis.prototype.init = function(sharedScript,rawState) {
 	for (var i = 0; i < passabilityMap.data.length; ++i)
 	{
 		// If impassable for land units, set to 0, else to 255.
-		obstructionTiles[i] = (passabilityMap.data[i] & this.obstructionMaskLand) ? 0 : 255;
+		obstructionTiles[i] = (passabilityMap.data[i] & obstructionMaskLand) ? 0 : 255;
 		
-		if (!(passabilityMap.data[i] & this.obstructionMaskWater) && obstructionTiles[i] === 0)
+		if (!(passabilityMap.data[i] & obstructionMaskWater) && obstructionTiles[i] === 0)
 			obstructionTiles[i] = 200; // if navigable and not walkable (ie basic water), set to 200.
-		else if (!(passabilityMap.data[i] & this.obstructionMaskWater) && obstructionTiles[i] === 255)
+		else if (!(passabilityMap.data[i] & obstructionMaskWater) && obstructionTiles[i] === 255)
 			obstructionTiles[i] = 201; // navigable and walkable.
 	}
 
@@ -64,21 +64,21 @@ m.TerrainAnalysis.prototype.init = function(sharedScript,rawState) {
 	var x = 0;
 	var y = 0;
 	var radius = 0;
-	for (var entI in sharedScript._entities)
+	for (let ent of sharedScript._entities.values())
 	{
-		var ent = sharedScript._entities[entI];
-		if (ent.hasClass("ForestPlant") === true) {
+		if (ent.hasClass("ForestPlant") === true)
+		{
 			pos = this.gamePosToMapPos(ent.position());
 			x = pos[0];
 			y = pos[1];
 			// unless it's impassable already, mark it as 40.
 			if (obstructionTiles[x + y*this.width] !== 0)
 				obstructionTiles[x + y*this.width] = 40;
-			for (var i in square)
+			for (let sq of square)
 			{
-				xx = square[i][0];
-				yy = square[i][1];
-				if (x+i[0] >= 0 && x+xx < this.width && y+yy >= 0 && y+yy < this.height) {
+				xx = sq[0];
+				yy = sq[1];
+				if (x+xx >= 0 && x+xx < this.width && y+yy >= 0 && y+yy < this.height) {
 					value = obstructionTiles[(x+xx) + (y+yy)*this.width];
 					if (value === 255)
 						obstructionTiles[(x+xx) + (y+yy)*this.width] = 41;
@@ -86,7 +86,9 @@ m.TerrainAnalysis.prototype.init = function(sharedScript,rawState) {
 						obstructionTiles[(x+xx) + (y+yy)*this.width] = value + 1;
 				}
 			}
-		} else if (ent.hasClass("Geology") === true) {
+		}
+		else if (ent.hasClass("Geology") === true)
+		{
 			radius = Math.floor(ent.obstructionRadius() / 4);
 			pos = this.gamePosToMapPos(ent.position());
 			x = pos[0];
@@ -100,18 +102,12 @@ m.TerrainAnalysis.prototype.init = function(sharedScript,rawState) {
 		}
 	}
 	// Okay now we have a pretty good knowledge of the map.
-	this.Map(rawState, obstructionTiles);
-
-	this.obstructionMaskLand = null;
-	this.obstructionMaskWater = null;
-	this.obstructionMask = null;
-	delete this.obstructionMaskLand;
-	delete this.obstructionMaskWater;
-	delete this.obstructionMask;
+	this.Map(rawState, "passability", obstructionTiles);
 };
 
 // Returns the (approximately) closest point which is passable by searching in a spiral pattern 
-m.TerrainAnalysis.prototype.findClosestPassablePoint = function(startPoint, onLand, limitDistance, quickscope){
+m.TerrainAnalysis.prototype.findClosestPassablePoint = function(startPoint, onLand, limitDistance, quickscope)
+{
 	var w = this.width;
 	var p = startPoint;
 	var direction = 1;
@@ -120,28 +116,26 @@ m.TerrainAnalysis.prototype.findClosestPassablePoint = function(startPoint, onLa
 		return undefined;
 	}
 	// quickscope
-	if (this.map[p[0] + w*p[1]] === 255) {
-		if (this.countConnected(p[0] + w*p[1], onLand) >= 2) {
+	if (this.map[p[0] + w*p[1]] === 255)
+		if (this.countConnected(p[0] + w*p[1], onLand) >= 2)
 			return p;
-		}
-	}
 	
 	var count = 0;
 	// search in a spiral pattern. We require a value that is actually accessible in this case, ie 255, 201 or 41 if land, 200/201 if water.
-	for (var i = 1; i < w; i++){
-		for (var j = 0; j < 2; j++){
-			for (var k = 0; k < i; k++){
+	for (var i = 1; i < w; i++)
+	{
+		for (var j = 0; j < 2; j++)
+		{
+			for (var k = 0; k < i; k++)
+			{
 				p[j] += direction;
 				// if the value is not markedly inaccessible
 				var index = p[0] + w*p[1];
-				if (this.map[index] !== 0 && this.map[index] !== 90 && this.map[index] !== 120 && this.map[index] !== 30 && this.map[index] !== 40){
-					if (quickscope || this.countConnected(index, onLand) >= 2){
+				if (this.map[index] !== 0 && this.map[index] !== 90 && this.map[index] !== 120 && this.map[index] !== 30 && this.map[index] !== 40)
+					if (quickscope || this.countConnected(index, onLand) >= 2)
 						return p;
-					}
-				}
-				if (limitDistance !== undefined && count > limitDistance){
+				if (limitDistance !== undefined && count > limitDistance)
 					return undefined;
-				}
 				count++;
 			}
 		}
@@ -153,87 +147,91 @@ m.TerrainAnalysis.prototype.findClosestPassablePoint = function(startPoint, onLa
 
 // Returns an estimate of a tile accessibility. It checks neighboring cells over two levels.
 // returns a count. It's not integer. About 2 should be fairly accessible already.
-m.TerrainAnalysis.prototype.countConnected = function(startIndex, byLand){
-	var count = 0.0;
-	
-	var w = this.width;
-	var positions = [[0,1], [0,-1], [1,0], [-1,0], [1,1], [-1,-1], [1,-1], [-1,1],
+m.TerrainAnalysis.prototype.countConnected = function(startIndex, byLand)
+{
+	let count = 0.0;
+	let w = this.width;
+	let positions = [[0,1], [0,-1], [1,0], [-1,0], [1,1], [-1,-1], [1,-1], [-1,1],
 					 [0,2], [0,-2], [2,0], [-2,0], [2,2], [-2,-2], [2,-2], [-2,2]/*,
 					 [1,2], [1,-2], [2,1], [-2,1], [-1,2], [-1,-2], [2,-1], [-2,-1]*/];
 	
-	for (var i in positions) {
-		var index = startIndex + positions[i][0] + positions[i][1]*w;
-		if (this.map[index] !== 0) {
-			if (byLand) {
-				if (this.map[index] === 201) count++;
-				else if (this.map[index] === 255) count++;
-				else if (this.map[index] === 41) count++;
-				else if (this.map[index] === 42) count += 0.5;
-				else if (this.map[index] === 43) count += 0.3;
-				else if (this.map[index] === 44) count += 0.13;
-				else if (this.map[index] === 45) count += 0.08;
-				else if (this.map[index] === 46) count += 0.05;
-				else if (this.map[index] === 47) count += 0.03;
-			} else {
-				if (this.map[index] === 201) count++;
-				if (this.map[index] === 200) count++;
-			}
+	for (let pos of positions)
+	{
+		var index = startIndex + pos[0] + pos[1]*w;
+		if (this.map[index] === 0)
+			continue;
+		if (byLand)
+		{
+			if (this.map[index] === 201) count++;
+			else if (this.map[index] === 255) count++;
+			else if (this.map[index] === 41) count++;
+			else if (this.map[index] === 42) count += 0.5;
+			else if (this.map[index] === 43) count += 0.3;
+			else if (this.map[index] === 44) count += 0.13;
+			else if (this.map[index] === 45) count += 0.08;
+			else if (this.map[index] === 46) count += 0.05;
+			else if (this.map[index] === 47) count += 0.03;
+		}
+		else
+		{
+			if (this.map[index] === 201) count++;
+			if (this.map[index] === 200) count++;
 		}
 	}
 	return count;
 };
 
 // TODO: for now this resets to 255.
-m.TerrainAnalysis.prototype.updateMapWithEvents = function(sharedAI) {
-	var self = this;
-	
+m.TerrainAnalysis.prototype.updateMapWithEvents = function(sharedAI)
+{
 	var events = sharedAI.events["Destroy"];
 	var passabilityMap = sharedAI.passabilityMap;
 	
 	// looking for creation or destruction of entities, and updates the map accordingly.
-	for (var i in events) {
-		var e = events[i];
-		if (e.entityObj){
-			var ent = e.entityObj;
-			if (ent.hasClass("Geology")) {
-				var x = self.gamePosToMapPos(ent.position())[0];
-				var y = self.gamePosToMapPos(ent.position())[1];
-				// remove it. Don't really care about surrounding and possible overlappings.
-				var radius = Math.floor(ent.obstructionRadius() / self.cellSize);
-				for (var xx = -radius; xx <= radius;xx++)
-					for (var yy = -radius; yy <= radius;yy++)
-					{
-						if (x+xx >= 0 && x+xx < self.width && y+yy >= 0 && y+yy < self.height && this.map[(x+xx) + (y+yy)*self.width] === 30)
-						{
-							this.map[(x+xx) + (y+yy)*self.width] = 255;
-						}
-					}
-			} else if (ent.hasClass("ForestPlant")){
-				var x = self.gamePosToMapPos(ent.position())[0];
-				var y = self.gamePosToMapPos(ent.position())[1];
-				var nbOfNeigh = 0;
-				for (var xx = -1; xx <= 1;xx++)
-					for (var yy = -1; yy <= 1;yy++)
-					{
-						if (xx == 0 && yy == 0)
-							continue;
-						if (this.map[(x+xx) + (y+yy)*self.width] === 40)
-							nbOfNeigh++;
-						else if (this.map[(x+xx) + (y+yy)*self.width] === 41)
-						{
-							this.map[(x+xx) + (y+yy)*self.width] = 255;
-						}
-						else if (this.map[(x+xx) + (y+yy)*self.width] > 41 && this.map[(x+xx) + (y+yy)*self.width] < 50)
-							this.map[(x+xx) + (y+yy)*self.width] = this.map[(x+xx) + (y+yy)*self.width] - 1;
-					}
-				if (nbOfNeigh > 0)
-					this.map[x + y*self.width] = this.map[x + y*self.width] = 40 + nbOfNeigh;
-				else
-					this.map[x + y*self.width] = this.map[x + y*self.width] = 255;
-			}
+	for (let e of events)
+	{
+		if (!e.entityObj)
+			continue;
+		let ent = e.entityObj;
+		if (ent.hasClass("Geology"))
+		{
+			let pos = this.gamePosToMapPos(ent.position());
+			let x = pos[0];
+			let y = pos[1];
+			// remove it. Don't really care about surrounding and possible overlappings.
+			let radius = Math.floor(ent.obstructionRadius() / this.cellSize);
+			for (let xx = -radius; xx <= radius;xx++)
+				for (let yy = -radius; yy <= radius;yy++)
+				{
+					if (x+xx >= 0 && x+xx < this.width && y+yy >= 0 && y+yy < this.height && this.map[(x+xx) + (y+yy)*this.width] === 30)
+						this.map[(x+xx) + (y+yy)*this.width] = 255;
+				}
+		}
+		else if (ent.hasClass("ForestPlant"))
+		{
+			let pos = this.gamePosToMapPos(ent.position());
+			let x = pos[0];
+			let y = pos[1];
+			let nbOfNeigh = 0;
+			for (let xx = -1; xx <= 1;xx++)
+				for (let yy = -1; yy <= 1;yy++)
+				{
+					if (xx == 0 && yy == 0)
+						continue;
+					if (this.map[(x+xx) + (y+yy)*this.width] === 40)
+						nbOfNeigh++;
+					else if (this.map[(x+xx) + (y+yy)*this.width] === 41)
+						this.map[(x+xx) + (y+yy)*this.width] = 255;
+					else if (this.map[(x+xx) + (y+yy)*this.width] > 41 && this.map[(x+xx) + (y+yy)*this.width] < 50)
+						this.map[(x+xx) + (y+yy)*this.width] = this.map[(x+xx) + (y+yy)*this.width] - 1;
+				}
+			if (nbOfNeigh > 0)
+				this.map[x + y*this.width] = this.map[x + y*this.width] = 40 + nbOfNeigh;
+			else
+				this.map[x + y*this.width] = this.map[x + y*this.width] = 255;
 		}
 	}
-}
+};
 
 /*
  * Accessibility inherits from TerrainAnalysis
@@ -242,18 +240,16 @@ m.TerrainAnalysis.prototype.updateMapWithEvents = function(sharedAI) {
  * it can also determine if any point is "probably" reachable, assuming the unit can get close enough
  * for optimizations it's called after the TerrainAnalyser has finished initializing his map
  * so this can use the land regions already.
-
  */
-m.Accessibility = function() {
-	
-}
+m.Accessibility = function()
+{	
+};
 
 m.copyPrototype(m.Accessibility, m.TerrainAnalysis);
 
-m.Accessibility.prototype.init = function(rawState, terrainAnalyser){
-	var self = this;
-	
-	this.Map(rawState, terrainAnalyser.map);
+m.Accessibility.prototype.init = function(rawState, terrainAnalyser)
+{
+	this.Map(rawState, "passability", terrainAnalyser.map);
 	this.landPassMap = new Uint8Array(terrainAnalyser.length);
 	this.navalPassMap = new Uint8Array(terrainAnalyser.length);
 
@@ -267,13 +263,17 @@ m.Accessibility.prototype.init = function(rawState, terrainAnalyser){
 	// So start at 2.
 	this.regionID = 2;
 	
-	for (var i = 0; i < this.landPassMap.length; ++i) {
-		if (this.map[i] !== 0) {	// any non-painted, non-inacessible area.
+	for (var i = 0; i < this.landPassMap.length; ++i)
+	{
+		if (this.map[i] !== 0)
+		{	// any non-painted, non-inacessible area.
 			if (this.landPassMap[i] === 0 && this.floodFill(i,this.regionID,false))
 				this.regionType[this.regionID++] = "land";
 			if (this.navalPassMap[i] === 0 && this.floodFill(i,this.regionID,true))
 				this.regionType[this.regionID++] = "water";
-		} else if (this.landPassMap[i] === 0) {	// any non-painted, inacessible area.
+		}
+		else if (this.landPassMap[i] === 0)
+		{	// any non-painted, inacessible area.
 			this.floodFill(i,1,false);
 			this.floodFill(i,1,true);
 		}
@@ -323,7 +323,8 @@ m.Accessibility.prototype.init = function(rawState, terrainAnalyser){
 	//Engine.DumpImage("NavalPassMap.png", this.navalPassMap, this.width, this.height, 255);
 };
 
-m.Accessibility.prototype.getAccessValue = function(position, onWater) {
+m.Accessibility.prototype.getAccessValue = function(position, onWater)
+{
 	var gamePos = this.gamePosToMapPos(position);
 	if (onWater === true)
 		return this.navalPassMap[gamePos[0] + this.width*gamePos[1]];
@@ -332,10 +333,10 @@ m.Accessibility.prototype.getAccessValue = function(position, onWater) {
 	{
 		// quick spiral search.
 		var indx = [ [-1,-1],[-1,0],[-1,1],[0,1],[1,1],[1,0],[1,-1],[0,-1]]
-		for (var i in indx)
+		for (let i of indx)
 		{
-			var id0 = gamePos[0] + indx[i][0];
-			var id1 = gamePos[1] + indx[i][1];
+			let id0 = gamePos[0] + i[0];
+			let id1 = gamePos[1] + i[1];
 			if (id0 < 0 || id0 >= this.width || id1 < 0 || id1 >= this.width)
 				continue;
 			ret = this.landPassMap[id0 + this.width*id1];
@@ -432,12 +433,9 @@ m.Accessibility.prototype.getTrajectToIndex = function(istart, iend, noBound){
 		return [startRegion, endRegion];
 	else
 	{
-		var rgs = this.regionLinks[startRegion];
-		for (var p in rgs)
-		{
-			if (this.regionLinks[rgs[p]].indexOf(endRegion) !== -1)
-				return [startRegion, rgs[p], endRegion];
-		}
+		for (let rgs of this.regionLinks[startRegion])
+			if (this.regionLinks[rgs].indexOf(endRegion) !== -1)
+				return [startRegion, rgs, endRegion];
 	}
 	// it appears to be difficult.
 	// computing A* over a graph with all nodes equally good (might want to change this sometimes), currently it returns the shortest path switch-wise.
@@ -486,9 +484,8 @@ m.Accessibility.prototype.getTrajectToIndex = function(istart, iend, noBound){
 				break;
 			}
 		}
-		for (var i in this.regionLinks[currentRegion])
+		for (let region of this.regionLinks[currentRegion])
 		{
-			var region = this.regionLinks[currentRegion][i];
 			if(this.isOpened[region] === undefined)
 			{
 				this.parentSquare[region] = currentRegion;
@@ -500,7 +497,9 @@ m.Accessibility.prototype.getTrajectToIndex = function(istart, iend, noBound){
 					found = true;
 					break;
 				}
-			} else {
+			}
+			else
+			{
 				if (this.gCostArray[region] > 1 + this.gCostArray[currentRegion])
 				{
 					this.parentSquare[region] = currentRegion;
@@ -689,24 +688,24 @@ m.Accessibility.prototype.floodFill = function(startIndex, value, onWater)
 // creates a map of resource density
 m.SharedScript.prototype.createResourceMaps = function(sharedScript) {
 	
-	for (var resource in this.decreaseFactor){
+	for (var resource in this.decreaseFactor)
+	{
 		// if there is no resourceMap create one with an influence for everything with that resource
-		if (! this.resourceMaps[resource]){
+		if (!this.resourceMaps[resource])
+		{
 			// We're creting them 8-bit. Things could go above 255 if there are really tons of resources
 			// But at that point the precision is not really important anyway. And it saves memory.
-			this.resourceMaps[resource] = new m.Map(sharedScript, new Uint8Array(sharedScript.passabilityMap.data.length));
-			this.resourceMaps[resource].setMaxVal(255);
-			this.CCResourceMaps[resource] = new m.Map(sharedScript, new Uint8Array(sharedScript.passabilityMap.data.length));
-			this.CCResourceMaps[resource].setMaxVal(255);
+			this.resourceMaps[resource] = new m.Map(sharedScript, "resource");
+			this.CCResourceMaps[resource] = new m.Map(sharedScript, "resource");
 		}
 	}
-	for (var entI in sharedScript._entities)
+	var cellSize = this.resourceMaps["food"].cellSize;
+	for (let ent of sharedScript._entities.values())
 	{
-		var ent = sharedScript._entities[entI];
 		if (ent && ent.position() && ent.resourceSupplyType() && ent.resourceSupplyType().generic !== "treasure") {
 			var resource = ent.resourceSupplyType().generic;
-			var x = Math.floor(ent.position()[0] / 4);
-			var z = Math.floor(ent.position()[1] / 4);
+			var x = Math.floor(ent.position()[0] / cellSize);
+			var z = Math.floor(ent.position()[1] / cellSize);
 			var strength = Math.floor(ent.resourceSupplyMax()/this.decreaseFactor[resource]);
 			if (resource === "wood" || resource === "food")
 			{
@@ -730,74 +729,74 @@ m.SharedScript.prototype.createResourceMaps = function(sharedScript) {
 // creates and maintains a map of unused resource density
 // this also takes dropsites into account.
 // resources that are "part" of a dropsite are not counted.
-m.SharedScript.prototype.updateResourceMaps = function(sharedScript, events) {
-	
-	for (var resource in this.decreaseFactor){
+m.SharedScript.prototype.updateResourceMaps = function(sharedScript, events)
+{	
+	for (let resource in this.decreaseFactor)
+	{
 		// if there is no resourceMap create one with an influence for everything with that resource
-		if (! this.resourceMaps[resource]){
+		if (! this.resourceMaps[resource])
+		{
 			// We're creting them 8-bit. Things could go above 255 if there are really tons of resources
 			// But at that point the precision is not really important anyway. And it saves memory.
-			this.resourceMaps[resource] = new m.Map(sharedScript, new Uint8Array(sharedScript.passabilityMap.data.length));
-			this.resourceMaps[resource].setMaxVal(255);
-			this.CCResourceMaps[resource] = new m.Map(sharedScript, new Uint8Array(sharedScript.passabilityMap.data.length));
-			this.CCResourceMaps[resource].setMaxVal(255);
+			this.resourceMaps[resource] = new m.Map(sharedScript, "resource");
+			this.CCResourceMaps[resource] = new m.Map(sharedScript, "resource");
 		}
 	}
-	
+	var cellSize = this.resourceMaps["food"].cellSize;
 	// Look for destroy events and subtract the entities original influence from the resourceMap
 	// TODO: perhaps do something when dropsites appear/disappear.
-	var destEvents = events["Destroy"];
-	var createEvents = events["Create"];
+	let destEvents = events["Destroy"];
+	let createEvents = events["Create"];
 	
-	for (var key in destEvents) {
-		var e = destEvents[key];
-		if (e.entityObj){
-			var ent = e.entityObj;
-			if (ent && ent.position() && ent.resourceSupplyType() && ent.resourceSupplyType().generic !== "treasure") {
-				var resource = ent.resourceSupplyType().generic;
-				var x = Math.floor(ent.position()[0] / 4);
-				var z = Math.floor(ent.position()[1] / 4);
-				var strength = Math.floor(ent.resourceSupplyMax()/this.decreaseFactor[resource]);
-				
-				if (resource === "wood" || resource === "food")
-				{
-					this.resourceMaps[resource].addInfluence(x, z, 2, 5,'constant');
-					this.resourceMaps[resource].addInfluence(x, z, 9.0, -strength,'constant');
-					this.CCResourceMaps[resource].addInfluence(x, z, 15, -strength/2.0,'constant');
-				}
-				else if (resource === "stone" || resource === "metal")
-				{
-					this.resourceMaps[resource].addInfluence(x, z, 8, 50);
-					this.resourceMaps[resource].addInfluence(x, z, 12.0, -strength/1.5);
-					this.resourceMaps[resource].addInfluence(x, z, 12.0, -strength/2.0,'constant');
-					this.CCResourceMaps[resource].addInfluence(x, z, 30, -strength,'constant');
-				}
+	for (let e of destEvents)
+	{
+		if (!e.entityObj)
+			continue;
+		let ent = e.entityObj;
+		if (ent && ent.position() && ent.resourceSupplyType() && ent.resourceSupplyType().generic !== "treasure")
+		{
+			let resource = ent.resourceSupplyType().generic;
+			let x = Math.floor(ent.position()[0] / cellSize);
+			let z = Math.floor(ent.position()[1] / cellSize);
+			let strength = Math.floor(ent.resourceSupplyMax()/this.decreaseFactor[resource]);
+			if (resource === "wood" || resource === "food")
+			{
+				this.resourceMaps[resource].addInfluence(x, z, 2, 5,'constant');
+				this.resourceMaps[resource].addInfluence(x, z, 9.0, -strength,'constant');
+				this.CCResourceMaps[resource].addInfluence(x, z, 15, -strength/2.0,'constant');
+			}
+			else if (resource === "stone" || resource === "metal")
+			{
+				this.resourceMaps[resource].addInfluence(x, z, 8, 50);
+				this.resourceMaps[resource].addInfluence(x, z, 12.0, -strength/1.5);
+				this.resourceMaps[resource].addInfluence(x, z, 12.0, -strength/2.0,'constant');
+				this.CCResourceMaps[resource].addInfluence(x, z, 30, -strength,'constant');
 			}
 		}
 	}
-	for (var key in createEvents) {
-		var e = createEvents[key];
-		if (e.entity){
-			var ent = sharedScript._entities[e.entity];
-			if (ent && ent.position() && ent.resourceSupplyType() && ent.resourceSupplyType().generic !== "treasure"){
-				var resource = ent.resourceSupplyType().generic;
-				
-				var x = Math.floor(ent.position()[0] / 4);
-				var z = Math.floor(ent.position()[1] / 4);
-				var strength = Math.floor(ent.resourceSupplyMax()/this.decreaseFactor[resource]);
-				if (resource === "wood" || resource === "food")
-				{
-					this.CCResourceMaps[resource].addInfluence(x, z, 15, strength/2.0,'constant');
-					this.resourceMaps[resource].addInfluence(x, z, 9.0, strength,'constant');
-					this.resourceMaps[resource].addInfluence(x, z, 2, -5,'constant');
-				}
-				else if (resource === "stone" || resource === "metal")
-				{
-					this.CCResourceMaps[resource].addInfluence(x, z, 30, strength,'constant');
-					this.resourceMaps[resource].addInfluence(x, z, 12.0, strength/1.5);
-					this.resourceMaps[resource].addInfluence(x, z, 12.0, strength/2.0,'constant');
-					this.resourceMaps[resource].addInfluence(x, z, 8, -50);
-				}
+	for (let e of createEvents)
+	{
+		if (!e.entity || !sharedScript._entities.has(e.entity))
+			continue;
+		let ent = sharedScript._entities.get(e.entity);
+		if (ent && ent.position() && ent.resourceSupplyType() && ent.resourceSupplyType().generic !== "treasure")
+		{
+			let resource = ent.resourceSupplyType().generic;
+			let x = Math.floor(ent.position()[0] / cellSize);
+			let z = Math.floor(ent.position()[1] / cellSize);
+			let strength = Math.floor(ent.resourceSupplyMax()/this.decreaseFactor[resource]);
+			if (resource === "wood" || resource === "food")
+			{
+				this.CCResourceMaps[resource].addInfluence(x, z, 15, strength/2.0,'constant');
+				this.resourceMaps[resource].addInfluence(x, z, 9.0, strength,'constant');
+				this.resourceMaps[resource].addInfluence(x, z, 2, -5,'constant');
+			}
+			else if (resource === "stone" || resource === "metal")
+			{
+				this.CCResourceMaps[resource].addInfluence(x, z, 30, strength,'constant');
+				this.resourceMaps[resource].addInfluence(x, z, 12.0, strength/1.5);
+				this.resourceMaps[resource].addInfluence(x, z, 12.0, strength/2.0,'constant');
+				this.resourceMaps[resource].addInfluence(x, z, 8, -50);
 			}
 		}
 	}
