@@ -60,24 +60,6 @@ m.AssocArraytoArray = function(assocArray) {
 	return endArray;
 };
 
-m.MemoizeInit = function(obj)
-{
-	obj._memoizeCache = {};
-}
-
-m.Memoize = function(funcname, func)
-{
-	return function() {
-		var args = funcname + '|' + Array.prototype.join.call(arguments, '|');
-		if (args in this._memoizeCache)
-			return this._memoizeCache[args];
-
-		var ret = func.apply(this, arguments);
-		this._memoizeCache[args] = ret;
-		return ret;
-	};
-}
-
 m.ShallowClone = function(obj)
 {
 	var ret = {};
@@ -87,12 +69,59 @@ m.ShallowClone = function(obj)
 }
 
 // Picks a random element from an array
-m.PickRandom = function(list){
+m.PickRandom = function(list)
+{
 	if (list.length === 0)
 		return undefined;
 	else
 		return list[Math.floor(Math.random()*list.length)];
-}
+};
+
+
+// Utility functions for conversions of maps of different sizes
+// It expects that cell size of map 1 is a multiple of cell size of map 2
+
+// return the index of map2 with max content from indices contained inside the cell i of map1 
+m.getMaxMapIndex = function(i, map1, map2)
+{
+	var ratio = map1.cellSize / map2.cellSize;
+	var ix = (i % map1.width) * ratio;
+	var iy = Math.floor(i / map1.width) * ratio;
+	var index = undefined;
+	for (var kx = 0; kx < ratio; ++kx)
+		for (var ky = 0; ky < ratio; ++ky)
+			if (!index || map2.map[ix+kx+(iy+ky)*map2.width] > map2.map[index])
+				index = ix+kx+(iy+ky)*map2.width;
+	return index;
+};
+
+// return the list of indices of map2 contained inside the cell i of map1 
+// map1.cellSize must be a multiple of map2.cellSize
+m.getMapIndices = function(i, map1, map2)
+{
+	var ratio = map1.cellSize / map2.cellSize;	// TODO check that this is integer >= 1 ?
+	var ix = (i % map1.width) * ratio;
+	var iy = Math.floor(i / map1.width) * ratio;
+	var ret = [];
+	for (var kx = 0; kx < ratio; ++kx)
+		for (var ky = 0; ky < ratio; ++ky)
+			ret.push(ix+kx+(iy+ky)*map2.width);
+	return ret;
+};
+
+// return the list of points of map2 contained inside the cell i of map1 
+// map1.cellSize must be a multiple of map2.cellSize
+m.getMapPoints = function(i, map1, map2)
+{
+	var ratio = map1.cellSize / map2.cellSize;	// TODO check that this is integer >= 1 ?
+	var ix = (i % map1.width) * ratio;
+	var iy = Math.floor(i / map1.width) * ratio;
+	var ret = [];
+	for (var kx = 0; kx < ratio; ++kx)
+		for (var ky = 0; ky < ratio; ++ky)
+			ret.push([ix+kx, iy+ky]);
+	return ret;
+};
 
 return m;
 
