@@ -12,37 +12,40 @@ var PETRA = function(m)
 	-updating whatever needs updating, keeping track of stuffs (rebuilding needs…)
  */
 
-m.BaseManager = function(Config)
+m.BaseManager = function(gameState, Config)
 {
 	this.Config = Config;
-	this.ID = m.playerGlobals[PlayerID].uniqueIDBases++;
-	
+	this.ID = gameState.ai.uniqueIDs.bases++;
+
 	// anchor building: seen as the main building of the base. Needs to have territorial influence
 	this.anchor = undefined;
+	this.anchorId = undefined;
 	this.accessIndex = undefined;
-	
+
 	// Maximum distance (from any dropsite) to look for resources
 	// 3 areas are used: from 0 to max/4, from max/4 to max/2 and from max/2 to max 
 	this.maxDistResourceSquare = 360*360;
-	
+
 	this.constructing = false;
-	
+	// Defenders to train in this cc when its construction is finished 
+	this.neededDefenders = ((this.Config.difficulty > 2) ? 3 + 2*(this.Config.difficulty - 3) : 0);
+
 	// vector for iterating, to check one use the HQ map.
 	this.territoryIndices = [];
 };
 
 m.BaseManager.prototype.init = function(gameState, unconstructed)
 {
-	this.constructing = unconstructed;
+	if (unconstructed !== undefined)
+		this.constructing = unconstructed;
+	else
+		this.neededDefenders = 0;
+	this.workerObject = new m.Worker(this);
 	// entitycollections
 	this.units = gameState.getOwnUnits().filter(API3.Filters.byMetadata(PlayerID, "base", this.ID));
 	this.workers = this.units.filter(API3.Filters.byMetadata(PlayerID,"role","worker"));
 	this.buildings = gameState.getOwnStructures().filter(API3.Filters.byMetadata(PlayerID, "base", this.ID));	
 
-	this.units.allowQuickIter();
-	this.workers.allowQuickIter();
-	this.buildings.allowQuickIter();
-	
 	this.units.registerUpdates();
 	this.workers.registerUpdates();
 	this.buildings.registerUpdates();
@@ -58,12 +61,14 @@ m.BaseManager.prototype.init = function(gameState, unconstructed)
 	}
 };
 
-m.BaseManager.prototype.assignEntity = function(unit)
+m.BaseManager.prototype.assignEntity = function(gameState, ent)
 {
-	unit.setMetadata(PlayerID, "base", this.ID);
-	this.units.updateEnt(unit);
-	this.workers.updateEnt(unit);
-	this.buildings.updateEnt(unit);
+	ent.setMetadata(PlayerID, "base", this.ID);
+	this.units.updateEnt(ent);
+	this.workers.updateEnt(ent);
+	this.buildings.updateEnt(ent);
+	if (ent.resourceDropsiteTypes() && !ent.hasClass("Elephant"))
+		this.assignResourceToDropsite(gameState, ent);
 };
 
 m.BaseManager.prototype.setAnchor = function(gameState, anchorEntity)
@@ -74,75 +79,49 @@ m.BaseManager.prototype.setAnchor = function(gameState, anchorEntity)
 		return false;
 	}
 	this.anchor = anchorEntity;
+	this.anchorId = anchorEntity.id();
 	this.anchor.setMetadata(PlayerID, "base", this.ID);
 	this.anchor.setMetadata(PlayerID, "baseAnchor", true);
 	this.buildings.updateEnt(this.anchor);
 	this.accessIndex = gameState.ai.accessibility.getAccessValue(this.anchor.position());
-	// in case all our other bases were destroyed, reaffect these destroyed bases to this base
-	for each (var base in gameState.ai.HQ.baseManagers)
+	// in case some of our other bases were destroyed, reaffect these destroyed bases to this base
+	for (var base of gameState.ai.HQ.baseManagers)
 	{
-		if (base.anchor || base.newbase)
+		if (base.anchor || base.newbaseID)
 			continue;
-		base.newbase = this.ID;
+		base.newbaseID = this.ID;
 	}
 	return true;
 };
 
 m.BaseManager.prototype.checkEvents = function (gameState, events, queues)
 {
-	var renameEvents = events["EntityRenamed"];
 	var destEvents = events["Destroy"];
 	var createEvents = events["Create"];
 	var cFinishedEvents = events["ConstructionFinished"];
 
-	for (var evt of renameEvents)
-	{
-		var ent = gameState.getEntityById(evt.newentity);
-		if (!ent)
-			continue;
-		var workerObject = ent.getMetadata(PlayerID, "worker-object");
-		if (workerObject)
-			workerObject.ent = ent;
-	}
-
 	for (var evt of destEvents)
 	{
 		// let's check we haven't lost an important building here.
-		if (evt && !evt.SuccessfulFoundation && evt.entityObj != undefined && evt.metadata !== undefined && evt.metadata[PlayerID] &&
-			evt.metadata[PlayerID]["base"] !== undefined && evt.metadata[PlayerID]["base"] == this.ID)
+		if (evt && !evt.SuccessfulFoundation && evt.entityObj && evt.metadata && evt.metadata[PlayerID] &&
+			evt.metadata[PlayerID]["base"] && evt.metadata[PlayerID]["base"] == this.ID)
 		{
 			var ent = evt.entityObj;
 			if (ent.resourceDropsiteTypes() && !ent.hasClass("Elephant"))
 				this.removeDropsite(gameState, ent);
-			if (evt.metadata[PlayerID]["baseAnchor"] && evt.metadata[PlayerID]["baseAnchor"] == true)
+			if (evt.metadata[PlayerID]["baseAnchor"] && evt.metadata[PlayerID]["baseAnchor"] === true)
 			{
 				// sounds like we lost our anchor. Let's reaffect our units and buildings
 				this.anchor = undefined;
-				var distmin = Math.min();
-				var basemin = undefined;
-				for each (var base in gameState.ai.HQ.baseManagers)
-				{
-					if (!base.anchor)
-						continue;
-					var dist = API3.SquareVectorDistance(base.anchor.position(), ent.position());
-					if (base.accessIndex !== this.accessIndex)
-						dist += 100000000;
-					if (dist > distmin)
-						continue;
-					distmin = dist;
-					basemin = base;
-				}
-				if (!basemin)
-				{
-					if (this.Config.debug > 1)
-						API3.warn(" base " + this.ID + " destroyed and no other bases found");
-					continue;
-				}
-				this.newbase = basemin.ID;
-				this.units.forEach( function (ent) { ent.setMetadata(PlayerID, "base", basemin.ID); });
-				this.buildings.forEach( function (ent) { ent.setMetadata(PlayerID, "base", basemin.ID); });
+				this.anchorId = undefined;
+				this.neededDefenders = 0;
+				let bestbase = m.getBestBase(ent, gameState);
+				this.newbaseID = bestbase.ID;
+				for (let entity of this.units.values())
+					bestbase.assignEntity(gameState, entity);
+				for (let entity of this.buildings.values())
+					bestbase.assignEntity(gameState, entity);
 			}
-			
 		}
 	}
 
@@ -153,10 +132,10 @@ m.BaseManager.prototype.checkEvents = function (gameState, events, queues)
 		var ent = gameState.getEntityById(evt.newentity);
 		if (ent === undefined)
 			continue;
-		if (evt.newentity === evt.entity)  // repaired building
+		if (evt.newentity == evt.entity)  // repaired building
 			continue;
 			
-		if (ent.getMetadata(PlayerID,"base") == this.ID)
+		if (ent.getMetadata(PlayerID, "base") == this.ID)
 		{
 			if (ent.resourceDropsiteTypes() && !ent.hasClass("Elephant"))
 				this.assignResourceToDropsite(gameState, ent);
@@ -173,6 +152,15 @@ m.BaseManager.prototype.checkEvents = function (gameState, events, queues)
 
 		// do necessary stuff here
 	}
+
+	let renameEvents = events["EntityRenamed"];
+	for (let evt of renameEvents)
+	{
+		if (!this.anchorId || this.anchorId !== evt.entity)
+			continue;
+		this.anchorId = evt.newentity;
+		this.anchor = gameState.getEntityById(evt.newentity);
+	}
 };
 
 /**
@@ -183,17 +171,29 @@ m.BaseManager.prototype.assignResourceToDropsite = function (gameState, dropsite
 {
 	if (this.dropsites[dropsite.id()])
 	{
-		if (this.Config.debug > 1)
+		if (this.Config.debug > 0)
 			warn("assignResourceToDropsite: dropsite already in the list. Should never happen");
 		return;
 	}
 	this.dropsites[dropsite.id()] = true;
 
 	var self = this;
+	let dropsitePos = dropsite.position();
+	let accessIndex = this.accessIndex;
+	if (this.ID === gameState.ai.HQ.baseManagers[0].ID)
+	{
+		accessIndex = dropsite.getMetadata(PlayerID, "access");
+		if (!accessIndex)
+		{
+			accessIndex = gameState.ai.accessibility.getAccessValue(dropsitePos);
+			dropsite.setMetadata(PlayerID, "access", accessIndex);
+		}
+	}
+
 	for (var type of dropsite.resourceDropsiteTypes())
 	{
 		var resources = gameState.getResourceSupplies(type);
-		if (resources.length === 0)
+		if (resources.length == 0)
 			continue;
 
 		var nearby = this.dropsiteSupplies[type]["nearby"];
@@ -204,25 +204,23 @@ m.BaseManager.prototype.assignResourceToDropsite = function (gameState, dropsite
 		{
 			if (!supply.position())
 				return;
-			if (supply.getMetadata(PlayerID, "inaccessible") === true)
-				return;
-			if (supply.hasClass("Animal"))    // moving resources are treated differently TODO
+			if (supply.hasClass("Animal"))    // moving resources are treated differently
 				return;
 			if (supply.hasClass("Field"))     // fields are treated separately
 				return;
 			if (supply.resourceSupplyType()["generic"] === "treasure")  // treasures are treated separately
 				return;
 			// quick accessibility check
-			var access = supply.getMetadata(PlayerID, "access");
+			let access = supply.getMetadata(PlayerID, "access");
 			if (!access)
 			{
 				access = gameState.ai.accessibility.getAccessValue(supply.position());
 				supply.setMetadata(PlayerID, "access", access);
 			}
-			if (access !== self.accessIndex)
+			if (access !== accessIndex)
 				return;
 
-			var dist = API3.SquareVectorDistance(supply.position(), dropsite.position());
+			let dist = API3.SquareVectorDistance(supply.position(), dropsitePos);
 			if (dist < self.maxDistResourceSquare)
 			{
 				if (dist < self.maxDistResourceSquare/16)        // distmax/4
@@ -267,7 +265,7 @@ m.BaseManager.prototype.removeDropsite = function (gameState, ent)
 			if (!supply[i].ent || !gameState.getEntityById(supply[i].id))
 				supply.splice(i--, 1);
 			// resource assigned to the removed dropsite, remove it
-			else if (supply[i].dropsite === entId)
+			else if (supply[i].dropsite == entId)
 				supply.splice(i--, 1);
 		}
 	};
@@ -298,21 +296,24 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 	var obstructions = m.createObstructionMap(gameState, this.accessIndex, template);
 	obstructions.expandInfluences();
 	
-	var DPFoundations = gameState.getOwnFoundations().filter(API3.Filters.byType(gameState.applyCiv("foundation|structures/{civ}_storehouse")));
-
+	var DPFoundations = gameState.getOwnFoundations().filter(API3.Filters.byType(gameState.applyCiv("foundation|structures/{civ}_storehouse"))).toEntityArray();
 	var ccEnts = gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
 
-	var width = obstructions.width;
 	var bestIdx = undefined;
 	var bestVal = undefined;
-	var radius = Math.ceil(template.obstructionRadius() / gameState.cellSize);
+	var radius = Math.ceil(template.obstructionRadius() / obstructions.cellSize);
+
+	var territoryMap = gameState.sharedScript.territoryMap;
+	var width = territoryMap.width;
+	var cellSize = territoryMap.cellSize;
 
 	for (var p = 0; p < this.territoryIndices.length; ++p)
 	{
 		var j = this.territoryIndices[p];
-		if (obstructions.map[j] <= radius)  // check room around
+		var i = API3.getMaxMapIndex(j, territoryMap, obstructions);
+		if (obstructions.map[i] <= radius)  // check room around
 			continue;
-		
+
 		// we add 3 times the needed resource and once the other two (not food)
 		var total = 0;
 		for (var i in gameState.sharedScript.resourceMaps)
@@ -326,8 +327,7 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 
 		total = 0.7*total;   // Just a normalisation factor as the locateMap is limited to 255
 
-		var pos = [j%width+0.5, Math.floor(j/width)+0.5];
-		pos = [gameState.cellSize*pos[0], gameState.cellSize*pos[1]];
+		var pos = [cellSize * (j%width+0.5), cellSize * (Math.floor(j/width)+0.5)];
 		for (var i in this.dropsites)
 		{
 			if (!gameState.getEntityById(i))
@@ -347,12 +347,12 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 		if (total == 0)
 			continue;
 
-		for (var i in DPFoundations._entities)
+		for (let dp of DPFoundations)
 		{
-			var dpPos = gameState.getEntityById(i).position();
+			let dpPos = dp.position();
 			if (!dpPos)
 				continue;
-			var dist = API3.SquareVectorDistance(dpPos, pos);
+			let dist = API3.SquareVectorDistance(dpPos, pos);
 			if (dist < 3600)
 			{
 				total = 0;
@@ -392,8 +392,9 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 
 	if (bestVal <= 0)
 		return {"quality": bestVal, "pos": [0, 0]};
-	var x = ((bestIdx % width) + 0.5) * gameState.cellSize;
-	var z = (Math.floor(bestIdx / width) + 0.5) * gameState.cellSize;
+	var i = API3.getMaxMapIndex(bestIdx, territoryMap, obstructions);
+	var x = ((i % obstructions.width) + 0.5) * obstructions.cellSize;
+	var z = (Math.floor(i / obstructions.width) + 0.5) * obstructions.cellSize;
 	return {"quality": bestVal, "pos": [x, z]};
 };
 
@@ -427,34 +428,30 @@ m.BaseManager.prototype.checkResourceLevels = function (gameState, queues)
 	{
 		if (type == "food")
 		{
-			var count = this.getResourceLevel(gameState, type);  // TODO animals are not accounted, may-be we should
-			var numFarms = gameState.countEntitiesByType(gameState.applyCiv("structures/{civ}_field"), true);
-			var numFound = gameState.countEntitiesByType(gameState.applyCiv("foundation|structures/{civ}_field"), true);
-			var numQueue = queues.field.countQueuedUnits();
-
-			// TODO  if not yet farms, add a check on time used/lost and build farmstead if needed
-			if (count < 1200 && numFarms + numFound + numQueue === 0)     // tell the queue manager we'll be trying to build fields shortly.
+			if (gameState.ai.HQ.canBuild(gameState, "structures/{civ}_field"))	// let's see if we need to add new farms.
 			{
-				for (var  i = 0; i < this.Config.Economy.initialFields; ++i)
+				var count = this.getResourceLevel(gameState, type);  // TODO animals are not accounted, may-be we should
+				var numFarms = gameState.countEntitiesByType(gameState.applyCiv("structures/{civ}_field"), true);
+				var numFound = gameState.countEntitiesByType(gameState.applyCiv("foundation|structures/{civ}_field"), true);
+				var numQueue = queues.field.countQueuedUnits();
+
+				// TODO  if not yet farms, add a check on time used/lost and build farmstead if needed
+				if (numFarms + numFound + numQueue === 0)	// starting game, rely on fruits as long as we have enough of them
 				{
-					var plan = new m.ConstructionPlan(gameState, "structures/{civ}_field", { "base" : this.ID });
-					plan.isGo = function() { return false; };	// don't start right away.
-					queues.field.addItem(plan);
+					if (count < 600)
+						queues.field.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_field", { "base" : this.ID }));
+				}
+				else
+				{
+					let goal = this.Config.Economy.provisionFields;
+					if (gameState.ai.HQ.saveResources || gameState.ai.HQ.saveSpace)
+						goal = Math.max(goal-1, 1);
+					if (numFound + numQueue < goal)
+						queues.field.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_field", { "base" : this.ID }));
 				}
 			}
-			else if (count < 400 && numFarms + numFound === 0)
-			{
-				for (var i in queues.field.queue)
-					queues.field.queue[i].isGo = function() { return true; };	// start them
-			}
-			else if (gameState.ai.HQ.canBuild(gameState, "structures/{civ}_field"))	// let's see if we need to add new farms.
-			{
-				if ((!gameState.ai.HQ.saveResources && numFound < 2 && numFound + numQueue < 3) ||
-					(gameState.ai.HQ.saveResources && numFound < 1 && numFound + numQueue < 2))
-					queues.field.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_field", { "base" : this.ID }));
-			}
 		}
-		else if (queues.dropsites.length() === 0 && gameState.countFoundationsByType(gameState.applyCiv("structures/{civ}_storehouse"), true) === 0)
+		else if (queues.dropsites.length() == 0 && gameState.countFoundationsByType(gameState.applyCiv("structures/{civ}_storehouse"), true) == 0)
 		{
 			if (gameState.ai.playedTurn > this.gatherers[type].nextCheck)
 			{
@@ -475,7 +472,7 @@ m.BaseManager.prototype.checkResourceLevels = function (gameState, queues)
 						var newDP = this.findBestDropsiteLocation(gameState, type);
 						if (newDP.quality > 50 && gameState.ai.HQ.canBuild(gameState, "structures/{civ}_storehouse"))
 							queues.dropsites.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_storehouse", { "base": this.ID }, newDP.pos));
-						else if (gameState.countFoundationsByType(gameState.ai.HQ.bBase[0], true) === 0 && queues.civilCentre.length() === 0)
+						else if (gameState.countFoundationsByType(gameState.ai.HQ.bBase[0], true) == 0 && queues.civilCentre.length() == 0)
 						{
 							// No good dropsite, try to build a new base if no base already planned,
 							// and if not possible, be less strict on dropsite quality
@@ -488,7 +485,7 @@ m.BaseManager.prototype.checkResourceLevels = function (gameState, queues)
 					this.gatherers[type].used = 0;
 					this.gatherers[type].lost = 0;
 				}
-				else if (total === 0)
+				else if (total == 0)
 					this.gatherers[type].nextCheck = gameState.ai.playedTurn + 10;
 			}
 		}
@@ -502,7 +499,7 @@ m.BaseManager.prototype.checkResourceLevels = function (gameState, queues)
 	
 };
 
-// let's return the estimated gather rates.
+// Adds the estimated gather rates from this base to the currentRates
 m.BaseManager.prototype.getGatherRates = function(gameState, currentRates)
 {
 	for (var i in currentRates)
@@ -512,25 +509,22 @@ m.BaseManager.prototype.getGatherRates = function(gameState, currentRates)
 		// Given that the faster you gather, the more travel time matters,
 		// I use some logarithms.
 		// TODO: this should take into account for unit speed and/or distance to target
-		
-		var units = this.gatherersByType(gameState, i);
-		units.forEach(function (ent) {
+
+		this.gatherersByType(gameState, i).forEach(function (ent) {
 			var gRate = ent.currentGatherRate();
 			if (gRate !== undefined)
 				currentRates[i] += Math.log(1+gRate)/1.1;
 		});
 		if (i === "food")
 		{
-			units = this.workers.filter(API3.Filters.byMetadata(PlayerID, "subrole", "hunter"));
-			units.forEach(function (ent) {
+			this.workersBySubrole(gameState, "hunter").forEach(function (ent) {
 				if (ent.isIdle())
 					return;
 				var gRate = ent.currentGatherRate()
 				if (gRate !== undefined)
 					currentRates[i] += Math.log(1+gRate)/1.1;
 			});
-			units = this.workers.filter(API3.Filters.byMetadata(PlayerID, "subrole", "fisher"));
-			units.forEach(function (ent) {
+			this.workersBySubrole(gameState, "fisher").forEach(function (ent) {
 				if (ent.isIdle())
 					return;
 				var gRate = ent.currentGatherRate()
@@ -545,14 +539,14 @@ m.BaseManager.prototype.getGatherRates = function(gameState, currentRates)
 m.BaseManager.prototype.assignRolelessUnits = function(gameState)
 {
 	// TODO: make this cleverer.
-	var roleless = this.units.filter(API3.Filters.not(API3.Filters.byHasMetadata(PlayerID, "role")));
-	var self = this;
-	roleless.forEach(function(ent) {
+	var roleless = this.units.filter(API3.Filters.not(API3.Filters.byHasMetadata(PlayerID, "role"))).values();
+	for (var ent of roleless)
+	{
 		if (ent.hasClass("Worker") || ent.hasClass("CitizenSoldier") || ent.hasClass("FishingBoat"))
 			ent.setMetadata(PlayerID, "role", "worker");
 		else if (ent.hasClass("Support") && ent.hasClass("Elephant"))
 			ent.setMetadata(PlayerID, "role", "worker");
-	});
+	}
 };
 
 // If the numbers of workers on the resources is unbalanced then set some of workers to idle so
@@ -560,9 +554,6 @@ m.BaseManager.prototype.assignRolelessUnits = function(gameState)
 // TODO: actually this probably should be in the HQ.
 m.BaseManager.prototype.setWorkersIdleByPriority = function(gameState)
 {
-	if (gameState.currentPhase() < 2)
-		return;
-
 	// change resource only towards one which is more needed, and if changing will not change this order
 	var nb = 1;    // no more than 1 change per turn (otherwise we should update the rates)
 	var mostNeeded = gameState.ai.HQ.pickMostNeededResources(gameState);
@@ -590,15 +581,26 @@ m.BaseManager.prototype.setWorkersIdleByPriority = function(gameState)
 			// but we require a bit more to avoid too frequent changes
 			if ((scale*moreNeed.wanted - moreNeed.current) - (scale*lessNeed.wanted - lessNeed.current) > 1.5)
 			{
-				this.gatherersByType(gameState, lessNeed.type).forEach( function (ent) {
-					if (nb === 0)
+				let only = undefined;
+				// in average, females are less efficient for stone and metal, and citizenSoldiers for food
+				let gatherers = this.gatherersByType(gameState, lessNeed.type);
+				if (lessNeed.type === "food" && gatherers.filter(API3.Filters.byClass("CitizenSoldier")).length)
+					only = "CitizenSoldier";
+				else if ((lessNeed.type === "stone" || lessNeed.type === "metal") && moreNeed.type !== "stone" && moreNeed.type !== "metal"
+					&& gatherers.filter(API3.Filters.byClass("Female")).length) 
+					only = "Female";
+
+				gatherers.forEach( function (ent) {
+					if (nb == 0)
+						return;
+					if (only && !ent.hasClass(only))
 						return;
 					--nb;
 					ent.stopMoving();
 					ent.setMetadata(PlayerID, "gather-type", moreNeed.type);
 					m.AddTCRessGatherer(gameState, moreNeed.type);
 				});
-				if (nb === 0)
+				if (nb == 0)
 					return;
 			}
 		}
@@ -609,7 +611,7 @@ m.BaseManager.prototype.setWorkersIdleByPriority = function(gameState)
 m.BaseManager.prototype.reassignIdleWorkers = function(gameState)
 {
 	// Search for idle workers, and tell them to gather resources based on demand
-	var filter = API3.Filters.or(API3.Filters.byMetadata(PlayerID,"subrole","idle"), API3.Filters.not(API3.Filters.byHasMetadata(PlayerID,"subrole")));
+	var filter = API3.Filters.byMetadata(PlayerID, "subrole", "idle");
 	var idleWorkers = gameState.updatingCollection("idle-workers-base-" + this.ID, filter, this.workers);
 
 	var self = this;
@@ -629,7 +631,9 @@ m.BaseManager.prototype.reassignIdleWorkers = function(gameState)
 
 			if (ent.hasClass("Worker"))
 			{
-				if (self.anchor && self.anchor.needsRepair() === true)
+				// Just emergency repairing here. It is better managed in assignToFoundations
+				if (self.anchor && self.anchor.needsRepair() === true
+					&& gameState.getOwnEntitiesByMetadata("target-foundation", self.anchor.id()).length < 2)
 					ent.repair(self.anchor);
 				else
 				{
@@ -656,7 +660,7 @@ m.BaseManager.prototype.reassignIdleWorkers = function(gameState)
 
 m.BaseManager.prototype.workersBySubrole = function(gameState, subrole)
 {
-	return gameState.updatingCollection("subrole-" + subrole +"-base-" + this.ID, API3.Filters.byMetadata(PlayerID, "subrole", subrole), this.workers, true);
+	return gameState.updatingCollection("subrole-" + subrole +"-base-" + this.ID, API3.Filters.byMetadata(PlayerID, "subrole", subrole), this.workers);
 };
 
 m.BaseManager.prototype.gatherersByType = function(gameState, type)
@@ -672,7 +676,7 @@ m.BaseManager.prototype.pickBuilders = function(gameState, workers, number)
 	var availableWorkers = this.workers.filter(function (ent) {
 		if (!ent.position())
 			return false;
-		if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
+		if (ent.getMetadata(PlayerID, "plan") == -2 || ent.getMetadata(PlayerID, "plan") == -3)
 			return false;
 		if (ent.getMetadata(PlayerID, "transport"))
 			return false;
@@ -721,7 +725,7 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 		if (ent.foundationProgress() === undefined && ent.needsRepair())
 			return true;
 		return false;
-	}).toEntityArray();
+	});
 	
 	// Check if nothing to build
 	if (!foundations.length && !damagedBuildings.length){
@@ -729,10 +733,10 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 	}
 	var workers = this.workers.filter(API3.Filters.not(API3.Filters.or(API3.Filters.byClass("Cavalry"), API3.Filters.byClass("Ship"))));
 	var builderWorkers = this.workersBySubrole(gameState, "builder");
-	var idleBuilderWorkers = this.workersBySubrole(gameState, "builder").filter(API3.Filters.isIdle());
+	var idleBuilderWorkers = builderWorkers.filter(API3.Filters.isIdle());
 
 	// if we're constructing and we have the foundations to our base anchor, only try building that.
-	if (this.constructing == true && this.buildings.filter(API3.Filters.and(API3.Filters.isFoundation(), API3.Filters.byMetadata(PlayerID, "baseAnchor", true))).length !== 0)
+	if (this.constructing == true && this.buildings.filter(API3.Filters.and(API3.Filters.isFoundation(), API3.Filters.byMetadata(PlayerID, "baseAnchor", true))).length != 0)
 	{
 		foundations = this.buildings.filter(API3.Filters.byMetadata(PlayerID, "baseAnchor", true)).toEntityArray();
 		var tID = foundations[0].id();
@@ -748,7 +752,7 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 
 	if (workers.length < 2)
 	{
-		var noobs = gameState.ai.HQ.bulkPickWorkers(gameState, this.ID, 2);
+		var noobs = gameState.ai.HQ.bulkPickWorkers(gameState, this, 2);
 		if(noobs)
 		{
 			noobs.forEach(function (worker) {
@@ -760,11 +764,8 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 			});
 		}
 	}
-	var addedWorkers = 0;
-	
-	var maxTotalBuilders = Math.ceil(workers.length * 0.2);
-	if (this.constructing == true && maxTotalBuilders < 15)
-		maxTotalBuilders = 15;
+
+	var builderTot = builderWorkers.length - idleBuilderWorkers.length;	
 	
 	for (var target of foundations)
 	{
@@ -772,83 +773,95 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 			continue; // we do not build fields
 
 		if (gameState.ai.HQ.isDangerousLocation(target.position()))
-			if (!target.hasClass("CivCentre") && !target.hasClass("StoneWall"))
+			if (!target.hasClass("CivCentre") && !target.hasClass("StoneWall") && (!target.hasClass("Wonder") || gameState.getGameType() !== "wonder"))
 				continue;
 
+		// if our territory has shrinked since this foundation was positioned, do not build it
+		if (m.isNotWorthBuilding(target, gameState))
+			continue;
+
 		var assigned = gameState.getOwnEntitiesByMetadata("target-foundation", target.id()).length;
-		var targetNB = this.Config.Economy.targetNumBuilders;	// TODO: dynamic that.
-		if (target.hasClass("House") || target.hasClass("Market"))
-			targetNB *= 2;
+		var maxTotalBuilders = Math.ceil(workers.length * 0.2);
+		var targetNB = 2;
+		if (target.hasClass("House") || target.hasClass("Market") || target.hasClass("DropsiteWood"))
+			targetNB = 3;
 		else if (target.hasClass("Barracks") || target.hasClass("Tower"))
 			targetNB = 4;
 		else if (target.hasClass("Fortress"))
 			targetNB = 7;
 		if (target.getMetadata(PlayerID, "baseAnchor") == true || (target.hasClass("Wonder") && gameState.getGameType() === "wonder"))
+		{
 			targetNB = 15;
+			maxTotalBuilders = Math.max(maxTotalBuilders, 15);
+		}
+		// if no base yet, everybody should build
+		if (gameState.ai.HQ.numActiveBase() === 0)
+		{
+			targetNB = workers.length;
+			maxTotalBuilders = targetNB;
+		}
 
 		if (assigned < targetNB)
 		{
-			if (builderWorkers.length - idleBuilderWorkers.length + addedWorkers < maxTotalBuilders) {
-
-				var addedToThis = 0;
-				
-				idleBuilderWorkers.forEach(function(ent) {
-					if (ent.position() && API3.SquareVectorDistance(ent.position(), target.position()) < 10000 && assigned + addedToThis < targetNB)
-					{
-						addedWorkers++;
-						addedToThis++;
-						ent.setMetadata(PlayerID, "target-foundation", target.id());
-					}
-				});
-				if (assigned + addedToThis < targetNB)
+			idleBuilderWorkers.forEach(function(ent) {
+				if (ent.getMetadata(PlayerID, "target-foundation") !== undefined)
+					return;
+				if (assigned >= targetNB || !ent.position() || API3.SquareVectorDistance(ent.position(), target.position()) > 40000)
+					return;
+				assigned++;
+				builderTot++;
+				ent.setMetadata(PlayerID, "target-foundation", target.id());
+			});
+			if (assigned < targetNB && builderTot < maxTotalBuilders)
+			{
+				var nonBuilderWorkers = workers.filter(function(ent) {
+					if (ent.getMetadata(PlayerID, "subrole") === "builder")
+						return false;
+					if (!ent.position())
+						return false;
+					if (ent.getMetadata(PlayerID, "plan") == -2 || ent.getMetadata(PlayerID, "plan") == -3)
+						return false;
+					if (ent.getMetadata(PlayerID, "transport"))
+						return false;
+					return true;
+				}).toEntityArray();
+				var time = target.buildTime();
+				nonBuilderWorkers.sort(function (workerA,workerB)
 				{
-					var nonBuilderWorkers = workers.filter(function(ent) {
-						if (ent.getMetadata(PlayerID, "subrole") === "builder")
-							return false;
-						if (!ent.position())
-							return false;
-						if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
-							return false;
-						if (ent.getMetadata(PlayerID, "transport"))
-							return false;
-						return true;
-					}).toEntityArray();
-					var time = target.buildTime();
-					nonBuilderWorkers.sort(function (workerA,workerB)
-					{
-						var coeffA = API3.SquareVectorDistance(target.position(),workerA.position());
-						// elephant moves slowly, so when far away they are only useful if build time is long
-						if (workerA.hasClass("Elephant"))
-							coeffA *= 0.5 * (1 + (Math.sqrt(coeffA)/150)*(30/time));
-						else if (workerA.getMetadata(PlayerID, "gather-type") === "food")
-							coeffA *= 3;
-						var coeffB = API3.SquareVectorDistance(target.position(),workerB.position());
-						if (workerB.hasClass("Elephant"))
-							coeffB *= 0.5 * (1 + (Math.sqrt(coeffB)/150)*(30/time));
-						else if (workerB.getMetadata(PlayerID, "gather-type") === "food")
-							coeffB *= 3;
-						return (coeffA - coeffB);						
-					});
-					var current = 0;
-					while (assigned + addedToThis < targetNB && current < nonBuilderWorkers.length)
-					{
-						addedWorkers++;
-						addedToThis++;
-						var ent = nonBuilderWorkers[current++];
-						ent.stopMoving();
-						ent.setMetadata(PlayerID, "subrole", "builder");
-						ent.setMetadata(PlayerID, "target-foundation", target.id());
-					};
-				}
+					let coeffA = API3.SquareVectorDistance(target.position(),workerA.position());
+					// elephant moves slowly, so when far away they are only useful if build time is long
+					if (workerA.hasClass("Elephant"))
+						coeffA *= 0.5 * (1 + (Math.sqrt(coeffA)/150)*(30/time));
+					else if (workerA.getMetadata(PlayerID, "gather-type") === "food")
+						coeffA *= 3;
+					let coeffB = API3.SquareVectorDistance(target.position(),workerB.position());
+					if (workerB.hasClass("Elephant"))
+						coeffB *= 0.5 * (1 + (Math.sqrt(coeffB)/150)*(30/time));
+					else if (workerB.getMetadata(PlayerID, "gather-type") === "food")
+						coeffB *= 3;
+					return (coeffA - coeffB);						
+				});
+				let current = 0;
+				let nonBuilderTot = nonBuilderWorkers.length;
+				while (assigned < targetNB && builderTot < maxTotalBuilders && current < nonBuilderTot)
+				{
+					assigned++;
+					builderTot++;
+					var ent = nonBuilderWorkers[current++];
+					ent.stopMoving();
+					ent.setMetadata(PlayerID, "subrole", "builder");
+					ent.setMetadata(PlayerID, "target-foundation", target.id());
+				};
 			}
 		}
 	}
 
-	// don't repair if we're still under attack, unless it's like a vital (civcentre or wall) building that's getting destroyed.
-	for (var target of damagedBuildings)
+	for (var target of damagedBuildings.values())
 	{
+		// don't repair if we're still under attack, unless it's a vital (civcentre or wall) building that's getting destroyed.
 		if (gameState.ai.HQ.isDangerousLocation(target.position()))
-			if (target.healthLevel() > 0.5 || (!target.hasClass("CivCentre") && !target.hasClass("StoneWall")))
+			if (target.healthLevel() > 0.5 ||
+				(!target.hasClass("CivCentre") && !target.hasClass("StoneWall") && (!target.hasClass("Wonder") || gameState.getGameType() !== "wonder")))
 				continue;
 		else if (noRepair && !target.hasClass("CivCentre"))
 			continue;
@@ -857,26 +870,53 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 			continue;
 		
 		var assigned = gameState.getOwnEntitiesByMetadata("target-foundation", target.id()).length;
-		if (assigned < targetNB/3)
+		var maxTotalBuilders = Math.ceil(workers.length * 0.2);
+		var targetNB = 1;
+		if (target.hasClass("Fortress"))
+			targetNB = 3;
+		if (target.getMetadata(PlayerID, "baseAnchor") == true || (target.hasClass("Wonder") && gameState.getGameType() === "wonder"))
 		{
-			if (builderWorkers.length + addedWorkers < targetNB*2)
+			maxTotalBuilders = Math.ceil(workers.length * 0.3);
+			targetNB = 5;
+			if (target.healthLevel() < 0.3)
 			{
-				var nonBuilderWorkers = workers.filter(function(ent) {
+				maxTotalBuilders = Math.ceil(workers.length * 0.6);
+				targetNB = 7;
+			}
+
+		}
+
+		if (assigned < targetNB)
+		{
+			idleBuilderWorkers.forEach(function(ent) {
+				if (ent.getMetadata(PlayerID, "target-foundation") !== undefined)
+					return;
+				if (assigned >= targetNB || !ent.position() || API3.SquareVectorDistance(ent.position(), target.position()) > 40000)
+					return;
+				assigned++;
+				builderTot++;
+				ent.setMetadata(PlayerID, "target-foundation", target.id());
+			});
+			if (assigned < targetNB && builderTot < maxTotalBuilders)
+			{
+				let nonBuilderWorkers = workers.filter(function(ent) {
 					if (ent.getMetadata(PlayerID, "subrole") === "builder")
 						return false;
 					if (!ent.position())
 						return false;
-					if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
+					if (ent.getMetadata(PlayerID, "plan") == -2 || ent.getMetadata(PlayerID, "plan") == -3)
 						return false;
 					if (ent.getMetadata(PlayerID, "transport"))
 						return false;
 					return true;
 				});
-				var nearestNonBuilders = nonBuilderWorkers.filterNearest(target.position(), targetNB/3 - assigned);
+				let num = Math.min(nonBuilderWorkers.length, targetNB-assigned);
+				let nearestNonBuilders = nonBuilderWorkers.filterNearest(target.position(), num);
 				
 				nearestNonBuilders.forEach(function(ent) {
+					assigned++;
+					builderTot++;
 					ent.stopMoving();
-					addedWorkers++;
 					ent.setMetadata(PlayerID, "subrole", "builder");
 					ent.setMetadata(PlayerID, "target-foundation", target.id());
 				});
@@ -887,19 +927,49 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 
 m.BaseManager.prototype.update = function(gameState, queues, events)
 {
-	if (!this.anchor)   // this base has been destroyed
+	if (this.ID === gameState.ai.HQ.baseManagers[0].ID)	// base for unaffected units
 	{
-		// transfer possible remaining units (probably they were in training during previous transfers)
-		if (this.newbase)
+		// if some active base, reassigns the workers/buildings
+		// otherwise look for anything useful to do, i.e. treasures to gather
+		if (gameState.ai.HQ.numActiveBase() > 0)
 		{
-			var newbase = this.newbase;
-			this.units.forEach( function (ent) { ent.setMetadata(PlayerID, "base", newbase); });
-			this.buildings.forEach( function (ent) { ent.setMetadata(PlayerID, "base", newbase); });
+			for (var ent of this.units.values())
+				m.getBestBase(ent, gameState).assignEntity(gameState, ent);
+			for (var ent of this.buildings.values())
+			{
+				if (ent.resourceDropsiteTypes() && !ent.hasClass("Elephant"))
+					this.removeDropsite(gameState, ent);
+				m.getBestBase(ent, gameState).assignEntity(gameState, ent);
+			}
+		}
+		else if (gameState.ai.HQ.canBuildUnits)
+		{
+			this.assignToFoundations(gameState);
+			if (gameState.ai.playedTurn % 4 === 0)
+				this.setWorkersIdleByPriority(gameState);
+			this.assignRolelessUnits(gameState);
+			this.reassignIdleWorkers(gameState);
+			for (var ent of this.workers.values())
+				this.workerObject.update(ent, gameState);
 		}
 		return;
 	}
 
-	if (this.anchor && this.anchor.getMetadata(PlayerID, "access") !== this.accessIndex)
+	if (!this.anchor)   // this base has been destroyed
+	{
+		// transfer possible remaining units (probably they were in training during previous transfers)
+		if (this.newbaseID)
+		{
+			var newbaseID = this.newbaseID;
+			for (let ent of this.units.values())
+				ent.setMetadata(PlayerID, "base", newbaseID);
+			for (let ent of this.buildings.values())
+				ent.setMetadata(PlayerID, "base", newbaseID);
+		}
+		return;
+	}
+
+	if (this.anchor.getMetadata(PlayerID, "access") != this.accessIndex)
 		API3.warn("Petra baseManager " + this.ID + " problem with accessIndex " + this.accessIndex
 			+ " while metadata access is " + this.anchor.getMetadata(PlayerID, "access"));
 
@@ -908,41 +978,61 @@ m.BaseManager.prototype.update = function(gameState, queues, events)
 	this.checkResourceLevels(gameState, queues);
 	this.assignToFoundations(gameState);
 
-	if (this.constructing && this.anchor)
+	if (this.constructing)
 	{
 		var owner = gameState.ai.HQ.territoryMap.getOwner(this.anchor.position());
 		if(owner !== 0 && !gameState.isPlayerAlly(owner))
 		{
 			// we're in enemy territory. If we're too close from the enemy, destroy us.
-			var eEnts = gameState.getEnemyStructures().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
-			for (var i in eEnts)
+			let ccEnts = gameState.updatingGlobalCollection("allCCs", API3.Filters.byClass("CivCentre"));
+			for (let cc of ccEnts.values())
 			{
-				var entPos = eEnts[i].position();
-				if (API3.SquareVectorDistance(entPos, this.anchor.position()) < 8000)
-					this.anchor.destroy();
+				if (cc.owner() !== owner)
+					continue;
+				if (API3.SquareVectorDistance(cc.position(), this.anchor.position()) > 8000)
+					continue;
+				this.anchor.destroy();
+				break;
 			}
 		}
 	}
+	else if (this.neededDefenders && gameState.ai.HQ.trainEmergencyUnits(gameState, [this.anchor.position()]))
+		--this.neededDefenders;
 
-	if (gameState.ai.playedTurn % 2 === 0)
+	if (gameState.ai.playedTurn % 2 === 0 && gameState.currentPhase() > 1)
 		this.setWorkersIdleByPriority(gameState);
 
 	this.assignRolelessUnits(gameState);
-	
-	// should probably be last to avoid reallocations of units that would have done stuffs otherwise.
 	this.reassignIdleWorkers(gameState);
-
-	// TODO: do this incrementally a la defense.js
-	var self = this;
-	this.workers.forEach(function(ent) {
-		if (!ent.getMetadata(PlayerID, "worker-object"))
-			ent.setMetadata(PlayerID, "worker-object", new m.Worker(ent));
-		ent.getMetadata(PlayerID, "worker-object").update(self, gameState);
-	});
+	// check if workers can find something useful to do
+	for (var ent of this.workers.values())
+		this.workerObject.update(ent, gameState);
 
 	Engine.ProfileStop();
 };
 
+m.BaseManager.prototype.Serialize = function()
+{
+	return {
+		"ID": this.ID,
+		"anchorId": this.anchorId,
+		"accessIndex": this.accessIndex,
+		"maxDistResourceSquare": this.maxDistResourceSquare,
+		"constructing": this.constructing,
+		"gatherers": this.gatherers,
+		"neededDefenders": this.neededDefenders,
+		"territoryIndices": this.territoryIndices
+	};
+};
+
+m.BaseManager.prototype.Deserialize = function(gameState, data)
+{
+	for (let key in data)
+		this[key] = data[key];
+
+	this.anchor = ((this.anchorId !== undefined) ? gameState.getEntityById(this.anchorId) : undefined);
+};
+
 return m;
 
 }(PETRA);
