@@ -1,12 +1,16 @@
 var PETRA = function(m)
 {
 
-// this defines the medium difficulty
-m.Config = function() {
-	this.difficulty = 2;	// 0 is sandbox, 1 is easy, 2 is medium, 3 is hard, 4 is very hard.
-	// debug level: 0=none, 1=sanity checks, 2=debug; 3=detailed debug
+m.Config = function(difficulty)
+{
+	// 0 is sandbox, 1 is very easy, 2 is easy, 3 is medium, 4 is hard and 5 is very hard.
+	this.difficulty = (difficulty !== undefined) ? difficulty : 3;
+
+	// debug level: 0=none, 1=sanity checks, 2=debug, 3=detailed debug, -100=serializatio debug
 	this.debug = 0;
 
+	this.chat = true;   // false to prevent AI's chats
+
 	this.popScaling = 1;  // scale factor depending on the max population
 
 	this.Military = {
@@ -21,11 +25,10 @@ m.Config = function() {
 		"cityPhase" : 840,	// time to start trying to reach city phase
 		"popForMarket" : 50,
 		"popForDock" : 25,
-		"targetNumBuilders" : 1.5, // Base number of builders per foundation.
-		"targetNumTraders" : 4, // Target number of traders
+		"targetNumTraders" : 5, // Target number of traders
 		"targetNumFishers" : 1, // Target number of fishers per sea
-		"femaleRatio" : 0.5, // percent of females among the workforce.
-		"initialFields" : 5
+		"femaleRatio" : 0.5, // fraction of females among the workforce
+		"provisionFields" : 2
 	};
 
 	this.distUnitGain = 110*110;   // TODO  take it directly from helpers/TraderGain.js
@@ -34,7 +37,7 @@ m.Config = function() {
 	// defense
 	this.Defense =
 	{
-		"defenseRatio" : 2,	// see defense.js for more info.
+		"defenseRatio" : 2,	// ratio of defenders/attackers.
 		"armyCompactSize" : 2000,	// squared. Half-diameter of an army.
 		"armyBreakawaySize" : 3500,  // squared.
 		"armyMergeSize" : 1400	// squared.
@@ -53,7 +56,7 @@ m.Config = function() {
 			"athen" : [ "structures/{civ}_gymnasion", "structures/{civ}_prytaneion", "structures/{civ}_theatron" ],
 			"brit" : [ "structures/{civ}_rotarymill" ],
 			"cart" : [ "structures/{civ}_embassy_celtic",
-					"structures/{civ}_embassy_iberian", "structures/{civ}_embassy_italiote" ],
+				"structures/{civ}_embassy_iberian", "structures/{civ}_embassy_italiote" ],
 			"gaul" : [ "structures/{civ}_tavern" ],
 			"hele" : [ "structures/{civ}_gymnasion", "structures/{civ}_prytaneion", "structures/{civ}_theatron" ],
 			"iber" : [ "structures/{civ}_monument" ],
@@ -81,6 +84,7 @@ m.Config = function() {
 		"house" : 350,
 		"dropsites" : 200,
 		"field" : 400,
+		"dock" : 90,
 		"economicBuilding" : 90,
 		"militaryBuilding" : 130,
 		"defenseBuilding" : 70,
@@ -100,35 +104,79 @@ m.Config = function() {
 	this.resources = ["food", "wood", "stone", "metal"];
 };
 
-//Config.prototype = new BaseConfig();
-
-m.Config.prototype.updateDifficulty = function(difficulty)
+m.Config.prototype.setConfig = function(gameState)
 {
-	this.difficulty = difficulty;
-	// changing settings based on difficulty.
-	this.Economy.targetNumTraders = 2 * this.difficulty;
-	if (this.difficulty === 1)
+	// initialize personality traits
+	if (this.difficulty > 1)
 	{
-		this.Military.popForBarracks1 = 35;
-		this.Military.popForBarracks2 = 150;	// shouldn't reach it
-		this.Military.popForBlacksmith = 150;	// shouldn't reach it
-
-		this.Economy.cityPhase = 1800;
-		this.Economy.popForMarket = 80;
-		this.Economy.femaleRatio = 0.6;
-		this.Economy.initialFields = 2;
+		this.personality.aggressive = Math.random();
+		this.personality.cooperative = Math.random();
+		this.personality.defensive = Math.random();
 	}
-	else if (this.difficulty === 0)
+	else
+		this.personality.aggressive = 0.1;
+
+	// changing settings based on difficulty or personality
+	if (this.difficulty < 2)
 	{
 		this.Military.popForBarracks1 = 60;
-		this.Military.popForBarracks2 = 150;	// shouldn't reach it
-		this.Military.popForBlacksmith = 150;	// shouldn't reach it
+		this.Military.popForBarracks2 = 300;
+		this.Military.popForBlacksmith = 300;
 
 		this.Economy.cityPhase = 240000;
 		this.Economy.popForMarket = 200;
 		this.Economy.femaleRatio = 0.7;
-		this.Economy.initialFields = 1;
+		this.Economy.provisionFields = 1;
 	}
+	else if (this.difficulty < 3)
+	{
+		this.Military.popForBarracks1 = 35;
+		this.Military.popForBarracks2 = 150;
+		this.Military.popForBlacksmith = 150;
+
+		this.Economy.cityPhase = 1800;
+		this.Economy.popForMarket = 80;
+		this.Economy.femaleRatio = 0.6;
+		this.Economy.provisionFields = 1;
+	}
+	else
+	{
+		this.Military.towerLapseTime += Math.round(20*(this.personality.defensive - 0.5));
+		this.Military.fortressLapseTime += Math.round(60*(this.personality.defensive - 0.5));
+
+		if (this.personality.aggressive > 0.7)
+		{
+			this.Military.popForBarracks1 = 12;
+			this.Economy.popForTown = 55;
+			this.Economy.popForMarket = 60;
+			this.Economy.femaleRatio = 0.3;
+			this.priorities.defenseBuilding = 60;
+		}
+	}
+
+	this.Economy.targetNumTraders = 2 + this.difficulty;
+
+	if (gameState.getPopulationMax() < 300)
+		this.popScaling = Math.sqrt(gameState.getPopulationMax() / 300);
+
+	if (this.debug < 2)
+		return;
+	API3.warn(" >>>  Petra bot: personality = " + uneval(this.personality));
+};
+
+m.Config.prototype.Serialize = function()
+{
+	var data = {};
+	for (let key in this)
+		if (this.hasOwnProperty(key) && key != "debug")
+			data[key] = this[key];
+	return data;
+};
+
+m.Config.prototype.Deserialize = function(data)
+{
+	for (let key in data)
+		this[key] = data[key];
 };
 
 return m;
