@@ -6,12 +6,8 @@ m.DefenseManager = function(Config)
 	this.armies = [];	// array of "army" Objects
 	this.Config = Config;
 	this.targetList = [];
-}
-
-m.DefenseManager.prototype.init = function(gameState)
-{
 	this.armyMergeSize = this.Config.Defense.armyMergeSize;
-};
+}
 
 m.DefenseManager.prototype.update = function(gameState, events)
 {
@@ -31,21 +27,21 @@ m.DefenseManager.prototype.update = function(gameState, events)
 m.DefenseManager.prototype.makeIntoArmy = function(gameState, entityID)
 {
 	// Try to add it to an existing army.
-	for (var o in this.armies)
-		if (this.armies[o].addFoe(gameState,entityID))
+	for (let army of this.armies)
+		if (army.addFoe(gameState, entityID))
 			return;	// over
 
 	// Create a new army for it.
-	var army = new m.DefenseArmy(gameState, this, [], [entityID]);
+	var army = new m.DefenseArmy(gameState, [], [entityID]);
 	this.armies.push(army);
 };
 
 m.DefenseManager.prototype.getArmy = function(partOfArmy)
 {
 	// Find the army corresponding to this ID partOfArmy
-	for (var o in this.armies)
-		if (this.armies[o].ID === partOfArmy)
-		    return this.armies[o];
+	for (let army of this.armies)
+		if (army.ID === partOfArmy)
+		    return army;
 
 	return undefined;
 };
@@ -74,9 +70,9 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 		else if (target && target.hasClass("CivCentre"))
 		{
 			var myBuildings = gameState.getOwnStructures();
-			for (var i in myBuildings._entities)
+			for (let building of myBuildings.values())
 			{
-				if (API3.SquareVectorDistance(myBuildings._entities[i].position(), entity.position()) > 30000)
+				if (API3.SquareVectorDistance(building.position(), entity.position()) > 30000)
 					continue;
 				this.targetList.push(targetId);
 				return true;
@@ -98,22 +94,24 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 
 	if (this.Config.personality.cooperative > 0.3)
 	{
-		var allyCC = gameState.getExclusiveAllyEntities().filter(API3.Filters.byClass("CivCentre"));
-		for (var i in allyCC._entities)
+		let ccEnts = gameState.updatingGlobalCollection("allCCs", API3.Filters.byClass("CivCentre"));
+		for (let cc of ccEnts.values())
 		{
-			if (this.Config.personality.cooperative < 0.6 && allyCC._entities[i].foundationProgress() !== undefined)
+			if (!gameState.isEntityExclusiveAlly(cc))
 				continue;
-			if (API3.SquareVectorDistance(allyCC._entities[i].position(), entity.position()) < 6000)
+			if (this.Config.personality.cooperative < 0.6 && cc.foundationProgress() !== undefined)
+				continue;
+			if (API3.SquareVectorDistance(cc.position(), entity.position()) < 6000)
 				return true;
 		}
 	}
 
 	var myBuildings = gameState.getOwnStructures();
-	for (var i in myBuildings._entities)
+	for (let building of myBuildings.values())
 	{
-		if (myBuildings._entities[i].foundationProgress() === 0)
+		if (building.foundationProgress() == 0)
 			continue;
-		if (API3.SquareVectorDistance(myBuildings._entities[i].position(), entity.position()) < 6000)
+		if (API3.SquareVectorDistance(building.position(), entity.position()) < 6000)
 			return true;
 	}
 
@@ -152,8 +150,6 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 
 m.DefenseManager.prototype.checkEnemyArmies = function(gameState, events)
 {
-	var self = this;
-
 	for (var o = 0; o < this.armies.length; ++o)
 	{
 		var army = this.armies[o];
@@ -162,11 +158,8 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState, events)
 		// this returns a list of IDs: the units that broke away from the army for being too far.
 		var breakaways = army.update(gameState);
 
-		for (var u in breakaways)
-		{
-			// assume dangerosity
-			this.makeIntoArmy(gameState,breakaways[u]);
-		}
+		for (let breakers of breakaways)
+			this.makeIntoArmy(gameState, breakers);		// assume dangerosity
 
 		if (army.getState(gameState) === 0)
 		{
@@ -210,19 +203,19 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState, events)
 
 		// army in neutral territory // TODO check smaller distance with all our buildings instead of only ccs with big distance
 		var stillDangerous = false;
-		if (this.Config.personality.cooperative > 0.3)
-			var bases = gameState.getAllyEntities().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
-		else
-			var bases = gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
-	 	for (var i in bases)
+		let bases = gameState.updatingGlobalCollection("allCCs", API3.Filters.byClass("CivCentre"));
+	 	for (let base of bases.values())
 		{
-			if (API3.SquareVectorDistance(bases[i].position(), army.foePosition) < 40000)
-			{
-				if(this.Config.debug > 1)
-					API3.warn("army in neutral territory, but still near one of our CC");
-				stillDangerous = true;
-				break;
-			}
+			if (!gameState.isEntityAlly(base))
+				continue;
+			if (this.Config.personality.cooperative < 0.3 && !gameState.isEntityOwn(base))
+				continue;
+			if (API3.SquareVectorDistance(base.position(), army.foePosition) > 40000)
+				continue;
+			if(this.Config.debug > 1)
+				API3.warn("army in neutral territory, but still near one of our CC");
+			stillDangerous = true;
+			break;
 		}
 		if (stillDangerous)
 			continue;
@@ -240,10 +233,9 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 	var armiesNeeding = [];
 	// Okay, let's add defenders
 	// TODO: this is dumb.
-	for (var i in this.armies)
+	for (let army of this.armies)
 	{
-		var army = this.armies[i];
-		var needsDef = army.needsDefenders(gameState);
+		let needsDef = army.needsDefenders(gameState);
 		if (needsDef === false)
 			continue;
 		
@@ -262,7 +254,9 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 			return;
 		if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
 			return;
-		if (ent.hasClass("Siege") || ent.hasClass("Support") || ent.attackTypes() === undefined)
+		if (ent.hasClass("Support") || ent.attackTypes() === undefined)
+			return;
+		if (ent.hasClass("Siege") && !ent.hasClass("Melee"))
 			return;
 		if (ent.hasClass("FishingBoat") || ent.hasClass("Trader"))
 			return;
@@ -314,7 +308,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 
 	if (armiesNeeding.length === 0)
 		return;
-	// If shortage of defenders, produce ranged infantry garrisoned in nearest civil centre
+	// If shortage of defenders, produce infantry garrisoned in nearest civil centre
 	var armiesPos = [];
 	for (var a = 0; a < armiesNeeding.length; ++a)
 		armiesPos.push(armiesNeeding[a]["army"].foePosition);
@@ -323,9 +317,9 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 
 // If our defense structures are attacked, garrison soldiers inside when possible
 // and if a support unit is attacked and has less than 45% health, garrison it inside the nearest cc
+// and if a ranged siege unit (not used for defense) is attacked, garrison it in the nearest fortress
 m.DefenseManager.prototype.checkEvents = function(gameState, events)
 {
-	var self = this;
 	var attackedEvents = events["Attacked"];
 	for (var evt of attackedEvents)
 	{
@@ -342,6 +336,19 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 			continue;
 		}
 
+		if (target.hasClass("Siege") && !target.hasClass("Melee") && !target.getMetadata(PlayerID, "transport")
+			&& target.getMetadata(PlayerID, "plan") !== -2 && target.getMetadata(PlayerID, "plan") !== -3)
+		{
+			if (target.getMetadata(PlayerID, "plan") !== undefined && target.getMetadata(PlayerID, "plan") !== -1)
+			{
+				var subrole = target.getMetadata(PlayerID, "subrole");
+				if (subrole && (subrole === "completing" || subrole === "walking" || subrole === "attacking")) 
+					continue;
+			}
+			this.garrisonSiegeUnit(gameState, target);
+			continue;
+		}
+
 		var attacker = gameState.getEntityById(evt.attacker);
 		if (!attacker || !attacker.position())
 			continue;
@@ -364,35 +371,68 @@ m.DefenseManager.prototype.garrisonRangedUnitsInside = function(gameState, targe
 		return;
 	var index = gameState.ai.accessibility.getAccessValue(target.position());
 	var garrisonManager = gameState.ai.HQ.garrisonManager;
-	gameState.getOwnUnits().filter(API3.Filters.byClassesAnd(["Infantry", "Ranged"])).filterNearest(target.position()).forEach(function(ent) {
+	var garrisonArrowClasses = target.getGarrisonArrowClasses();
+	var units = gameState.getOwnUnits().filter(function (ent) { return MatchesClassList(garrisonArrowClasses, ent.classes()); }).filterNearest(target.position());
+	for (let ent of units.values())
+	{
 		if (garrisonManager.numberOfGarrisonedUnits(target) >= target.garrisonMax())
-			return;
+			break;
 		if (!ent.position())
-			return;
+			continue;
 		if (ent.getMetadata(PlayerID, "transport") !== undefined)
-			return;
+			continue;
 		if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
-			return;
+			continue;
 		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") !== -1)
 		{
 			var subrole = ent.getMetadata(PlayerID, "subrole");
 			if (subrole && (subrole === "completing" || subrole === "walking" || subrole === "attacking")) 
-				return;
+				continue;
 		}
 		if (gameState.ai.accessibility.getAccessValue(ent.position()) !== index)
-			return;
+			continue;
 		garrisonManager.garrison(gameState, ent, target, "protection");
+	}
+};
+
+// garrison a attacked siege ranged unit inside the nearest fortress
+m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
+{
+	let distmin = Math.min();
+	let nearest = undefined;
+	let unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
+	let garrisonManager = gameState.ai.HQ.garrisonManager;
+	gameState.getAllyStructures().forEach(function(ent) {
+		if (!MatchesClassList(ent.garrisonableClasses(), unit.classes()))
+			return;
+		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax())
+			return;
+		var entAccess = ent.getMetadata(PlayerID, "access");
+		if (!entAccess)
+		{
+			entAccess = gameState.ai.accessibility.getAccessValue(ent.position());
+			ent.setMetadata(PlayerID, "access", entAccess);
+		}
+		if (entAccess !== unitAccess)
+			return;
+		var dist = API3.SquareVectorDistance(ent.position(), unit.position());
+		if (dist > distmin)
+			return;
+		distmin = dist;
+		nearest = ent;
 	});
+	if (nearest)
+		garrisonManager.garrison(gameState, unit, nearest, "protection");
 };
 
 // garrison a hurt unit inside the nearest healing structure
 m.DefenseManager.prototype.garrisonUnitForHealing = function(gameState, unit)
 {
-	var distmin = Math.min();
-	var nearest = undefined;
-	var unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
-	var garrisonManager = gameState.ai.HQ.garrisonManager;
-	gameState.getAllyEntities().filter(API3.Filters.byClass("Structure")).forEach(function(ent) {
+	let distmin = Math.min();
+	let nearest = undefined;
+	let unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
+	let garrisonManager = gameState.ai.HQ.garrisonManager;
+	gameState.getAllyStructures().forEach(function(ent) {
 		if (!ent.buffHeal())
 			return;
 		if (!MatchesClassList(ent.garrisonableClasses(), unit.classes()))
@@ -417,5 +457,33 @@ m.DefenseManager.prototype.garrisonUnitForHealing = function(gameState, unit)
 		garrisonManager.garrison(gameState, unit, nearest, "protection");
 };
 
+m.DefenseManager.prototype.Serialize = function()
+{
+	let properties = {
+		"targetList" : this.targetList,
+		"armyMergeSize": this.armyMergeSize
+	};
+
+	let armies = [];
+	for (var army of this.armies)
+		armies.push(army.Serialize());
+
+	return { "properties": properties, "armies": armies };
+};
+
+m.DefenseManager.prototype.Deserialize = function(gameState, data)
+{
+	for (let key in data.properties)
+		this[key] = data.properties[key];
+
+	this.armies = [];
+	for (let dataArmy of data.armies)
+	{
+		let army = new m.DefenseArmy(gameState, [], []);
+		army.Deserialize(dataArmy);
+		this.armies.push(army);
+	}
+};
+
 return m;
 }(PETRA);
