@@ -44,7 +44,7 @@ m.GarrisonManager.prototype.update = function(gameState, queues)
 			var ent = gameState.getEntityById(list[j]);
 			if (!ent)	// unit must have been killed while garrisoning
 				list.splice(j--, 1);    
-			else if (holder._entity.garrisoned.indexOf(list[j]) !== -1)   // unit is garrisoned
+			else if (holder.garrisoned().indexOf(list[j]) !== -1)   // unit is garrisoned
 			{
 				this.leaveGarrison(ent);
 				list.splice(j--, 1);
@@ -70,7 +70,7 @@ m.GarrisonManager.prototype.update = function(gameState, queues)
 				}
 				else
 				{
-					if (gameState.ai.HQ.Config.debug > 0)
+					if (gameState.ai.Config.debug > 0)
 					{
 						API3.warn("Petra garrison error: unit " + ent.id() + " (" + ent.genericName()
 							+ ") is expected to garrison in " + id + " (" + holder.genericName()
@@ -101,7 +101,7 @@ m.GarrisonManager.prototype.update = function(gameState, queues)
 				return false;
 			});
 
-			for (var entId of holder._entity.garrisoned)
+			for (var entId of holder.garrisoned())
 			{
 				var ent = gameState.getEntityById(entId);
 				if (ent.owner() === PlayerID && !this.keepGarrisoned(ent, holder, enemiesAround))
@@ -145,7 +145,7 @@ m.GarrisonManager.prototype.garrison = function(gameState, ent, holder, type)
 	this.registerHolder(gameState, holder);
 	this.holders[holder.id()].push(ent.id());
 
-	if (gameState.ai.HQ.Config.debug > 2)
+	if (gameState.ai.Config.debug > 2)
 	{
 		warn("garrison unit " + ent.genericName() + " in " + holder.genericName() + " with type " + type);
 		warn(" we try to garrison a unit with plan " + ent.getMetadata(PlayerID, "plan") + " and role " + ent.getMetadata(PlayerID, "role")
@@ -182,10 +182,10 @@ m.GarrisonManager.prototype.keepGarrisoned = function(ent, holder, enemiesAround
 			return false;
 		case 'trade':		// trader garrisoned in ship
 			return true;
-		case 'protection':	// hurt unit for healing or ranged infantry for defense
-			if (ent.isHurt() && holder.buffHeal())
+		case 'protection':	// hurt unit for healing or infantry for defense
+			if (ent.needsHeal() && holder.buffHeal())
 				return true;
-			if (enemiesAround && (ent.hasClass("Support") || (ent.hasClass("Ranged") && ent.hasClass("Infantry"))))
+			if (enemiesAround && (ent.hasClass("Support") || MatchesClassList(holder.getGarrisonArrowClasses(), ent.classes())))
 				return true;
 			return false;
 		default:
@@ -205,5 +205,15 @@ m.GarrisonManager.prototype.registerHolder = function(gameState, holder)
 	holder.setMetadata(PlayerID, "holderTimeUpdate", gameState.ai.elapsedTime);
 };
 
+m.GarrisonManager.prototype.Serialize = function()
+{
+	return { "holders": this.holders };
+};
+
+m.GarrisonManager.prototype.Deserialize = function(data)
+{
+	this.holders = data.holders;
+};
+
 return m;
 }(PETRA);
