@@ -15,12 +15,6 @@ var PETRA = function(m)
 m.NavalManager = function(Config)
 {
 	this.Config = Config;
-	// accessibility zones for which we have a dock.
-	// Connexion is described as [landindex] = [seaIndexes];
-	// technically they also exist for sea zones but I don't care.
-	this.landZoneDocked = [];	
-	// list of seas I have a dock on.
-	this.accessibleSeas = [];
 	
 	// ship subCollections. Also exist for land zones, idem, not caring.
 	this.seaShips = [];
@@ -43,15 +37,14 @@ m.NavalManager = function(Config)
 };
 
 // More initialisation for stuff that needs the gameState
-m.NavalManager.prototype.init = function(gameState, queues)
+m.NavalManager.prototype.init = function(gameState, deserializing)
 {
 	// finished docks
 	this.docks = gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClassesOr(["Dock", "Shipyard"]),
 		API3.Filters.not(API3.Filters.isFoundation())));
-	this.docks.allowQuickIter();
 	this.docks.registerUpdates();
 	
-	this.ships = gameState.getOwnUnits().filter(API3.Filters.byClass("Ship"));
+	this.ships = gameState.getOwnUnits().filter(API3.Filters.and(API3.Filters.byClass("Ship"), API3.Filters.not(API3.Filters.byMetadata(PlayerID, "role", "trader"))));
 	// note: those two can overlap (some transport ships are warships too and vice-versa).
 	this.transportShips = this.ships.filter(API3.Filters.and(API3.Filters.byCanGarrison(), API3.Filters.not(API3.Filters.byClass("FishingBoat"))));
 	this.warShips = this.ships.filter(API3.Filters.byClass("Warship"));
@@ -62,15 +55,27 @@ m.NavalManager.prototype.init = function(gameState, queues)
 	this.warShips.registerUpdates();
 	this.fishShips.registerUpdates();
 	
+	var fishes = gameState.getFishableSupplies();
+	var availableFishes = {};
+	for (let fish of fishes.values())
+	{
+		let sea = gameState.ai.accessibility.getAccessValue(fish.position(), true);
+		fish.setMetadata(PlayerID, "sea", sea);
+		if (availableFishes[sea])
+			availableFishes[sea] += fish.resourceSupplyAmount();
+		else
+			availableFishes[sea] = fish.resourceSupplyAmount();
+	}
+
 	for (var i = 0; i < gameState.ai.accessibility.regionSize.length; ++i)
 	{
-		if (gameState.ai.accessibility.regionType[i] !== "water")
+		if (!gameState.ai.HQ.navalRegions[i])
 		{
 			// push dummies
-			this.seaShips.push(new API3.EntityCollection(gameState.sharedScript));
-			this.seaTransportShips.push(new API3.EntityCollection(gameState.sharedScript));
-			this.seaWarShips.push(new API3.EntityCollection(gameState.sharedScript));
-			this.seaFishShips.push(new API3.EntityCollection(gameState.sharedScript));
+			this.seaShips.push(undefined);
+			this.seaTransportShips.push(undefined);
+			this.seaWarShips.push(undefined);
+			this.seaFishShips.push(undefined);
 			this.wantedTransportShips.push(0);
 			this.wantedWarShips.push(0);
 			this.wantedFishShips.push(0);
@@ -93,12 +98,13 @@ m.NavalManager.prototype.init = function(gameState, queues)
 			this.seaFishShips.push(collec);
 			this.wantedTransportShips.push(0);
 			this.wantedWarShips.push(0);
-			this.wantedFishShips.push(this.Config.Economy.targetNumFishers);
+			if (availableFishes[i] && availableFishes[i] > 1000)
+				this.wantedFishShips.push(this.Config.Economy.targetNumFishers);
+			else
+				this.wantedFishShips.push(0);
 			this.neededTransportShips.push(0);
 			this.neededWarShips.push(0);
 		}
-		
-		this.landZoneDocked.push([]);
 	}
 
 	// determination of the possible landing zones
@@ -168,9 +174,14 @@ m.NavalManager.prototype.init = function(gameState, queues)
 	for (var i in this.bNaval)
 		this.bNaval[i] = gameState.applyCiv(this.bNaval[i]);
 
-	// Assign our docks
-	var self = this;
-	this.docks.forEach(function(dock) { self.assignDock(gameState, dock); });
+	if (deserializing)
+		return;
+
+	// Assign our initial docks and ships
+	for (let ship of this.ships.values())
+		this.setShipIndex(gameState, ship);
+	for (let dock of this.docks.values())
+		this.setDockIndex(gameState, dock);
 };
 
 m.NavalManager.prototype.resetFishingBoats = function(gameState)
@@ -179,7 +190,13 @@ m.NavalManager.prototype.resetFishingBoats = function(gameState)
 		this.wantedFishShips[i] = 0;
 };
 
-m.NavalManager.prototype.assignDock = function(gameState, dock)
+m.NavalManager.prototype.setShipIndex = function(gameState, ship)
+{
+	let sea = gameState.ai.accessibility.getAccessValue(ship.position(), true);
+	ship.setMetadata(PlayerID, "sea", sea);
+};
+
+m.NavalManager.prototype.setDockIndex = function(gameState, dock)
 {
 	var land = dock.getMetadata(PlayerID, "access");
 	if (land === undefined)
@@ -193,11 +210,6 @@ m.NavalManager.prototype.assignDock = function(gameState, dock)
 		sea = this.getDockIndex(gameState, dock, true);
 		dock.setMetadata(PlayerID, "sea", sea);
 	}
-
-	if (this.landZoneDocked[land].indexOf(sea) === -1)
-		this.landZoneDocked[land].push(sea);
-	if (this.accessibleSeas.indexOf(sea) === -1)
-		this.accessibleSeas.push(sea);
 };
 
 // get the indices for our starting docks and those of our allies
@@ -224,17 +236,18 @@ m.NavalManager.prototype.getDockIndex = function(gameState, dock, onWater)
 	return index;
 };
 
+// get the list of seas around this region not connected by a dock
 m.NavalManager.prototype.getUnconnectedSeas = function(gameState, region)
 {
-	var seas = gameState.ai.accessibility.regionLinks[region]
-	if (seas.length === 0)
-		return [];
-
-	for (var i = 0; i < seas.length; ++i)
-	{
-		if (this.landZoneDocked[region].indexOf(seas[i]) !== -1)
+	var seas = gameState.ai.accessibility.regionLinks[region].slice();
+	var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
+	docks.forEach(function (dock) {
+		if (dock.getMetadata(PlayerID, "access") !== region)
+			return;
+		var i = seas.indexOf(dock.getMetadata(PlayerID, "sea"));
+		if (i !== -1)
 			seas.splice(i--,1);
-	}
+	});
 	return seas;
 };
 
@@ -248,7 +261,7 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 			continue;
 		var entity = gameState.getEntityById(evt.newentity);
 		if (entity && entity.hasClass("Dock") && entity.isOwn(PlayerID))
-			this.assignDock(gameState, entity);
+			this.setDockIndex(gameState, entity);
 	}
 
 	var evts = events["TrainingFinished"];
@@ -261,10 +274,7 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 			var entity = gameState.getEntityById(entId);
 			if (!entity || !entity.hasClass("Ship") || !entity.isOwn(PlayerID))
 				continue;
-			var pos = gameState.ai.accessibility.gamePosToMapPos(entity.position());
-			var index = pos[0] + pos[1]*gameState.ai.accessibility.width;
-			var sea = gameState.ai.accessibility.navalPassMap[index];
-			entity.setMetadata(PlayerID, "sea", sea);
+			this.setShipIndex(gameState, entity);
 		}
 	}
 
@@ -360,6 +370,7 @@ m.NavalManager.prototype.requireTransport = function(gameState, entity, startInd
 			API3.warn(">>>> transport plan aborted <<<<");
 		return false;
 	}
+	plan.init(gameState);
 	this.transportPlans.push(plan);
 	return true;
 };
@@ -376,6 +387,7 @@ m.NavalManager.prototype.splitTransport = function(gameState, plan)
 			API3.warn(">>>> split of transport plan aborted <<<<");
 		return false;
 	}
+	newplan.init(gameState);
 
 	var nbUnits = 0;
 	plan.units.forEach(function (ent) {
@@ -391,6 +403,37 @@ m.NavalManager.prototype.splitTransport = function(gameState, plan)
 	return (nbUnits !== 0);
 };
 
+/**
+ * create a transport from a garrisoned ship to a land location
+ * needed at start game when starting with a garrisoned ship
+ */
+m.NavalManager.prototype.createTransportIfNeeded = function(gameState, fromPos, toPos)
+{
+	let fromAccess = gameState.ai.accessibility.getAccessValue(fromPos);
+	if (fromAccess !== 1)
+		return;
+	let toAccess = gameState.ai.accessibility.getAccessValue(toPos);
+	if (toAccess < 2)
+		return;
+
+	for (let ship of this.ships.values())
+	{
+		if (!ship.isGarrisonHolder() || !ship.garrisoned().length)
+			continue;
+		if (ship.getMetadata(PlayerID, "transporter") !== undefined)
+			continue;
+		let units = [];
+		for (let entId of ship.garrisoned())
+			units.push(gameState.getEntityById(entId));
+		// TODO check that the garrisoned units have not another purpose
+		let plan = new m.TransportPlan(gameState, units, fromAccess, toAccess, toPos, ship);
+		if (plan.failed)
+			continue;
+		plan.init(gameState);
+		this.transportPlans.push(plan);
+	}
+};
+
 // set minimal number of needed ships when a new event (new base or new attack plan)
 m.NavalManager.prototype.setMinimalTransportShips = function(gameState, sea, number)
 {
@@ -440,7 +483,7 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 	// check if we have enough transport ships per region.
 	for (var sea = 0; sea < this.seaShips.length; ++sea)
 	{
-		if (this.accessibleSeas.indexOf(sea) === -1)
+		if (this.seaShips[sea] === undefined)
 			continue;
 		if (gameState.countOwnQueuedEntitiesWithMetadata("sea", sea) > 0)
 			continue;
@@ -480,23 +523,53 @@ m.NavalManager.prototype.assignShipsToPlans = function(gameState)
 m.NavalManager.prototype.moveApart = function(gameState)
 {
 	var self = this;
-	for (var sea = 0; sea < gameState.ai.accessibility.regionSize.length; ++sea)
-	{
-		this.seaShips[sea].forEach(function(ship) {
-			if (ship.getMetadata(PlayerID, "transporter") === undefined)
+
+	this.ships.forEach(function(ship) {
+		if (ship.hasClass("FishingBoat"))   // small ships should not be a problem
+			return;
+		var sea = ship.getMetadata(PlayerID, "sea");
+		if (ship.getMetadata(PlayerID, "transporter") === undefined)
+		{
+			if (ship.isIdle())	// do not stay idle near a dock to not disturb other ships
+			{
+				gameState.getOwnStructures().filter(API3.Filters.byClass("Dock")).forEach(function(dock) {
+					if (dock.getMetadata(PlayerID, "sea") !== sea)
+						return;
+					if (API3.SquareVectorDistance(ship.position(), dock.position()) > 2500)
+						return;
+					ship.moveApart(dock.position(), 50);
+				});
+			}
+			return;
+		}
+		// if transporter ship not idle, move away other ships which could block it
+		self.seaShips[sea].forEach(function(blockingShip) {
+			if (blockingShip === ship || !blockingShip.isIdle())
 				return;
-			self.seaShips[sea].forEach(function(blockingShip) {
-				if (blockingShip === ship || !blockingShip.isIdle())
-					return;
-				if (API3.SquareVectorDistance(ship.position(), blockingShip.position()) > 900)
-					return;
-				if (blockingShip.getMetadata(PlayerID, "transporter") === undefined)
-					blockingShip.moveApart(ship.position(), 12);
-				else
-					blockingShip.moveApart(ship.position(), 6);
-			});
+			if (API3.SquareVectorDistance(ship.position(), blockingShip.position()) > 900)
+				return;
+			if (blockingShip.getMetadata(PlayerID, "transporter") === undefined)
+				blockingShip.moveApart(ship.position(), 12);
+			else
+				blockingShip.moveApart(ship.position(), 6);
 		});
-	}
+	});
+
+	gameState.ai.HQ.tradeManager.traders.filter(API3.Filters.byClass("Ship")).forEach(function(ship) {
+		if (ship.getMetadata(PlayerID, "route") === undefined)
+			return;
+		var sea = ship.getMetadata(PlayerID, "sea");
+		self.seaShips[sea].forEach(function(blockingShip) {
+			if (blockingShip === ship || !blockingShip.isIdle())
+				return;
+			if (API3.SquareVectorDistance(ship.position(), blockingShip.position()) > 900)
+				return;
+			if (blockingShip.getMetadata(PlayerID, "transporter") === undefined)
+				blockingShip.moveApart(ship.position(), 12);
+			else
+				blockingShip.moveApart(ship.position(), 6);
+		});
+	});
 };
 
 m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
@@ -506,42 +579,57 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 
 	if (gameState.getPopulation() > this.Config.Economy.popForDock)
 	{
-		if (queues.economicBuilding.countQueuedUnitsWithClass("NavalMarket") === 0 &&
-			gameState.countEntitiesAndQueuedByType(gameState.applyCiv("structures/{civ}_dock"), true) === 0)
+		if (queues.dock.countQueuedUnitsWithClass("NavalMarket") === 0
+			&& gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClass("NavalMarket"), API3.Filters.isFoundation())).length === 0
+			&& gameState.ai.HQ.canBuild(gameState, "structures/{civ}_dock"))
 		{
-			if (gameState.ai.HQ.canBuild(gameState, "structures/{civ}_dock"))
+			var dockStarted = false;
+			for (var base of gameState.ai.HQ.baseManagers)
 			{
-				var remaining = this.getUnconnectedSeas(gameState, gameState.ai.HQ.baseManagers[1].accessIndex);
+				if (dockStarted)
+					break;
+				if (!base.anchor || base.constructing)
+					continue;
+				var remaining = this.getUnconnectedSeas(gameState, base.accessIndex);
 				for (var sea of remaining)
 				{
-					if (gameState.ai.HQ.navalRegions.indexOf(sea) !== -1)
-					{
-						queues.economicBuilding.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "sea": sea }));
-						break;
-					}
+					if (!gameState.ai.HQ.navalRegions[sea])
+						continue;
+					queues.dock.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "land": [base.accessIndex], "sea": sea }));
+					dockStarted = true;
+					break;
 				}
 			}
 		}
 	}
 
-	if (gameState.currentPhase() > 1 && gameState.getPopulation() > this.Config.Economy.popForTown + 15
-		&& queues.militaryBuilding.length() === 0 && this.bNaval.length !== 0)
-	{
-		var nNaval = 0;
-		for (var naval of this.bNaval)
-			nNaval += gameState.countEntitiesAndQueuedByType(naval, true);
+	if (gameState.currentPhase() < 2 || gameState.getPopulation() < this.Config.Economy.popForTown + 15
+		|| queues.militaryBuilding.length() !== 0 || this.bNaval.length === 0)
+		return;
+	var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
+	if (!docks.length)
+		return;
+	var nNaval = 0;
+	for (var naval of this.bNaval)
+		nNaval += gameState.countEntitiesAndQueuedByType(naval, true);
 
-		var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock")).toEntityArray();
-		if (docks.length && (nNaval === 0 || (nNaval < this.bNaval.length && gameState.getPopulation() > 120)))
+	if (nNaval === 0 || (nNaval < this.bNaval.length && gameState.getPopulation() > 120))
+	{
+		for (var naval of this.bNaval)
 		{
-			for (var naval of this.bNaval)
+			if (gameState.countEntitiesAndQueuedByType(naval, true) < 1 && gameState.ai.HQ.canBuild(gameState, naval))
 			{
-				if (gameState.countEntitiesAndQueuedByType(naval, true) < 1 && gameState.ai.HQ.canBuild(gameState, naval))
+				var land = [];
+				for (var base of gameState.ai.HQ.baseManagers)
 				{
-					var sea = docks[0].getMetadata(PlayerID, "sea");
-					queues.militaryBuilding.addItem(new m.ConstructionPlan(gameState, naval, { "sea": sea }));
-					break;
+					if (!base.anchor)
+						continue;
+					if (land.indexOf(base.accessIndex) === -1)
+						land.push(base.accessIndex);
 				}
+				var sea = docks.toEntityArray()[0].getMetadata(PlayerID, "sea");
+				queues.militaryBuilding.addItem(new m.ConstructionPlan(gameState, naval, { "land": land, "sea": sea }));
+				break;
 			}
 		}
 	}
@@ -555,6 +643,8 @@ m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 		var trainables = ent.trainableEntities();
 		for (var trainable of trainables)
 		{
+			if (gameState.isDisabledTemplates(trainable))
+				continue;
 			var template = gameState.getTemplate(trainable);
 			if (template.hasClass("Ship") && trainableShips.indexOf(trainable) === -1)
 				trainableShips.push(trainable);
@@ -571,15 +661,8 @@ m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 		if (!template.available(gameState))
 			continue;
 
-		var aboveLimit = false;
-		for (var limitedClass in limits)
-		{
-			if (!template.hasClass(limitedClass) || current[limitedClass] < limits[limitedClass])
-				continue;
-			aboveLimit = true;
-			break;
-		}
-		if (aboveLimit)
+		var category = template.trainingCategory();
+		if (category && limits[category] && current[category] >= limits[category])
 			continue;
 
 		var arrows = +(template.getDefaultArrow() || 0);
@@ -614,10 +697,8 @@ m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 m.NavalManager.prototype.update = function(gameState, queues, events)
 {
 	Engine.ProfileStart("Naval Manager update");
-	
-	this.checkEvents(gameState, queues, events);
 
-	this.buildNavalStructures(gameState, queues);
+	this.checkEvents(gameState, queues, events);
 
 	// close previous transport plans if finished
 	for (var i = 0; i < this.transportPlans.length; ++i)
@@ -631,17 +712,56 @@ m.NavalManager.prototype.update = function(gameState, queues, events)
 			this.transportPlans.splice(i--, 1);
 		}
 	}
-
 	// assign free ships to plans which need them
 	this.assignShipsToPlans(gameState);
-	// and require for more ships if needed
-	this.checkLevels(gameState, queues);
-	this.maintainFleet(gameState, queues);
+
+	// and require for more ships/structures if needed
+	if (gameState.ai.playedTurn % 3 === 0)
+	{
+		this.checkLevels(gameState, queues);
+		this.maintainFleet(gameState, queues);
+		this.buildNavalStructures(gameState, queues);
+	}
 	// let inactive ships move apart from active ones (waiting for a better pathfinder)
 	this.moveApart(gameState);
 
 	Engine.ProfileStop();
 };
 
+m.NavalManager.prototype.Serialize = function()
+{
+	let properties = {
+		"wantedTransportShips": this.wantedTransportShips,
+		"wantedWarShips": this.wantedWarShips,
+		"wantedFishShips": this.wantedFishShips,
+		"neededTransportShips": this.neededTransportShips,
+		"neededWarShips": this.neededWarShips,
+		"landingZones": this.landingZones
+	};
+
+	let transports = {};
+	for (let plan in this.transportPlans)
+		transports[plan] = this.transportPlans[plan].Serialize();
+
+	return { "properties": properties, "transports": transports };
+};
+
+m.NavalManager.prototype.Deserialize = function(gameState, data)
+{
+	for (let key in data.properties)
+		this[key] = data.properties[key];
+
+	this.transportPlans = [];
+	for (let i in data.transports)
+	{
+		let dataPlan = data.transports[i];
+		let plan = new m.TransportPlan(gameState, [], dataPlan.startIndex, dataPlan.endIndex, dataPlan.endPos);
+		plan.Deserialize(dataPlan);
+		plan.init(gameState);
+		this.transportPlans.push(plan);
+	}
+};
+
+
 return m;
 }(PETRA);
