@@ -49,5 +49,41 @@ m.ResearchPlan.prototype.start = function(gameState)
 	this.onStart(gameState);
 };
 
+m.ResearchPlan.prototype.Serialize = function()
+{
+	let prop = {
+		"category": this.category,
+		"type": this.type,
+		"ID": this.ID,
+		"metadata": this.metadata,
+		"cost": this.cost.Serialize(),
+		"number": this.number,
+		"rush": this.rush,
+		"lastIsGo": this.lastIsGo,
+	};
+
+	let func = {
+		"isGo": uneval(this.isGo),
+		"onGo": uneval(this.onGo),
+		"onNotGo": uneval(this.onNotGo),
+		"onStart": uneval(this.onStart)
+	};
+
+	return { "prop": prop, "func": func };
+};
+
+m.ResearchPlan.prototype.Deserialize = function(gameState, data)
+{
+	for (let key in data.prop)
+		this[key] = data.prop[key];
+
+	let cost = new API3.Resources();
+	cost.Deserialize(data.prop.cost);
+	this.cost = cost;
+
+	for (let fun in data.func)
+		this[fun] = eval(data.func[fun]);
+};
+
 return m;
 }(PETRA);
