@@ -10,7 +10,7 @@ m.TrainingPlan = function(gameState, type, metadata, number, maxMerge)
 	}
 
 	this.category = "unit";
-	this.cost = new API3.Resources(this.template.cost(), this.template._template.Cost.Population);
+	this.cost = new API3.Resources(this.template.cost(), +this.template._template.Cost.Population);
 	
 	this.number = number !== undefined ? number : 1;
 	this.maxMerge = maxMerge !== undefined ? maxMerge : 5;
@@ -65,10 +65,11 @@ m.TrainingPlan.prototype.start = function(gameState)
 		var wantedIndex = undefined;
 		if (this.metadata && this.metadata.index)
 			wantedIndex = this.metadata.index;
+		var workerUnit = (this.metadata && this.metadata.role && this.metadata.role == "worker");
 		var supportUnit = this.template.hasClass("Support");
 		trainers.sort(function(a, b) {
-			var aa = a.trainingQueueTime();
-			var bb = b.trainingQueueTime();
+			let aa = a.trainingQueueTime();
+			let bb = b.trainingQueueTime();
 			if (a.hasClass("Civic") && !supportUnit)
 				aa += 10;
 			if (b.hasClass("Civic") && !supportUnit)
@@ -80,18 +81,28 @@ m.TrainingPlan.prototype.start = function(gameState)
 				if (gameState.ai.HQ.isDangerousLocation(b.position()))
 					bb += 50;
 			}
+			let aBase = a.getMetadata(PlayerID, "base");
+			let bBase = b.getMetadata(PlayerID, "base");
 			if (wantedIndex)
 			{
-				var aBase = a.getMetadata(PlayerID, "base");
-				if (!aBase || gameState.ai.HQ.baseManagers[aBase].accessIndex !== wantedIndex)
+				if (!aBase || gameState.ai.HQ.getBaseByID(aBase).accessIndex != wantedIndex)
 					aa += 30;
-				var bBase = b.getMetadata(PlayerID, "base");
-				if (!bBase || gameState.ai.HQ.baseManagers[bBase].accessIndex !== wantedIndex)
+				if (!bBase || gameState.ai.HQ.getBaseByID(bBase).accessIndex != wantedIndex)
 					bb += 30;
 			}
+			// then, if worker, small preference for bases with less workers
+			if (workerUnit && aBase && bBase && aBase != bBase)
+			{
+				let apop = gameState.ai.HQ.getBaseByID(aBase).workers.length;
+				let bpop = gameState.ai.HQ.getBaseByID(bBase).workers.length;
+				if (apop > bpop)
+					aa++;
+				else if (bpop > apop)
+					bb++;
+			}
 			return (aa - bb);
 		});
-		if (this.metadata && this.metadata.base !== undefined && this.metadata.base === 0)
+		if (this.metadata && this.metadata.base !== undefined && this.metadata.base == 0)
 			this.metadata.base = trainers[0].getMetadata(PlayerID, "base");
 		trainers[0].train(this.type, this.number, this.metadata, this.promotedTypes(gameState));
 	}
@@ -128,5 +139,31 @@ m.TrainingPlan.prototype.promotedTypes = function(gameState)
 	return types;
 };
 
+m.TrainingPlan.prototype.Serialize = function()
+{
+	let prop = {
+		"category": this.category,
+		"type": this.type,
+		"ID": this.ID,
+		"metadata": this.metadata,
+		"cost": this.cost.Serialize(),
+		"number": this.number,
+		"maxMerge": this.maxMerge,
+		"lastIsGo": this.lastIsGo
+	};
+
+	return { "prop": prop };
+};
+
+m.TrainingPlan.prototype.Deserialize = function(gameState, data)
+{
+	for (let key in data.prop)
+		this[key] = data.prop[key];
+
+	let cost = new API3.Resources();
+	cost.Deserialize(data.prop.cost);
+	this.cost = cost;
+};
+
 return m;
 }(PETRA);
