@@ -23,10 +23,10 @@ var PETRA = function(m)
    transporter = this.ID
 */
 
-m.TransportPlan = function(gameState, units, startIndex, endIndex, endPos)
+m.TransportPlan = function(gameState, units, startIndex, endIndex, endPos, ship)
 {
-	this.ID = m.playerGlobals[PlayerID].uniqueIDTPlans++;
-	this.debug = gameState.ai.HQ.Config.debug;
+	this.ID = gameState.ai.uniqueIDs.transports++;
+	this.debug = gameState.ai.Config.debug;
 	this.flotilla = false;   // when false, only one ship per transport ... not yet tested when true
 
 	this.endPos = endPos;
@@ -34,43 +34,59 @@ m.TransportPlan = function(gameState, units, startIndex, endIndex, endPos)
 	this.startIndex = startIndex;
 	// TODO only cases with land-sea-land are allowed for the moment
 	// we could also have land-sea-land-sea-land
-	this.sea = gameState.ai.HQ.getSeaIndex(gameState, startIndex, endIndex);
-	if (!this.sea)
+	if (startIndex === 1)
 	{
-		this.failed = true;
-		if (this.debug > 1)
-			API3.warn("transport plan with bad path: startIndex " + startIndex + " endIndex " + endIndex);
-		return false;
+		// special transport from already garrisoned ship
+		if (!ship)
+		{
+			this.failed = true;
+			return false;
+		}
+		this.sea = ship.getMetadata(PlayerID, "sea");
+		ship.setMetadata(PlayerID, "transporter", this.ID);
+		for (let ent of units)
+			ent.setMetadata(PlayerID, "onBoard", "onBoard");
+	}
+	else
+	{
+		this.sea = gameState.ai.HQ.getSeaIndex(gameState, startIndex, endIndex);
+		if (!this.sea)
+		{
+			this.failed = true;
+			if (this.debug > 1)
+				API3.warn("transport plan with bad path: startIndex " + startIndex + " endIndex " + endIndex);
+			return false;
+		}
 	}
 
-	this.units = gameState.getOwnUnits().filter(API3.Filters.byMetadata(PlayerID, "transport", this.ID));
-	this.units.registerUpdates();
-
-	for (var ent of units)
+	for (let ent of units)
 	{
 		ent.setMetadata(PlayerID, "transport", this.ID);
 		ent.setMetadata(PlayerID, "endPos", endPos);
-		this.units.updateEnt(ent);
 	}
 
 	if (this.debug > 1)
 		API3.warn("Starting a new transport plan with ID " +  this.ID + " to index " + endIndex
 			+ " with units length " + units.length);
 
-	this.ships = gameState.ai.HQ.navalManager.ships.filter(API3.Filters.byMetadata(PlayerID, "transporter", this.ID));
-	// note: those two can overlap (some transport ships are warships too and vice-versa).
-	this.transportShips = gameState.ai.HQ.navalManager.transportShips.filter(API3.Filters.byMetadata(PlayerID, "transporter", this.ID));
-	
-	this.ships.registerUpdates();
-	this.transportShips.registerUpdates();
-
 	this.state = "boarding";
 	this.boardingPos = {};
-	this.needTransportShips = true;
+	this.needTransportShips = (ship === undefined);
 	this.nTry = {};
 	return true;
 };
 
+m.TransportPlan.prototype.init = function(gameState)
+{
+	this.units = gameState.getOwnUnits().filter(API3.Filters.byMetadata(PlayerID, "transport", this.ID));
+	this.ships = gameState.ai.HQ.navalManager.ships.filter(API3.Filters.byMetadata(PlayerID, "transporter", this.ID));
+	this.transportShips = gameState.ai.HQ.navalManager.transportShips.filter(API3.Filters.byMetadata(PlayerID, "transporter", this.ID));
+	
+	this.units.registerUpdates();
+	this.ships.registerUpdates();
+	this.transportShips.registerUpdates();
+};
+
 // count available slots
 m.TransportPlan.prototype.countFreeSlots = function()
 {
@@ -82,7 +98,7 @@ m.TransportPlan.prototype.countFreeSlots = function()
 
 m.TransportPlan.prototype.countFreeSlotsOnShip = function(ship)
 {
-	var occupied = ship._entity.garrisoned.length
+	var occupied = ship.garrisoned().length
 		+ this.units.filter(API3.Filters.byMetadata(PlayerID, "onBoard", ship.id())).length;
 	return (ship.garrisonMax() - occupied);
 };
@@ -131,24 +147,20 @@ m.TransportPlan.prototype.assignShip = function(gameState)
 		pos = ent.position();
 	});
 	// and choose the nearest available ship from this unit
-	for each (var ship in gameState.ai.HQ.navalManager.seaTransportShips[this.sea]._entities)
-	{
+	gameState.ai.HQ.navalManager.seaTransportShips[this.sea].forEach(function (ship) {
 		if (ship.getMetadata(PlayerID, "transporter"))
-			continue;
+			return;
 		if (pos)
 		{
 			var dist = API3.SquareVectorDistance(pos, ship.position());
 			if (dist > distmin)
-				continue;
+				return;
 			distmin = dist;
 			nearest = ship;
 		}
-		else
-		{
+		else if (!nearest)
 			nearest = ship;
-			break;
-		}
-	}
+	});
 	if (!nearest)
 		return false;
 
@@ -169,39 +181,40 @@ m.TransportPlan.prototype.addUnit = function(unit, endPos)
 
 m.TransportPlan.prototype.releaseAll = function()
 {
-	this.ships.forEach(function (ent) { ent.setMetadata(PlayerID, "transporter", undefined); });
+	this.ships.forEach(function (ship) {
+		ship.setMetadata(PlayerID, "transporter", undefined);
+		if (ship.getMetadata(PlayerID, "role") === "switchToTrader")
+			ship.setMetadata(PlayerID, "role", "trader");
+	});
 	this.units.forEach(function (ent) {
 		ent.setMetadata(PlayerID, "endPos", undefined);
 		ent.setMetadata(PlayerID, "onBoard", undefined);
 		ent.setMetadata(PlayerID, "transport", undefined);
 // TODO if the index of the endPos of the entity is !== , require again another transport (we could need land-sea-land-sea-land)
 	});
-};
-
-m.TransportPlan.prototype.releaseAllShips = function()
-{
-	this.ships.forEach(function (ent) { ent.setMetadata(PlayerID, "transporter", undefined) });
+	this.transportShips.unregister();
+	this.ships.unregister();
+	this.units.unregister();
 };
 
 m.TransportPlan.prototype.cancelTransport = function(gameState)
 {
 	var ent = this.units.toEntityArray()[0];
-	var base = gameState.ai.HQ.baseManagers[ent.getMetadata(PlayerID, "base")];
+	var base = gameState.ai.HQ.getBaseByID(ent.getMetadata(PlayerID, "base"));
 	if (!base.anchor || !base.anchor.position())
 	{
-		for (var i in gameState.ai.HQ.baseManagers)
+		for (let newbase of gameState.ai.HQ.baseManagers)
 		{
-			base = gameState.ai.HQ.baseManagers[i];
-			if (base.anchor && base.anchor.position())
-			{
-				ent.setMetadata(PlayerID, "base", +i);
-				break;
-			}
+			if (!newbase.anchor || !newbase.anchor.position())
+				continue;
+			ent.setMetadata(PlayerID, "base", newbase.ID);
+			base = newbase;
+			break;
 		}
 		if (!base.anchor || !base.anchor.position())
 			return false;
 		this.units.forEach(function (ent) {
-			ent.setMetadata(PlayerID, "base", base);
+			ent.setMetadata(PlayerID, "base", base.ID);
 		});
 	}
 	this.endIndex = this.startIndex;
@@ -244,7 +257,7 @@ m.TransportPlan.prototype.onBoarding = function(gameState)
 				var ship = gameState.getEntityById(shipId);
 				if (!self.boardingPos[shipId])
 				{
-					self.boardingPos[shipId] = self.getBoardingPos(gameState, self.startIndex, self.sea, ent.position(), false);
+					self.boardingPos[shipId] = self.getBoardingPos(gameState, ship, self.startIndex, self.sea, ent.position(), false);
 					ship.move(self.boardingPos[shipId][0], self.boardingPos[shipId][1]);
 					ship.setMetadata(PlayerID, "timeGarrison", time);
 				}
@@ -274,7 +287,7 @@ m.TransportPlan.prototype.onBoarding = function(gameState)
 						self.nTry[shipId] = 0;
 						if (self.debug > 1)
 							API3.warn("ship " + shipId + " new attempt for a landing point ");
-						self.boardingPos[shipId] = self.getBoardingPos(gameState, self.startIndex, self.sea, undefined, false);
+						self.boardingPos[shipId] = self.getBoardingPos(gameState, ship, self.startIndex, self.sea, undefined, false);
 					}
 					ship.move(self.boardingPos[shipId][0], self.boardingPos[shipId][1]);
 					ship.setMetadata(PlayerID, "timeGarrison", time);				
@@ -320,7 +333,7 @@ m.TransportPlan.prototype.onBoarding = function(gameState)
 
 	this.ships.forEach(function (ship) {
 		self.boardingPos[ship.id()] = undefined;
-		self.boardingPos[ship.id()] = self.getBoardingPos(gameState, self.endIndex, self.sea, self.endPos, true);
+		self.boardingPos[ship.id()] = self.getBoardingPos(gameState, ship, self.endIndex, self.sea, self.endPos, true);
 		ship.move(self.boardingPos[ship.id()][0], self.boardingPos[ship.id()][1]);
 	});
 	this.state = "sailing";
@@ -334,7 +347,7 @@ m.TransportPlan.prototype.isOnBoard = function(ent)
 {
 	var ret = false;
 	this.transportShips.forEach(function (ship) {
-		if (ret || ship._entity.garrisoned.indexOf(ent.id()) === -1)
+		if (ret || ship.garrisoned().indexOf(ent.id()) === -1)
 			return;
 		ret = true;
 		ent.setMetadata(PlayerID, "onBoard", "onBoard");
@@ -343,7 +356,7 @@ m.TransportPlan.prototype.isOnBoard = function(ent)
 };
 
 // when avoidEnnemy is true, we try to not board/unboard in ennemy territory
-m.TransportPlan.prototype.getBoardingPos = function(gameState, landIndex, seaIndex, destination, avoidEnnemy)
+m.TransportPlan.prototype.getBoardingPos = function(gameState, ship, landIndex, seaIndex, destination, avoidEnnemy)
 {
 	if (!gameState.ai.HQ.navalManager.landingZones[landIndex][seaIndex])
 	{
@@ -351,7 +364,7 @@ m.TransportPlan.prototype.getBoardingPos = function(gameState, landIndex, seaInd
 		return destination;
 	}
 
-	var startPos = 	this.transportShips.getCentrePosition();
+	var startPos = ship.position();
 	var distmin = Math.min();
 	var posmin = destination;
 	var width = gameState.getMap().width;
@@ -372,7 +385,7 @@ m.TransportPlan.prototype.getBoardingPos = function(gameState, landIndex, seaInd
 		// require a small distance between all ships of the transport plan to avoid path finder problems
 		// this is also used when the ship is blocked and we want to find a new boarding point
 		for (var shipId in this.boardingPos)
-			if (this.boardingPos[shipId] !== undefined && API3.SquareVectorDistance(this.boardingPos[shipId], pos) < 100)
+			if (this.boardingPos[shipId] !== undefined && API3.SquareVectorDistance(this.boardingPos[shipId], pos) < 225)
 				dist += 1000000;
 		if (dist > distmin)
 			continue;
@@ -399,13 +412,13 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 				ship.moveApart(recov.entPos, 15);
 			continue;
 		}
-		if (gameState.ai.HQ.Config.debug > 1)
+		if (this.debug > 1)
 			API3.warn(">>> transport " + this.ID + " reloading failed ... <<<");
 		// destroy the unit if inaccessible otherwise leave it there
 		var index = gameState.ai.accessibility.getAccessValue(ent.position());
-		if (gameState.ai.HQ.allowedRegions[index])
+		if (gameState.ai.HQ.landRegions[index])
 		{
-			if (gameState.ai.HQ.Config.debug > 1)
+			if (this.debug > 1)
 				API3.warn(" recovered entity kept " + ent.id());
 			this.resetUnit(gameState, ent);
 			// TODO we should not destroy it, but now the unit could still be reloaded on the next turn
@@ -414,7 +427,7 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 		}
 		else
 		{
-			if (gameState.ai.HQ.Config.debug > 1)
+			if (this.debug > 1)
 				API3.warn("recovered entity destroyed " + ent.id());
 			this.resetUnit(gameState, ent);
 			ent.destroy();
@@ -434,7 +447,7 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 			var ship = gameState.getEntityById(ent.getMetadata(PlayerID, "onBoard"));
 			if (ship)
 			{
-				if (ship._entity.garrisoned.indexOf(entId) !== -1)
+				if (ship.garrisoned().indexOf(entId) !== -1)
 					ent.setMetadata(PlayerID, "onBoard", "onBoard");
 				else
 				{
@@ -453,7 +466,7 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 		else if (gameState.ai.accessibility.getAccessValue(ent.position()) !== this.endIndex)
 		{
 			// unit unloaded on a wrong region - try to regarrison it and move a bit the ship
-			if (gameState.ai.HQ.Config.debug > 1)
+			if (this.debug > 1)
 				API3.warn(">>> unit unloaded on a wrong region ! try to garrison it again <<<");
 			var ship = gameState.getEntityById(ent.getMetadata(PlayerID, "onBoard"));
 			if (ship && !this.canceled)
@@ -465,7 +478,7 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 			}
 			else
 			{
-				if (gameState.ai.HQ.Config.debug > 1)
+				if (this.debug > 1)
 					API3.warn("no way ... we destroy it");
 				this.resetUnit(gameState, ent);
 				ent.destroy();
@@ -480,7 +493,7 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 	}
 	for (var shipId in shipsToMove)
 	{
-		this.boardingPos[shipId] = this.getBoardingPos(gameState, this.endIndex, this.sea, this.endPos, true);
+		this.boardingPos[shipId] = this.getBoardingPos(gameState, shipsToMove[shipId], this.endIndex, this.sea, this.endPos, true);
 		shipsToMove[shipId].move(this.boardingPos[shipId][0], this.boardingPos[shipId][1]);
 	}
 	this.unloaded = [];
@@ -489,7 +502,7 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 	{
 		this.ships.forEach(function (ship) {
 			self.boardingPos[ship.id()] = undefined;
-			self.boardingPos[ship.id()] = self.getBoardingPos(gameState, self.endIndex, self.sea, self.endPos, true);
+			self.boardingPos[ship.id()] = self.getBoardingPos(gameState, ship, self.endIndex, self.sea, self.endPos, true);
 			ship.move(self.boardingPos[ship.id()][0], self.boardingPos[ship.id()][1]);
 		});
 		this.canceled = undefined;
@@ -502,7 +515,7 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 		var shipId = ship.id();
 		var dist = API3.SquareVectorDistance(ship.position(), self.boardingPos[shipId]);
 		var remaining = 0;
-		for (var entId of ship._entity.garrisoned)
+		for (var entId of ship.garrisoned())
 		{
 			var ent = gameState.getEntityById(entId);
 			if (!ent.getMetadata(PlayerID, "transport"))
@@ -525,6 +538,8 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 		{
 			ship.moveApart(self.boardingPos[shipId], 15);
 			ship.setMetadata(PlayerID, "transporter", undefined);
+			if (ship.getMetadata(PlayerID, "role") === "switchToTrader")
+				ship.setMetadata(PlayerID, "role", "trader");
 			return;
 		}
 
@@ -539,7 +554,7 @@ m.TransportPlan.prototype.onSailing = function(gameState)
 				self.nTry[shipId] = 0;
 				if (self.debug > 1)
 					API3.warn(shipId + " new attempt for a landing point ");
-				self.boardingPos[shipId] = self.getBoardingPos(gameState, self.endIndex, self.sea, undefined, true);
+				self.boardingPos[shipId] = self.getBoardingPos(gameState, ship, self.endIndex, self.sea, undefined, true);
 			}
 			ship.move(self.boardingPos[shipId][0], self.boardingPos[shipId][1]);
 		}
@@ -554,9 +569,9 @@ m.TransportPlan.prototype.resetUnit = function(gameState, ent)
 	// if from an army or attack, remove it
 	if (ent.getMetadata(PlayerID, "plan") >= 0)
 	{
-		var plan = gameState.ai.HQ.attackManager.getPlan(ent.getMetadata(PlayerID, "plan"));
-		if (plan)
-			plan.removeUnit(ent);
+		var attackPlan = gameState.ai.HQ.attackManager.getPlan(ent.getMetadata(PlayerID, "plan"));
+		if (attackPlan)
+			attackPlan.removeUnit(ent, true);
 	}
 	if (ent.getMetadata(PlayerID, "PartOfArmy"))
 	{
@@ -564,7 +579,34 @@ m.TransportPlan.prototype.resetUnit = function(gameState, ent)
 		if (army)
 			army.removeOwn(gameState, ent.id());
 	}
-}
+};
+
+m.TransportPlan.prototype.Serialize = function()
+{
+	return {
+		"ID": this.ID,
+		"flotilla": this.flotilla,
+		"endPos": this.endPos,
+		"endIndex": this.endIndex,
+		"startIndex": this.startIndex,
+		"sea": this.sea,
+		"state": this.state,
+		"boardingPos": this.boardingPos,
+		"needTransportShips": this.needTransportShips,
+		"nTry": this.nTry,
+		"canceled": this.canceled,
+		"unloaded": this.unloaded,
+		"recovered": this.recovered
+	};
+};
+
+m.TransportPlan.prototype.Deserialize = function(data)
+{
+	for (let key in data)
+		this[key] = data[key];
+
+	this.failed = false;
+};
 
 return m;
 }(PETRA);
