@@ -5,32 +5,50 @@ var PETRA = function(m)
  * This class makes a worker do as instructed by the economy manager
  */
 
-m.Worker = function(ent)
+m.Worker = function(base)
 {
-	this.ent = ent;
-	this.baseID = 0;
+	this.ent = undefined;
+	this.base = base
+	this.baseID = base.ID;
 };
 
-m.Worker.prototype.update = function(baseManager, gameState)
+m.Worker.prototype.update = function(ent, gameState)
 {
-	if (!this.ent.position() || this.ent.getMetadata(PlayerID, "plan") === -2 || this.ent.getMetadata(PlayerID, "plan") === -3)
+	if (!ent.position() || ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
 		return;
 
 	// If we are waiting for a transport or we are sailing, just wait
-	if (this.ent.getMetadata(PlayerID, "transport") !== undefined)
+	if (ent.getMetadata(PlayerID, "transport") !== undefined)
 		return;
 
+	this.ent = ent;
+	// base 0 for unassigned entities has no accessIndex, so take the one from the entity
+	if (this.baseID === gameState.ai.HQ.baseManagers[0].ID)
+		this.accessIndex = gameState.ai.accessibility.getAccessValue(ent.position());
+	else
+		this.accessIndex = this.base.accessIndex;
+
+	var subrole = this.ent.getMetadata(PlayerID, "subrole");
+	if (!subrole)	// subrole may-be undefined after a transport, garrisoning, army, ...
+	{
+		this.ent.setMetadata(PlayerID, "subrole", "idle");
+		return;
+	}
+
+	// Check for inaccessible targets (in RMS maps, we quite often have chicken or bushes inside obstruction of other entities).
+	if (ent.unitAIState() === "INDIVIDUAL.GATHER.APPROACHING" || ent.unitAIState() === "INDIVIDUAL.COMBAT.APPROACHING")
+		if (this.isInaccessibleSupply(gameState) && ((subrole === "hunter" && !this.startHunting(gameState))
+				|| (subrole === "gatherer" && !this.startGathering(gameState))))
+			this.ent.stopMoving();
+
 	// If we're fighting or hunting, let's not start gathering
-	if (this.ent.unitAIState().split(".")[1] === "COMBAT")
+	if (ent.unitAIState().split(".")[1] === "COMBAT")
 		return;
 
 	// Okay so we have a few tasks.
 	// If we're gathering, we'll check that we haven't run idle.
 	// And we'll also check that we're gathering a resource we want to gather.
 
-	this.baseID = baseManager.ID;
-	var subrole = this.ent.getMetadata(PlayerID, "subrole");
-
 	if (subrole === "gatherer")
 	{
 		if (this.ent.isIdle())
@@ -41,17 +59,17 @@ m.Worker.prototype.update = function(baseManager, gameState)
 			if (!this.ent.resourceCarrying() || this.ent.resourceCarrying().length === 0 ||
 					this.ent.resourceCarrying()[0].type === this.ent.getMetadata(PlayerID, "gather-type"))
 			{
-				this.startGathering(gameState, baseManager);
+				this.startGathering(gameState);
 			}
-			else if (!this.returnResources(gameState))     // try to deposit resources
+			else if (!m.returnResources(this.ent, gameState))     // try to deposit resources
 			{
 				// no dropsite, abandon old resources and start gathering new ones
-				this.startGathering(gameState, baseManager);
+				this.startGathering(gameState);
 			}
 		}
 		else if (this.ent.unitAIState().split(".")[1] === "GATHER")
 		{
-			// we're already gathering. But let's check from time to time if there is nothing better
+			// we're already gathering. But let's check if there is nothing better
 			// in case UnitAI did something bad
 			if (this.ent.unitAIOrderData().length)
 			{
@@ -65,12 +83,12 @@ m.Worker.prototype.update = function(baseManager, gameState)
 					if ((nbGatherers > 0 && supply.resourceSupplyAmount()/nbGatherers < 40))
 					{
 						m.RemoveTCGatherer(gameState, supplyId);
-						this.startGathering(gameState, baseManager);
+						this.startGathering(gameState);
 					}
 					else
 					{
 						var gatherType = this.ent.getMetadata(PlayerID, "gather-type");
-						var nearby = baseManager.dropsiteSupplies[gatherType]["nearby"];
+						var nearby = this.base.dropsiteSupplies[gatherType]["nearby"];
 						var isNearby = nearby.some(function(sup) {
 							if (sup.id === supplyId)
 								return true;
@@ -81,7 +99,7 @@ m.Worker.prototype.update = function(baseManager, gameState)
 						else
 						{
 							m.RemoveTCGatherer(gameState, supplyId);
-							this.startGathering(gameState, baseManager);
+							this.startGathering(gameState);
 						}
 					}
 				}
@@ -102,14 +120,14 @@ m.Worker.prototype.update = function(baseManager, gameState)
 					dropsite.setMetadata(PlayerID, "access", goalAccess);
 				}
 				if (access !== goalAccess)
-					this.returnResources(gameState);
+					m.returnResources(this.ent, gameState);
 			}
 		}
 	}
 	else if (subrole === "builder")
-	{	
+	{
 		if (this.ent.unitAIState().split(".")[1] === "REPAIR")
-			return;
+				return;
 		// okay so apparently we aren't working.
 		// Unless we've been explicitely told to keep our role, make us idle.
 		var target = gameState.getEntityById(this.ent.getMetadata(PlayerID, "target-foundation"));
@@ -119,7 +137,7 @@ m.Worker.prototype.update = function(baseManager, gameState)
 			this.ent.setMetadata(PlayerID, "target-foundation", undefined);
 			// If worker elephant, move away to avoid being trapped in between constructions
 			if (this.ent.hasClass("Elephant"))
-				this.moveAway(baseManager, gameState);
+				this.moveAway(gameState);
 		}
 		else
 		{
@@ -145,9 +163,8 @@ m.Worker.prototype.update = function(baseManager, gameState)
 			{
 				// nothing to hunt around. Try another region if any
 				var nowhereToHunt = true;
-				for (var i in gameState.ai.HQ.baseManagers)
+				for (var base of gameState.ai.HQ.baseManagers)
 				{
-					var base = gameState.ai.HQ.baseManagers[i];
 					if (!base.anchor || !base.anchor.position())
 						continue;
 					var basePos = base.anchor.position();
@@ -167,7 +184,7 @@ m.Worker.prototype.update = function(baseManager, gameState)
 					this.ent.setMetadata(PlayerID, "lastHuntSearch", gameState.ai.elapsedTime);
 			}
 		}
-		else if (gameState.ai.playedTurn % 10 === 0)  // Perform some checks from time to time
+		else	// Perform some sanity checks
 		{
 			if (this.ent.unitAIState().split(".")[1] === "GATHER"
 				|| this.ent.unitAIState().split(".")[1] === "RETURNRESOURCE")
@@ -190,7 +207,7 @@ m.Worker.prototype.update = function(baseManager, gameState)
 							dropsite.setMetadata(PlayerID, "access", goalAccess);
 						}
 						if (access !== goalAccess)
-							this.returnResources(gameState);
+							m.returnResources(this.ent, gameState);
 					}
 				}
 			}
@@ -209,46 +226,14 @@ m.Worker.prototype.update = function(baseManager, gameState)
 	}
 };
 
-m.Worker.prototype.startGathering = function(gameState, baseManager)
+m.Worker.prototype.startGathering = function(gameState)
 {
 	var self = this;
 	var access = gameState.ai.accessibility.getAccessValue(this.ent.position());
 
 	// First look for possible treasure if any
-	var treasureFound = undefined;
-	var distmin = Math.min();
-	gameState.ai.HQ.treasures.forEach(function (treasure) {
-		var treasureAccess = treasure.getMetadata(PlayerID, "access");
-		if (!treasureAccess)
-		{
-			treasureAccess = gameState.ai.accessibility.getAccessValue(treasure.position());
-			treasure.setMetadata(PlayerID, "access", treasureAccess);
-		}
-		if (treasureAccess !== access)
-			return;
-		var territoryOwner = gameState.ai.HQ.territoryMap.getOwner(treasure.position());
-		if (territoryOwner !== 0 && !gameState.isPlayerAlly(territoryOwner))
-			return;
-		var lastGathered = treasure.getMetadata(PlayerID, "lastGathered");
-		// let some time for the previous gatherer to reach the treasure
-		if (lastGathered && gameState.ai.elapsedTime - lastGathered < 20)
-			return;
-		var dist = API3.SquareVectorDistance(self.ent.position(), treasure.position());
-		if (territoryOwner !== PlayerID && dist > 14000)  //  AI has no LOS, so restrict it a bit
-			return;
-		if (dist > distmin)
-			return;
-		distmin = dist;
-		treasureFound = treasure;
-	});
-	if (treasureFound)
-	{
-		treasureFound.setMetadata(PlayerID, "lastGathered", gameState.ai.elapsedTime);
-		this.ent.gather(treasureFound);
-		m.AddTCGatherer(gameState, treasureFound.id());
-		this.ent.setMetadata(PlayerID, "supply", treasureFound.id());
+	if (this.gatherTreasure(gameState))
 		return true;
-	}
 
 	var resource = this.ent.getMetadata(PlayerID, "gather-type");
 
@@ -258,7 +243,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 	
 	var findSupply = function(ent, supplies) {
 		var ret = false;
-		for (var i = 0; i < supplies.length; ++i)
+		for (let i = 0; i < supplies.length; ++i)
 		{
 			// exhausted resource, remove it from this list
 			if (!supplies[i].ent || !gameState.getEntityById(supplies[i].id))
@@ -268,6 +253,9 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 			}
 			if (m.IsSupplyFull(gameState, supplies[i].ent) === true)
 				continue;
+			let inaccessibleTime = supplies[i].ent.getMetadata(PlayerID, "inaccessibleTime");
+			if (inaccessibleTime && gameState.ai.elapsedTime < inaccessibleTime)
+				continue;
 			// check if available resource is worth one additionnal gatherer (except for farms)
 			var nbGatherers = supplies[i].ent.resourceSupplyGatherers().length
 				+ m.GetTCGatherer(gameState, supplies[i].id);
@@ -290,9 +278,9 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 	var supply;
 
 	// first look in our own base if accessible from our present position
-	if (baseManager.accessIndex === access)
+	if (this.accessIndex === access)
 	{
-		if ((supply = findSupply(this.ent, baseManager.dropsiteSupplies[resource]["nearby"])))
+		if ((supply = findSupply(this.ent, this.base.dropsiteSupplies[resource]["nearby"])))
 		{
 			this.ent.gather(supply);
 			return true;
@@ -311,7 +299,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 				return true;
 			}
 		}
-		if ((supply = findSupply(this.ent, baseManager.dropsiteSupplies[resource]["medium"])))
+		if ((supply = findSupply(this.ent, this.base.dropsiteSupplies[resource]["medium"])))
 		{
 			this.ent.gather(supply);
 			return true;
@@ -319,7 +307,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 	}
 	// So if we're here we have checked our whole base for a proper resource (or it was not accessible)
 	// --> check other bases directly accessible
-	for each (var base in gameState.ai.HQ.baseManagers)
+	for (var base of gameState.ai.HQ.baseManagers)
 	{
 		if (base.ID === this.baseID)
 			continue;
@@ -334,7 +322,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 	}
 	if (resource === "food")	// --> for food, try to gather from fields if any, otherwise build one if any
 	{
-		for each (var base in gameState.ai.HQ.baseManagers)
+		for (var base of gameState.ai.HQ.baseManagers)
 		{
 			if (base.ID === this.baseID)
 				continue;
@@ -354,7 +342,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 			}
 		}
 	}
-	for each (var base in gameState.ai.HQ.baseManagers)
+	for (var base of gameState.ai.HQ.baseManagers)
 	{
 		if (base.ID === this.baseID)
 			continue;
@@ -388,7 +376,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 		return true;
 
 	// Still nothing ... try bases which need a transport
-	for each (var base in gameState.ai.HQ.baseManagers)
+	for (var base of gameState.ai.HQ.baseManagers)
 	{
 		if (base.accessIndex === access)
 			continue;
@@ -402,7 +390,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 	}
 	if (resource === "food")	// --> for food, try to gather from fields if any, otherwise build one if any
 	{
-		for each (var base in gameState.ai.HQ.baseManagers)
+		for (var base of gameState.ai.HQ.baseManagers)
 		{
 			if (base.accessIndex === access)
 				continue;
@@ -422,7 +410,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 			}
 		}
 	}
-	for each (var base in gameState.ai.HQ.baseManagers)
+	for (var base of gameState.ai.HQ.baseManagers)
 	{
 		if (base.accessIndex === access)
 			continue;
@@ -436,7 +424,6 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 	}
 	// Okay so we haven't found any appropriate dropsite anywhere.
 	// Try to help building one if any non-accessible foundation available
-	var foundations = gameState.getOwnFoundations().toEntityArray();
 	var shouldBuild = foundations.some(function(foundation) {
 		if (!foundation || foundation.getMetadata(PlayerID, "access") === access)
 			return false;
@@ -454,15 +441,15 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 		return true;
 
 	// Still nothing, we look now for faraway resources, first in the accessible ones, then in the others
-	if (baseManager.accessIndex === access)
+	if (this.accessIndex === access)
 	{
-		if ((supply = findSupply(this.ent, baseManager.dropsiteSupplies[resource]["faraway"])))
+		if ((supply = findSupply(this.ent, this.base.dropsiteSupplies[resource]["faraway"])))
 		{
 			this.ent.gather(supply);
 			return true;
 		}
 	}
-	for each (var base in gameState.ai.HQ.baseManagers)
+	for (var base of gameState.ai.HQ.baseManagers)
 	{
 		if (base.ID === this.baseID)
 			continue;
@@ -475,7 +462,7 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 			return true;
 		}
 	}
-	for each (var base in gameState.ai.HQ.baseManagers)
+	for (var base of gameState.ai.HQ.baseManagers)
 	{
 		if (base.accessIndex === access)
 			continue;
@@ -490,44 +477,22 @@ m.Worker.prototype.startGathering = function(gameState, baseManager)
 
 	// If we are here, we have nothing left to gather ... certainly no more resources of this type
 	gameState.ai.HQ.lastFailedGather[resource] = gameState.ai.elapsedTime;
-	if (gameState.ai.HQ.Config.debug > 2)
+	if (gameState.ai.Config.debug > 2)
 		warn(" >>>>> worker with gather-type " + resource + " with nothing to gather ");
 	this.ent.setMetadata(PlayerID, "subrole", "idle");
 	return false;
 };
 
-// Makes the worker deposit the currently carried resources at the closest accessible dropsite
-m.Worker.prototype.returnResources = function(gameState)
-{
-	if (!this.ent.resourceCarrying() || this.ent.resourceCarrying().length === 0 || !this.ent.position())
-		return false;
-
-	var resource = this.ent.resourceCarrying()[0].type;
-	var self = this;
-
-	var closestDropsite = undefined;
-	var distmin = Math.min();
-	var access = gameState.ai.accessibility.getAccessValue(this.ent.position());
-	gameState.getOwnDropsites(resource).forEach(function(dropsite) {
-		if (!dropsite.position() || dropsite.getMetadata(PlayerID, "access") !== access)
-			return;
-		var dist = API3.SquareVectorDistance(self.ent.position(), dropsite.position());
-		if (dist > distmin)
-			return;
-		distmin = dist;
-		closestDropsite = dropsite;
-	});
-	
-	if (!closestDropsite)
-		return false;	
-	this.ent.returnResources(closestDropsite);
-	return true;
-};
-
-// if position is given, we only check if we could hunt from this position but do nothing
-// otherwise the position of the entity is taken, and if something is found, we directly start the hunt
+/**
+ * if position is given, we only check if we could hunt from this position but do nothing
+ * otherwise the position of the entity is taken, and if something is found, we directly start the hunt
+ */
 m.Worker.prototype.startHunting = function(gameState, position)
 {
+	// First look for possible treasure if any
+	if (!position && this.gatherTreasure(gameState))
+		return true;
+
 	var resources = gameState.getHuntableSupplies();	
 	if (resources.length === 0)
 		return false;
@@ -562,8 +527,9 @@ m.Worker.prototype.startHunting = function(gameState, position)
 	{
 		if (!supply.position())
 			return;
-		
-		if (supply.getMetadata(PlayerID, "inaccessible") === true)
+
+		let inaccessibleTime = supply.getMetadata(PlayerID, "inaccessibleTime");
+		if (inaccessibleTime && gameState.ai.elapsedTime < inaccessibleTime)
 			return;
 
 		if (m.IsSupplyFull(gameState, supply) === true)
@@ -592,23 +558,6 @@ m.Worker.prototype.startHunting = function(gameState, position)
 		if (!isCavalry && dist > 25000)
 			return;
 
-		// some simple accessibility check: if they're in an inaccessible square, we won't gather from them.
-		// (happen only at start of the game, as animals should not be able to walk to an inaccessible area)
-		// TODO as the animal can move, check again from time to time
-		if (supply.getMetadata(PlayerID, "inaccessible") === undefined)
-		{
-			var fakeMap = new API3.Map(gameState.sharedScript, gameState.getMap().data);
-			var mapPos = fakeMap.gamePosToMapPos(supply.position());
-			var id = mapPos[0] + fakeMap.width*mapPos[1];
-			if (gameState.sharedScript.passabilityClasses["pathfinderObstruction"] & gameState.getMap().data[id])
-			{
-				supply.setMetadata(PlayerID, "inaccessible", true)
-				return;
-			}
-			else
-				supply.setMetadata(PlayerID, "inaccessible", false)
-		}
-
 		// Avoid ennemy territory
 		var territoryOwner = gameState.ai.HQ.territoryMap.getOwner(supply.position());
 		if (territoryOwner != 0 && !gameState.isPlayerAlly(territoryOwner))  // player is its own ally
@@ -734,21 +683,21 @@ m.Worker.prototype.startFishing = function(gameState)
 
 m.Worker.prototype.gatherNearestField = function(gameState, baseID)
 {
-	var self = this;
 	var ownFields = gameState.getOwnEntitiesByType(gameState.applyCiv("structures/{civ}_field"), true).filter(API3.Filters.byMetadata(PlayerID, "base", baseID));
 	var bestFarmEnt = false;
 	var bestFarmDist = 10000000;
 
-	ownFields.forEach(function (field) {
+	for (var field of ownFields.values())
+	{
 		if (m.IsSupplyFull(gameState, field) === true)
-			return;
-		var dist = API3.SquareVectorDistance(field.position(), self.ent.position());
+			continue;
+		var dist = API3.SquareVectorDistance(field.position(), this.ent.position());
 		if (dist < bestFarmDist)
 		{
 			bestFarmEnt = field;
 			bestFarmDist = dist;
 		}
-	});
+	}
 	if (bestFarmEnt)
 	{
 		m.AddTCGatherer(gameState, bestFarmEnt.id());
@@ -763,38 +712,83 @@ m.Worker.prototype.gatherNearestField = function(gameState, baseID)
  */
 m.Worker.prototype.buildAnyField = function(gameState, baseID)
 {
-	var self = this;
-	var foundations = gameState.getOwnFoundations();
-	var baseFoundations = foundations.filter(API3.Filters.byMetadata(PlayerID, "base", baseID));
-	
+	var baseFoundations = gameState.getOwnFoundations().filter(API3.Filters.byMetadata(PlayerID, "base", baseID));	
 	var maxGatherers = gameState.getTemplate(gameState.applyCiv("structures/{civ}_field")).maxGatherers();
-
 	var bestFarmEnt = false;
 	var bestFarmDist = 10000000;
-	baseFoundations.forEach(function (found) {
-		if (found.hasClass("Field")) {
-			var current = found.getBuildersNb();
-			if (current === undefined || current >= maxGatherers)
-				return;
-			var dist = API3.SquareVectorDistance(found.position(), self.ent.position());
-			if (dist < bestFarmDist)
-			{
-				bestFarmEnt = found;
-				bestFarmDist = dist;
-			}
-		}
-	});
+	var pos = this.ent.position();
+	for (var found of baseFoundations.values())
+	{
+		if (!found.hasClass("Field"))
+			continue;
+		var current = found.getBuildersNb();
+		if (current === undefined || current >= maxGatherers)
+			continue;
+		var dist = API3.SquareVectorDistance(found.position(), pos);
+		if (dist > bestFarmDist)
+			continue;
+		bestFarmEnt = found;
+		bestFarmDist = dist;
+	}
 	return bestFarmEnt;
 };
 
-// Workers elephant should move away from the buildings they've built to avoid being trapped in between constructions
-// For the time being, we move towards the nearest gatherer (providing him a dropsite)
-m.Worker.prototype.moveAway = function(baseManager, gameState){
-	var gatherers = baseManager.workersBySubrole(gameState, "gatherer").toEntityArray();
+/**
+ * Look for some treasure to gather
+ */
+m.Worker.prototype.gatherTreasure = function(gameState)
+{
+	var rates = this.ent.resourceGatherRates();
+	if (!rates || !rates["treasure"] || rates["treasure"] <= 0)
+		return false;
+	var treasureFound = undefined;
+	var distmin = Math.min();
+	var access = gameState.ai.accessibility.getAccessValue(this.ent.position());
+	for (var treasure of gameState.ai.HQ.treasures.values())
+	{
+		// let some time for the previous gatherer to reach the treasure befor trying again
+		var lastGathered = treasure.getMetadata(PlayerID, "lastGathered");
+		if (lastGathered && gameState.ai.elapsedTime - lastGathered < 20)
+			continue;
+		var treasureAccess = treasure.getMetadata(PlayerID, "access");
+		if (!treasureAccess)
+		{
+			treasureAccess = gameState.ai.accessibility.getAccessValue(treasure.position());
+			treasure.setMetadata(PlayerID, "access", treasureAccess);
+		}
+		if (treasureAccess !== access)
+			continue;
+		var territoryOwner = gameState.ai.HQ.territoryMap.getOwner(treasure.position());
+		if (territoryOwner !== 0 && !gameState.isPlayerAlly(territoryOwner))
+			continue;
+		var dist = API3.SquareVectorDistance(this.ent.position(), treasure.position());
+		if (territoryOwner !== PlayerID && dist > 14000)  //  AI has no LOS, so restrict it a bit
+			continue;
+		if (dist > distmin)
+			continue;
+		distmin = dist;
+		treasureFound = treasure;
+	}
+	if (!treasureFound)
+		return false;
+	treasureFound.setMetadata(PlayerID, "lastGathered", gameState.ai.elapsedTime);
+	this.ent.gather(treasureFound);
+	m.AddTCGatherer(gameState, treasureFound.id());
+	this.ent.setMetadata(PlayerID, "supply", treasureFound.id());
+	return true;
+};
+
+/**
+ * Workers elephant should move away from the buildings they've built to avoid being trapped in between constructions
+ * For the time being, we move towards the nearest gatherer (providing him a dropsite)
+ */
+m.Worker.prototype.moveAway = function(gameState)
+{
+	var gatherers = this.base.workersBySubrole(gameState, "gatherer");
 	var pos = this.ent.position();
 	var dist = Math.min();
 	var destination = pos;
-	for (var gatherer of gatherers)
+	for (var gatherer of gatherers.values())
 	{
 		if (!gatherer.position() || gatherer.getMetadata(PlayerID, "transport") !== undefined)
 			continue;
@@ -809,5 +803,33 @@ m.Worker.prototype.moveAway = function(baseManager, gameState){
 	this.ent.move(destination[0], destination[1]);
 };
 
+// Check accessibility of the target when in approach (in RMS maps, we quite often have chicken or bushes
+// inside obstruction of other entities). The resource will be flagged as inaccessible during 10 mn (in case
+// it will be cleared later).
+m.Worker.prototype.isInaccessibleSupply = function(gameState)
+{
+	if (!this.ent.unitAIOrderData()[0] || !this.ent.unitAIOrderData()[0]["target"])
+		return false;
+	let approachingTime = this.ent.getMetadata(PlayerID, "approachingTime");
+	if (!approachingTime || gameState.ai.elapsedTime - approachingTime > 5)
+	{
+		let presentPos = this.ent.position();
+		let approachingPos = this.ent.getMetadata(PlayerID, "approachingPos");
+		if (!approachingPos || approachingPos[0] != presentPos[0] || approachingPos[1] != presentPos[1])
+		{
+			this.ent.setMetadata(PlayerID, "approachingTime", gameState.ai.elapsedTime);
+			this.ent.setMetadata(PlayerID, "approachingPos", presentPos);
+		}
+		else if (gameState.ai.elapsedTime - approachingTime > 15)
+		{
+			let targetId = this.ent.unitAIOrderData()[0]["target"];
+			let target = gameState.getEntityById(targetId);
+			if (target)
+				target.setMetadata(PlayerID, "inaccessibleTime", gameState.ai.elapsedTime + 600);
+			return true;
+		}
+	}
+	return false;
+}
 return m;
-}(PETRA);
+}(PETRA);
\ No newline at end of file
