@@ -16,7 +16,8 @@ AIInterface.prototype.EventNames = [
 	"EntityRenamed",
 	"OwnershipChanged",
 	"Garrison",
-	"UnGarrison"
+	"UnGarrison",
+	"TributeExchanged"
 ];
 
 AIInterface.prototype.Init = function()
@@ -39,7 +40,7 @@ AIInterface.prototype.GetNonEntityRepresentation = function()
 	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	
 	// Return the same game state as the GUI uses
-	var state = cmpGuiInterface.GetExtendedSimulationState(-1);
+	var state = cmpGuiInterface.GetSimulationState(-1);
 	
 	// Add some extra AI-specific data
 	// add custom events and reset them for the next turn
@@ -134,6 +135,11 @@ AIInterface.prototype.OnGlobalEntityRenamed = function(msg)
 		this.events["EntityRenamed"].push(msg);
 };
 
+AIInterface.prototype.OnGlobalTributeExchanged = function(msg)
+{
+	this.events["TributeExchanged"].push(msg);
+};
+
 // When a new technology is researched, check which templates it affects,
 // and send the updated values to the AI.
 // this relies on the fact that any "value" in a technology can only ever change
