@@ -1,6 +1,6 @@
 function Attack() {}
 
-var bonusesSchema = 
+Attack.prototype.bonusesSchema = 
 	"<optional>" +
 		"<element name='Bonuses'>" +
 			"<zeroOrMore>" +
@@ -18,7 +18,7 @@ var bonusesSchema =
 		"</element>" +
 	"</optional>";
 
-var preferredClassesSchema =
+Attack.prototype.preferredClassesSchema =
 	"<optional>" +
 		"<element name='PreferredClasses' a:help='Space delimited list of classes preferred for attacking. If an entity has any of theses classes, it is preferred. The classes are in decending order of preference'>" +
 			"<attribute name='datatype'>" +
@@ -28,7 +28,7 @@ var preferredClassesSchema =
 		"</element>" +
 	"</optional>";
 
-var restrictedClassesSchema =
+Attack.prototype.restrictedClassesSchema =
 	"<optional>" +
 		"<element name='RestrictedClasses' a:help='Space delimited list of classes that cannot be attacked by this entity. If target entity has any of these classes, it cannot be attacked'>" +
 			"<attribute name='datatype'>" +
@@ -112,9 +112,9 @@ Attack.prototype.Schema =
 				"<element name='RepeatTime' a:help='Time between attacks (in milliseconds). The attack animation will be stretched to match this time'>" + // TODO: it shouldn't be stretched
 					"<data type='positiveInteger'/>" +
 				"</element>" +
-				bonusesSchema +
-				preferredClassesSchema +
-				restrictedClassesSchema +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
 			"</interleave>" +
 		"</element>" +
 	"</optional>" +
@@ -139,9 +139,9 @@ Attack.prototype.Schema =
 					"<ref name='nonNegativeDecimal'/>" +
 				"</element>" +
 				"<element name='Spread' a:help='Radius over which missiles will tend to land (when shooting to the maximum range). Roughly 2/3 will land inside this radius (in metres). Spread is linearly diminished as the target gets closer.'><ref name='nonNegativeDecimal'/></element>" +
-				bonusesSchema +
-				preferredClassesSchema +
-				restrictedClassesSchema +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
 				"<optional>" +
 					"<element name='Splash'>" +
 						"<interleave>" +
@@ -151,7 +151,7 @@ Attack.prototype.Schema =
 							"<element name='Hack' a:help='Hack damage strength'><ref name='nonNegativeDecimal'/></element>" +
 							"<element name='Pierce' a:help='Pierce damage strength'><ref name='nonNegativeDecimal'/></element>" +
 							"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
-							bonusesSchema +
+							Attack.prototype.bonusesSchema +
 						"</interleave>" +
 					"</element>" +
 				"</optional>" +
@@ -166,9 +166,9 @@ Attack.prototype.Schema =
 				"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
 				"<element name='MaxRange'><ref name='nonNegativeDecimal'/></element>" + // TODO: how do these work?
 				"<element name='MinRange'><ref name='nonNegativeDecimal'/></element>" +
-				bonusesSchema +
-				preferredClassesSchema +
-				restrictedClassesSchema +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
 			"</interleave>" +
 		"</element>" +
 	"</optional>" +
@@ -179,9 +179,9 @@ Attack.prototype.Schema =
 				"<element name='Pierce' a:help='Pierce damage strength'><ref name='nonNegativeDecimal'/></element>" +
 				"<element name='Crush' a:help='Crush damage strength'><ref name='nonNegativeDecimal'/></element>" +
 				"<element name='MaxRange'><ref name='nonNegativeDecimal'/></element>" + // TODO: how do these work?
-				bonusesSchema +
-				preferredClassesSchema +
-				restrictedClassesSchema +
+				Attack.prototype.bonusesSchema +
+				Attack.prototype.preferredClassesSchema +
+				Attack.prototype.restrictedClassesSchema +
 			"</interleave>" +
 		"</element>" +
 	"</optional>";
@@ -227,6 +227,16 @@ Attack.prototype.CanAttack = function(target)
 	if (cmpFormation)
 		return true;
 
+	var cmpThisPosition = Engine.QueryInterface(this.entity, IID_Position);
+	var cmpTargetPosition = Engine.QueryInterface(target, IID_Position);
+	if (!cmpThisPosition || !cmpTargetPosition || !cmpThisPosition.IsInWorld() || !cmpTargetPosition.IsInWorld())
+		return false;
+
+	// Check if the relative height difference is larger than the attack range
+	// If the relative height is bigger, it means they will never be able to
+	// reach each other, no matter how close they come.
+	var heightDiff = Math.abs(cmpThisPosition.GetHeightOffset() - cmpTargetPosition.GetHeightOffset());
+
 	const cmpIdentity = Engine.QueryInterface(target, IID_Identity);
 	if (!cmpIdentity) 
 		return undefined;
@@ -235,6 +245,9 @@ Attack.prototype.CanAttack = function(target)
 
 	for each (var type in this.GetAttackTypes())
 	{
+		if (heightDiff > this.GetRange(type).max)
+			continue;
+
 		var canAttack = true;
 		var restrictedClasses = this.GetRestrictedClasses(type);
 
@@ -267,15 +280,14 @@ Attack.prototype.GetPreference = function(target)
 	const targetClasses = cmpIdentity.GetClassesList();
 
 	var minPref = null;
-	for each (var type in this.GetAttackTypes())
+	for (var type of this.GetAttackTypes())
 	{
-		for each (var targetClass in targetClasses)
+		var preferredClasses = this.GetPreferredClasses(type);
+		for (var targetClass of targetClasses)
 		{
-			var pref = this.GetPreferredClasses(type).indexOf(targetClass);
+			var pref = preferredClasses.indexOf(targetClass);
 			if (pref != -1 && (minPref === null || minPref > pref))
-			{
 				minPref = pref;
-			}
 		}
 	}
 	return minPref;
@@ -435,6 +447,8 @@ Attack.prototype.PerformAttack = function(type, target)
 	// If this is a ranged attack, then launch a projectile
 	if (type == "Ranged")
 	{
+		var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
+		var turnLength = cmpTimer.GetLatestTurnLength()/1000;
 		// In the future this could be extended:
 		//  * Obstacles like trees could reduce the probability of the target being hit
 		//  * Obstacles like walls should block projectiles entirely
@@ -460,7 +474,7 @@ Attack.prototype.PerformAttack = function(type, target)
 		var relativePosition = Vector3D.sub(targetPosition, selfPosition);
 		var previousTargetPosition = Engine.QueryInterface(target, IID_Position).GetPreviousPosition();
 
-		var targetVelocity = Vector3D.sub(targetPosition, previousTargetPosition).div(this.turnLength);
+		var targetVelocity = Vector3D.sub(targetPosition, previousTargetPosition).div(turnLength);
 		// the component of the targets velocity radially away from the archer
 		var radialSpeed = relativePosition.dot(targetVelocity) / relativePosition.length();
 
@@ -499,7 +513,7 @@ Attack.prototype.PerformAttack = function(type, target)
 		var cmpProjectileManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_ProjectileManager);
 		var id = cmpProjectileManager.LaunchProjectileAtPoint(this.entity, realTargetPosition, horizSpeed, gravity);
 
-		var playerId = Engine.QueryInterface(this.entity, IID_Ownership).GetOwner()
+		var playerId = Engine.QueryInterface(this.entity, IID_Ownership).GetOwner();
 		var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
  		cmpTimer.SetTimeout(this.entity, IID_Attack, "MissileHit", timeToTarget*1000, {"type": type, "target": target, "position": realTargetPosition, "direction": missileDirection, "projectileId": id, "playerId":playerId});
 	}
@@ -513,15 +527,17 @@ Attack.prototype.PerformAttack = function(type, target)
 
 Attack.prototype.InterpolatedLocation = function(ent, lateness)
 {
+	var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
+	var turnLength = cmpTimer.GetLatestTurnLength()/1000;
 	var cmpTargetPosition = Engine.QueryInterface(ent, IID_Position);
 	if (!cmpTargetPosition || !cmpTargetPosition.IsInWorld()) // TODO: handle dead target properly
 		return undefined;
 	var curPos = cmpTargetPosition.GetPosition();
 	var prevPos = cmpTargetPosition.GetPreviousPosition();
 	lateness /= 1000;
-	return new Vector3D((curPos.x * (this.turnLength - lateness) + prevPos.x * lateness) / this.turnLength,
+	return new Vector3D((curPos.x * (turnLength - lateness) + prevPos.x * lateness) / turnLength,
 			0,
-	        (curPos.z * (this.turnLength - lateness) + prevPos.z * lateness) / this.turnLength);
+	        (curPos.z * (turnLength - lateness) + prevPos.z * lateness) / turnLength);
 };
 
 // Tests whether it point is inside of ent's footprint
@@ -541,7 +557,7 @@ Attack.prototype.testCollision = function(ent, point, lateness)
 	if (targetShape.type === 'circle')
 	{
 		// Use VectorDistanceSquared and square targetShape.radius to avoid square roots.
-		return (point.horizDistanceTo(targetPosition) < (targetShape.radius * targetShape.radius));
+		return (targetPosition.horizDistanceTo(point) < (targetShape.radius * targetShape.radius));
 	}
 	else
 	{
@@ -559,7 +575,7 @@ Attack.prototype.MissileHit = function(data, lateness)
 	var targetPosition = this.InterpolatedLocation(data.target, lateness);
 	if (!targetPosition)
 		return;
-	
+
 	if (this.template.Ranged.Splash) // splash damage, do this first in case the direct hit kills the target
 	{
 		var friendlyFire = this.template.Ranged.Splash.FriendlyFire;
@@ -567,7 +583,7 @@ Attack.prototype.MissileHit = function(data, lateness)
 		var splashShape = this.template.Ranged.Splash.Shape;
 		var playersToDamage;
 		// If friendlyFire isn't enabled, get all player enemies to pass to "Damage.CauseSplashDamage".
-		if (friendlyFire == false)
+		if (friendlyFire == "false")
 		{
 			var cmpPlayer = Engine.QueryInterface(Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetPlayerByID(data.playerId), IID_Player)
 			playersToDamage = cmpPlayer.GetEnemies();
@@ -592,7 +608,7 @@ Attack.prototype.MissileHit = function(data, lateness)
 	{
 		// If we didn't hit the main target look for nearby units
 		var cmpPlayer = Engine.QueryInterface(Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetPlayerByID(data.playerId), IID_Player)
-		var ents = Damage.EntitiesNearPoint(data.position, data.position.horizDistanceTo(targetPosition) * 2, cmpPlayer.GetEnemies());
+		var ents = Damage.EntitiesNearPoint(Vector2D.from3D(data.position), targetPosition.horizDistanceTo(data.position) * 2, cmpPlayer.GetEnemies());
 
 		for (var i = 0; i < ents.length; i++)
 		{
@@ -608,9 +624,4 @@ Attack.prototype.MissileHit = function(data, lateness)
 	}
 };
 
-Attack.prototype.OnUpdate = function(msg)
-{
-	this.turnLength = msg.turnLength;
-}
-
 Engine.RegisterComponentType(IID_Attack, "Attack", Attack);
