@@ -5,37 +5,65 @@ const VIS_VISIBLE = 2;
 function Fogging() {}
 
 Fogging.prototype.Schema =
-	"<a:help>Allows this entity to be replaced by mirages entities in the fog-of-war.</a:help>" +
+	"<a:help>Allows this entity to be replaced by mirage entities in the fog-of-war.</a:help>" +
 	"<empty/>";
 
 Fogging.prototype.Init = function()
 {
+	this.activated = false;
 	this.mirages = [];
+	this.miraged = [];
 	this.seen = [];
 
-	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	for (var player = 0; player < cmpPlayerManager.GetNumPlayers(); ++player)
+	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+	for (let player = 0; player < cmpPlayerManager.GetNumPlayers(); ++player)
 	{
 		this.mirages.push(INVALID_ENTITY);
+		this.miraged.push(false);
 		this.seen.push(false);
 	}
 };
 
+Fogging.prototype.Activate = function()
+{
+	let mustUpdate = !this.activated;
+	this.activated = true;
+
+	if (mustUpdate)
+	{
+		// Load a mirage for each player who has already seen the entity
+		let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+		for (let player = 0; player < cmpPlayerManager.GetNumPlayers(); ++player)
+		{
+			if (this.seen[player])
+				this.LoadMirage(player);
+		}
+	}
+};
+
+Fogging.prototype.IsActivated = function()
+{
+	return this.activated;
+};
+
 Fogging.prototype.LoadMirage = function(player)
 {
-	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	var templateName = "mirage|" + cmpTemplateManager.GetCurrentTemplateName(this.entity);
-	
-	// If this is an entity without visibility (e.g. a foundation), it should be
-	// marked as seen for its owner	
-	var cmpParentOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
-	if (cmpParentOwnership && cmpParentOwnership.GetOwner() == player)
-		this.seen[player] = true;
-
-	if (!this.seen[player] || this.mirages[player] != INVALID_ENTITY)
+	if (!this.activated)
+	{
+		error("LoadMirage called for an entity with fogging deactivated");
 		return;
+	}
+
+	this.miraged[player] = true;
+
+	if (this.mirages[player] == INVALID_ENTITY)
+	{
+		var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+		var templateName = "mirage|" + cmpTemplateManager.GetCurrentTemplateName(this.entity);
+
+		this.mirages[player] = Engine.AddEntity(templateName);
+	}
 
-	this.mirages[player] = Engine.AddEntity(templateName);
 	var cmpMirage = Engine.QueryInterface(this.mirages[player], IID_Mirage);
 	if (!cmpMirage)
 	{
@@ -44,15 +72,16 @@ Fogging.prototype.LoadMirage = function(player)
 		return;
 	}
 
-	// Setup basic mirage properties
+	// Copy basic mirage properties
 	cmpMirage.SetPlayer(player);
 	cmpMirage.SetParent(this.entity);
 
 	// Copy cmpOwnership data
+	var cmpParentOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
 	var cmpMirageOwnership = Engine.QueryInterface(this.mirages[player], IID_Ownership);
 	if (!cmpParentOwnership || !cmpMirageOwnership)
 	{
-		error("Failed to setup the ownership data of the fogged entity " + templateName);
+		error("Failed to copy the ownership data of the fogged entity " + templateName);
 		return;
 	}
 	cmpMirageOwnership.SetOwner(cmpParentOwnership.GetOwner());
@@ -62,7 +91,7 @@ Fogging.prototype.LoadMirage = function(player)
 	var cmpMiragePosition = Engine.QueryInterface(this.mirages[player], IID_Position);
 	if (!cmpParentPosition || !cmpMiragePosition)
 	{
-		error("Failed to setup the position data of the fogged entity " + templateName);
+		error("Failed to copy the position data of the fogged entity " + templateName);
 		return;
 	}
 	if (!cmpParentPosition.IsInWorld())
@@ -78,7 +107,7 @@ Fogging.prototype.LoadMirage = function(player)
 	var cmpMirageVisualActor = Engine.QueryInterface(this.mirages[player], IID_Visual);
 	if (!cmpParentVisualActor || !cmpMirageVisualActor)
 	{
-		error("Failed to setup the visual data of the fogged entity " + templateName);
+		error("Failed to copy the visual data of the fogged entity " + templateName);
 		return;
 	}
 	cmpMirageVisualActor.SetActorSeed(cmpParentVisualActor.GetActorSeed());
@@ -86,11 +115,11 @@ Fogging.prototype.LoadMirage = function(player)
 	// Store valuable information into the mirage component (especially for the GUI)
 	var cmpFoundation = Engine.QueryInterface(this.entity, IID_Foundation);
 	if (cmpFoundation)
-		cmpMirage.AddFoundation(cmpFoundation.GetBuildPercentage());
+		cmpMirage.CopyFoundation(cmpFoundation.GetBuildPercentage());
 
 	var cmpHealth = Engine.QueryInterface(this.entity, IID_Health);
 	if (cmpHealth)
-		cmpMirage.AddHealth(
+		cmpMirage.CopyHealth(
 			cmpHealth.GetMaxHitpoints(), 
 			cmpHealth.GetHitpoints(), 
 			cmpHealth.IsRepairable() && (cmpHealth.GetHitpoints() < cmpHealth.GetMaxHitpoints())
@@ -98,7 +127,7 @@ Fogging.prototype.LoadMirage = function(player)
 
 	var cmpResourceSupply = Engine.QueryInterface(this.entity, IID_ResourceSupply);
 	if (cmpResourceSupply)
-		cmpMirage.AddResourceSupply(
+		cmpMirage.CopyResourceSupply(
 			cmpResourceSupply.GetMaxAmount(), 
 			cmpResourceSupply.GetCurrentAmount(), 
 			cmpResourceSupply.GetType(), 
@@ -108,10 +137,17 @@ Fogging.prototype.LoadMirage = function(player)
 	// Notify the GUI the entity has been replaced by a mirage, in case it is selected at this moment
 	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	cmpGuiInterface.AddMiragedEntity(player, this.entity, this.mirages[player]);
+
+	// Notify the range manager the visibility of this entity must be updated
+	let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+	cmpRangeManager.RequestVisibilityUpdate(this.entity);
 };
 
 Fogging.prototype.ForceMiraging = function(player)
 {
+	if (!this.activated)
+		return;
+
 	this.seen[player] = true;
 	this.LoadMirage(player);
 };
@@ -121,7 +157,7 @@ Fogging.prototype.IsMiraged = function(player)
 	if (player >= this.mirages.length)
 		return false;
 
-	return this.mirages[player] != INVALID_ENTITY;
+	return this.miraged[player];
 };
 
 Fogging.prototype.GetMirage = function(player)
@@ -140,6 +176,30 @@ Fogging.prototype.WasSeen = function(player)
 	return this.seen[player];
 };
 
+Fogging.prototype.OnDestroy = function(msg)
+{
+	for (var player = 0; player < this.mirages.length; ++player)
+	{
+		var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+		if (cmpRangeManager.GetLosVisibility(this.mirages[player], player) == "hidden")
+		{
+			Engine.DestroyEntity(this.mirages[player]);
+			continue;
+		}
+
+		var cmpMirage = Engine.QueryInterface(this.mirages[player], IID_Mirage);
+		if (cmpMirage)
+			cmpMirage.SetParent(INVALID_ENTITY);
+	}
+};
+
+Fogging.prototype.OnOwnershipChanged = function(msg)
+{
+	// Always activate fogging for non-Gaia entities
+	if (msg.to > 0)
+		this.Activate();
+};
+
 Fogging.prototype.OnVisibilityChanged = function(msg)
 {
 	if (msg.player >= this.mirages.length)
@@ -147,29 +207,12 @@ Fogging.prototype.OnVisibilityChanged = function(msg)
 
 	if (msg.newVisibility == VIS_VISIBLE)
 	{
+		this.miraged[msg.player] = false;
 		this.seen[msg.player] = true;
-
-		// Destroy mirages when we get back into LoS
-		if (this.mirages[msg.player] != INVALID_ENTITY)
-		{
-			Engine.DestroyEntity(this.mirages[msg.player]);
-			this.mirages[msg.player] = INVALID_ENTITY;
-		}
 	}
 
-	// Intermediate LoS state, meaning we must create a mirage
-	if (msg.newVisibility == VIS_FOGGED)
+	if (msg.newVisibility == VIS_FOGGED && this.activated)
 		this.LoadMirage(msg.player);
 };
 
-Fogging.prototype.OnDestroy = function(msg)
-{
-	for (var player = 0; player < this.mirages.length; ++player)
-	{
-		var cmpMirage = Engine.QueryInterface(this.mirages[player], IID_Mirage);
-		if (cmpMirage)
-			cmpMirage.SetParent(INVALID_ENTITY);
-	}
-};
-
 Engine.RegisterComponentType(IID_Fogging, "Fogging", Fogging);
