@@ -53,6 +53,9 @@ GarrisonHolder.prototype.Schema =
 
 /**
  * Initialize GarrisonHolder Component
+ *
+ * Garrisoning when loading a map is set in the script of the map, by setting initGarrison 
+ * which should contain the array of garrisoned entities
  */
 GarrisonHolder.prototype.Init = function()
 {
@@ -239,6 +242,9 @@ GarrisonHolder.prototype.Garrison = function(entity)
 		vgp.entity = entity;
 		cmpPosition.SetTurretParent(this.entity, vgp.offset);
 		visiblyGarrisoned = true;
+		var cmpUnitAI = Engine.QueryInterface(entity, IID_UnitAI);
+		if (cmpUnitAI)
+			cmpUnitAI.SetTurretStance();
 		break;
 	}
 	if (!visiblyGarrisoned)
@@ -296,7 +302,6 @@ GarrisonHolder.prototype.PerformGarrison = function(entity)
  */
 GarrisonHolder.prototype.Eject = function(entity, forced)
 {
-
 	var entityIndex = this.entities.indexOf(entity);
 	// Error: invalid entity ID, usually it's already been ejected
 	if (entityIndex == -1)
@@ -335,6 +340,9 @@ GarrisonHolder.prototype.Eject = function(entity, forced)
 		if (vgp.entity != entity)
 			continue;
 		cmpNewPosition.SetTurretParent(INVALID_ENTITY, new Vector3D());
+		var cmpUnitAI = Engine.QueryInterface(entity, IID_UnitAI);
+		if (cmpUnitAI)
+			cmpUnitAI.ResetTurretStance();
 		vgp.entity = null;
 		break;
 	}
@@ -596,7 +604,8 @@ GarrisonHolder.prototype.OnGlobalOwnershipChanged = function(msg)
 			if (msg.to == -1 || !IsOwnedByMutualAllyOfEntity(this.entity, entity))
 				entities.push(entity);
 		}
-		this.EjectOrKill(entities);
+		if (entities.length)
+			this.EjectOrKill(entities);
 		return;
 	}
 
@@ -632,6 +641,23 @@ GarrisonHolder.prototype.OnGlobalEntityRenamed = function(msg)
 		this.Eject(msg.entity);
 		this.Garrison(msg.newentity);
 	}
+
+	if (!this.initGarrison)
+		return;
+
+	// update the pre-game garrison because of SkirmishReplacement
+	if (msg.entity == this.entity)
+	{
+		let cmpGarrisonHolder = Engine.QueryInterface(msg.newentity, IID_GarrisonHolder);
+		if (cmpGarrisonHolder)
+			cmpGarrisonHolder.initGarrison = this.initGarrison;
+	}
+	else
+	{
+		let entityIndex = this.initGarrison.indexOf(msg.entity);
+		if (entityIndex != -1)
+			this.initGarrison[entityIndex] = msg.newentity;
+	}
 };
 
 
@@ -697,5 +723,23 @@ GarrisonHolder.prototype.IsEjectable = function(entity)
 	return false;
 };
 
+/**
+ * Initialise the garrisoned units
+ */
+GarrisonHolder.prototype.OnGlobalInitGame = function(msg)
+{
+	if (!this.initGarrison)
+		return;
+
+	for (let ent of this.initGarrison)
+	{
+		let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
+		if (!cmpUnitAI || !cmpUnitAI.CanGarrison(this.entity))
+			continue;
+		this.Garrison(ent);
+	}
+	this.initGarrison = undefined;
+};
+
 Engine.RegisterComponentType(IID_GarrisonHolder, "GarrisonHolder", GarrisonHolder);
 
