@@ -46,7 +46,7 @@ GuiInterface.prototype.GetSimulationState = function(player)
 	var ret = {
 		"players": []
 	};
-	
+
 	var cmpPlayerMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
 	var n = cmpPlayerMan.GetNumPlayers();
 	for (var i = 0; i < n; ++i)
@@ -54,17 +54,20 @@ GuiInterface.prototype.GetSimulationState = function(player)
 		var playerEnt = cmpPlayerMan.GetPlayerByID(i);
 		var cmpPlayerEntityLimits = Engine.QueryInterface(playerEnt, IID_EntityLimits);
 		var cmpPlayer = Engine.QueryInterface(playerEnt, IID_Player);
-		
+
 		// Work out what phase we are in
-		var cmpTechnologyManager = Engine.QueryInterface(playerEnt, IID_TechnologyManager);
 		var phase = "";
-		if (cmpTechnologyManager.IsTechnologyResearched("phase_city"))
-			phase = "city";
-		else if (cmpTechnologyManager.IsTechnologyResearched("phase_town"))
-			phase = "town";
-		else if (cmpTechnologyManager.IsTechnologyResearched("phase_village"))
-			phase = "village";
-		
+		var cmpTechnologyManager = Engine.QueryInterface(playerEnt, IID_TechnologyManager);
+		if (cmpTechnologyManager)
+		{
+			if (cmpTechnologyManager.IsTechnologyResearched("phase_city"))
+				phase = "city";
+			else if (cmpTechnologyManager.IsTechnologyResearched("phase_town"))
+				phase = "town";
+			else if (cmpTechnologyManager.IsTechnologyResearched("phase_village"))
+				phase = "village";
+		}
+
 		// store player ally/neutral/enemy data as arrays
 		var allies = [];
 		var mutualAllies = [];
@@ -91,40 +94,60 @@ GuiInterface.prototype.GetSimulationState = function(player)
 			"team": cmpPlayer.GetTeam(),
 			"teamsLocked": cmpPlayer.GetLockTeams(),
 			"cheatsEnabled": cmpPlayer.GetCheatsEnabled(),
+			"disabledTemplates": cmpPlayer.GetDisabledTemplates(),
 			"phase": phase,
 			"isAlly": allies,
 			"isMutualAlly": mutualAllies,
 			"isNeutral": neutrals,
 			"isEnemy": enemies,
-			"entityLimits": cmpPlayerEntityLimits.GetLimits(),
-			"entityCounts": cmpPlayerEntityLimits.GetCounts(),
-			"entityLimitChangers": cmpPlayerEntityLimits.GetLimitChangers(),
-			"researchQueued": cmpTechnologyManager.GetQueuedResearch(),
-			"researchStarted": cmpTechnologyManager.GetStartedResearch(),
-			"researchedTechs": cmpTechnologyManager.GetResearchedTechs(),
-			"classCounts": cmpTechnologyManager.GetClassCounts(),
-			"typeCountsByClass": cmpTechnologyManager.GetTypeCountsByClass()
+			"entityLimits": cmpPlayerEntityLimits ? cmpPlayerEntityLimits.GetLimits() : null,
+			"entityCounts": cmpPlayerEntityLimits ? cmpPlayerEntityLimits.GetCounts() : null,
+			"entityLimitChangers": cmpPlayerEntityLimits ? cmpPlayerEntityLimits.GetLimitChangers() : null,
+			"researchQueued": cmpTechnologyManager ? cmpTechnologyManager.GetQueuedResearch() : null,
+			"researchStarted": cmpTechnologyManager ? cmpTechnologyManager.GetStartedResearch() : null,
+			"researchedTechs": cmpTechnologyManager ? cmpTechnologyManager.GetResearchedTechs() : null,
+			"classCounts": cmpTechnologyManager ? cmpTechnologyManager.GetClassCounts() : null,
+			"typeCountsByClass": cmpTechnologyManager ? cmpTechnologyManager.GetTypeCountsByClass() : null
 		};
 		ret.players.push(playerData);
 	}
 
 	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	if (cmpRangeManager)
-	{
 		ret.circularMap = cmpRangeManager.GetLosCircular();
-	}
-	
+
 	// Add timeElapsed
 	var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
 	ret.timeElapsed = cmpTimer.GetTime();
 
-	// and the game type
+	// Add the game type
 	var cmpEndGameManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_EndGameManager);
 	ret.gameType = cmpEndGameManager.GetGameType();
 
+	// Add bartering prices
+	var cmpBarter = Engine.QueryInterface(SYSTEM_ENTITY, IID_Barter);
+	ret.barterPrices = cmpBarter.GetPrices();
+
+	// Add basic statistics to each player
+	var cmpPlayerMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+	var n = cmpPlayerMan.GetNumPlayers();
+	for (var i = 0; i < n; ++i)
+	{
+		var playerEnt = cmpPlayerMan.GetPlayerByID(i);
+		var cmpPlayerStatisticsTracker = Engine.QueryInterface(playerEnt, IID_StatisticsTracker);
+		if (cmpPlayerStatisticsTracker)
+			ret.players[i].statistics = cmpPlayerStatisticsTracker.GetBasicStatistics();
+	}
+
 	return ret;
 };
 
+/**
+ * Returns global information about the current game state, plus statistics.
+ * This is used by the GUI at the end of a game, in the summary screen.
+ * Note: Amongst statistics, the team exploration map percentage is computed from
+ * scratch, so the extended simulation state should not be requested too often.
+ */
 GuiInterface.prototype.GetExtendedSimulationState = function(player)
 {
 	// Get basic simulation info
@@ -137,13 +160,10 @@ GuiInterface.prototype.GetExtendedSimulationState = function(player)
 	{
 		var playerEnt = cmpPlayerMan.GetPlayerByID(i);
 		var cmpPlayerStatisticsTracker = Engine.QueryInterface(playerEnt, IID_StatisticsTracker);
-		ret.players[i].statistics = cmpPlayerStatisticsTracker.GetStatistics();
+		if (cmpPlayerStatisticsTracker)
+			ret.players[i].statistics = cmpPlayerStatisticsTracker.GetStatistics();
 	}
 
-	// Add bartering prices
-	var cmpBarter = Engine.QueryInterface(SYSTEM_ENTITY, IID_Barter);
-	ret.barterPrices = cmpBarter.GetPrices();
-
 	return ret;
 };
 
@@ -186,13 +206,14 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 		"template": template,
 
 		"alertRaiser": null,
-		"buildEntities": null,
+		"builder": null,
 		"identity": null,
 		"fogging": null,
 		"foundation": null,
 		"garrisonHolder": null,
 		"gate": null,
 		"guard": null,
+		"mirage": null,
 		"pack": null,
 		"player": -1,
 		"position": null,
@@ -205,8 +226,9 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 		"visibility": null,
 	};
 
-	// Used for several components
 	var cmpMirage = Engine.QueryInterface(ent, IID_Mirage);
+	if (cmpMirage)
+		ret.mirage = true;
 
 	var cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
 	if (cmpIdentity)
@@ -243,9 +265,7 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 
 	var cmpBuilder = Engine.QueryInterface(ent, IID_Builder);
 	if (cmpBuilder)
-	{
-		ret.buildEntities = cmpBuilder.GetEntitiesList();
-	}
+		ret.builder = true;
 
 	var cmpPack = Engine.QueryInterface(ent, IID_Pack);
 	if (cmpPack)
@@ -321,7 +341,7 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 			"garrisonedEntitiesCount": cmpGarrisonHolder.GetGarrisonedEntitiesCount()
 		};
 	}
-	
+
 	var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 	if (cmpUnitAI)
 	{
@@ -332,6 +352,7 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 			"canGuard": cmpUnitAI.CanGuard(),
 			"isGuarding": cmpUnitAI.IsGuardOf(),
 			"possibleStances": cmpUnitAI.GetPossibleStances(),
+			"isIdle":cmpUnitAI.IsIdle(),
 		};
 		// Add some information needed for ungarrisoning
 		if (cmpUnitAI.IsGarrisoned() && ret.player !== undefined)
@@ -371,7 +392,7 @@ GuiInterface.prototype.GetEntityState = function(player, ent)
 	}
 
 	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-	ret.visibility = cmpRangeManager.GetLosVisibility(ent, player, false);
+	ret.visibility = cmpRangeManager.GetLosVisibility(ent, player);
 
 	return ret;
 };
@@ -387,7 +408,6 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 		"barterMarket": null,
 		"buildingAI": null,
 		"healer": null,
-		"mirage": null,
 		"obstruction": null,
 		"turretParent":null,
 		"promotion": null,
@@ -397,51 +417,51 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 	};
 
 	var cmpMirage = Engine.QueryInterface(ent, IID_Mirage);
-	if (cmpMirage)
-		ret.mirage = true;
 
 	var cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
 
 	var cmpAttack = Engine.QueryInterface(ent, IID_Attack);
 	if (cmpAttack)
 	{
-		var type = cmpAttack.GetBestAttack(); // TODO: how should we decide which attack to show? show all?
-		ret.attack = cmpAttack.GetAttackStrengths(type);
-		var range = cmpAttack.GetRange(type);
-		ret.attack.type = type;
-		ret.attack.minRange = range.min;
-		ret.attack.maxRange = range.max;
-		var timers = cmpAttack.GetTimers(type);
-		ret.attack.prepareTime = timers.prepare;
-		ret.attack.repeatTime = timers.repeat;
-		if (type == "Ranged")
+		var types = cmpAttack.GetAttackTypes();
+		if (types.length)
+			ret.attack = {};
+		for (var type of types)
 		{
-			ret.attack.elevationBonus = range.elevationBonus;
+			ret.attack[type] = cmpAttack.GetAttackStrengths(type);
+			var range = cmpAttack.GetRange(type);
+			ret.attack[type].minRange = range.min;
+			ret.attack[type].maxRange = range.max;
+			var timers = cmpAttack.GetTimers(type);
+			ret.attack[type].prepareTime = timers.prepare;
+			ret.attack[type].repeatTime = timers.repeat;
+			if (type != "Ranged")
+			{
+				// not a ranged attack, set some defaults
+				ret.attack[type].elevationBonus = 0;
+				ret.attack[type].elevationAdaptedRange = ret.attack.maxRange;
+				continue;
+			}
+
+			ret.attack[type].elevationBonus = range.elevationBonus;
 			var cmpPosition = Engine.QueryInterface(ent, IID_Position);
 			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 			var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 			if (cmpUnitAI && cmpPosition && cmpPosition.IsInWorld())
 			{
 				// For units, take the rage in front of it, no spread. So angle = 0
-				ret.attack.elevationAdaptedRange = cmpRangeManager.GetElevationAdaptedRange(cmpPosition.GetPosition(), cmpPosition.GetRotation(), range.max, range.elevationBonus, 0);
+				ret.attack[type].elevationAdaptedRange = cmpRangeManager.GetElevationAdaptedRange(cmpPosition.GetPosition(), cmpPosition.GetRotation(), range.max, range.elevationBonus, 0);
 			}
 			else if(cmpPosition && cmpPosition.IsInWorld())
 			{
 				// For buildings, take the average elevation around it. So angle = 2*pi
-				ret.attack.elevationAdaptedRange = cmpRangeManager.GetElevationAdaptedRange(cmpPosition.GetPosition(), cmpPosition.GetRotation(), range.max, range.elevationBonus, 2*Math.PI);
+				ret.attack[type].elevationAdaptedRange = cmpRangeManager.GetElevationAdaptedRange(cmpPosition.GetPosition(), cmpPosition.GetRotation(), range.max, range.elevationBonus, 2*Math.PI);
 			}
 			else
 			{
 				// not in world, set a default?
-				ret.attack.elevationAdaptedRange = ret.attack.maxRange;
+				ret.attack[type].elevationAdaptedRange = ret.attack.maxRange;
 			}
-			
-		}
-		else
-		{
-			// not a ranged attack, set some defaults
-			ret.attack.elevationBonus = 0;
-			ret.attack.elevationAdaptedRange = ret.attack.maxRange;
 		}
 	}
 
@@ -517,7 +537,7 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 			"types": cmpResourceDropsite.GetTypes()
 		};
 	}
-	
+
 	var cmpPromotion = Engine.QueryInterface(ent, IID_Promotion);
 	if (cmpPromotion)
 	{
@@ -525,7 +545,7 @@ GuiInterface.prototype.GetExtendedEntityState = function(player, ent)
 			"curr": cmpPromotion.GetCurrentXp(),
 			"req": cmpPromotion.GetRequiredXp()
 		};
-	}	
+	}
 
 	var cmpFoundation = Engine.QueryInterface(ent, IID_Foundation);
 	if (!cmpFoundation && cmpIdentity && cmpIdentity.HasClass("BarterMarket"))
@@ -572,235 +592,22 @@ GuiInterface.prototype.GetTemplateData = function(player, extendedName)
 	if (!template)
 		return null;
 
-	var ret = {};
-
-	if (template.Armour)
-	{
-		ret.armour = {
-			"hack": ApplyValueModificationsToTemplate("Armour/Hack", +template.Armour.Hack, player, template),
-			"pierce": ApplyValueModificationsToTemplate("Armour/Pierce", +template.Armour.Pierce, player, template),
-			"crush": ApplyValueModificationsToTemplate("Armour/Crush", +template.Armour.Crush, player, template),
-		};
-	}
-	
-	if (template.Attack)
-	{
-		ret.attack = {};
-		for (var type in template.Attack)
-		{
-			ret.attack[type] = {
-				"hack": ApplyValueModificationsToTemplate("Attack/"+type+"/Hack", +(template.Attack[type].Hack || 0), player, template),
-				"pierce": ApplyValueModificationsToTemplate("Attack/"+type+"/Pierce", +(template.Attack[type].Pierce || 0), player, template),
-				"crush": ApplyValueModificationsToTemplate("Attack/"+type+"/Crush", +(template.Attack[type].Crush || 0), player, template),
-				"minRange": ApplyValueModificationsToTemplate("Attack/"+type+"/MinRange", +(template.Attack[type].MinRange || 0), player, template),
-				"maxRange": ApplyValueModificationsToTemplate("Attack/"+type+"/MaxRange", +template.Attack[type].MaxRange, player, template),
-				"elevationBonus": ApplyValueModificationsToTemplate("Attack/"+type+"/ElevationBonus", +(template.Attack[type].ElevationBonus || 0), player, template),
-			};
-		}
-	}
-
-	if (template.Auras)
-	{
-		ret.auras = {};
-		for each (var aura in template.Auras)
-			if (aura.AuraName)
-				ret.auras[aura.AuraName] = aura.AuraDescription || null;
-	}
-
-	if (template.BuildRestrictions)
-	{
-		// required properties
-		ret.buildRestrictions = {
-			"placementType": template.BuildRestrictions.PlacementType,
-			"territory": template.BuildRestrictions.Territory,
-			"category": template.BuildRestrictions.Category,
-		};
-		
-		// optional properties
-		if (template.BuildRestrictions.Distance)
-		{
-			ret.buildRestrictions.distance = {
-				"fromCategory": template.BuildRestrictions.Distance.FromCategory,
-			};
-			if (template.BuildRestrictions.Distance.MinDistance) ret.buildRestrictions.distance.min = +template.BuildRestrictions.Distance.MinDistance;
-			if (template.BuildRestrictions.Distance.MaxDistance) ret.buildRestrictions.distance.max = +template.BuildRestrictions.Distance.MaxDistance;
-		}
-	}
-
-	if (template.TrainingRestrictions)
-	{
-		ret.trainingRestrictions = {
-			"category": template.TrainingRestrictions.Category,
-		};
-	}
-
-	if (template.Cost)
-	{
-		ret.cost = {};
-		if (template.Cost.Resources.food) ret.cost.food = ApplyValueModificationsToTemplate("Cost/Resources/food", +template.Cost.Resources.food, player, template);
-		if (template.Cost.Resources.wood) ret.cost.wood = ApplyValueModificationsToTemplate("Cost/Resources/wood", +template.Cost.Resources.wood, player, template);
-		if (template.Cost.Resources.stone) ret.cost.stone = ApplyValueModificationsToTemplate("Cost/Resources/stone", +template.Cost.Resources.stone, player, template);
-		if (template.Cost.Resources.metal) ret.cost.metal = ApplyValueModificationsToTemplate("Cost/Resources/metal", +template.Cost.Resources.metal, player, template);
-		if (template.Cost.Population) ret.cost.population = ApplyValueModificationsToTemplate("Cost/Population", +template.Cost.Population, player, template);
-		if (template.Cost.PopulationBonus) ret.cost.populationBonus = ApplyValueModificationsToTemplate("Cost/PopulationBonus", +template.Cost.PopulationBonus, player, template);
-		if (template.Cost.BuildTime) ret.cost.time = ApplyValueModificationsToTemplate("Cost/BuildTime", +template.Cost.BuildTime, player, template);
-	}
-	
-	if (template.Footprint)
-	{
-		ret.footprint = {"height": template.Footprint.Height};
-		
-		if (template.Footprint.Square)
-			ret.footprint.square = {"width": +template.Footprint.Square["@width"], "depth": +template.Footprint.Square["@depth"]};
-		else if (template.Footprint.Circle)
-			ret.footprint.circle = {"radius": +template.Footprint.Circle["@radius"]};
-		else
-			warn("[GetTemplateData] Unrecognized Footprint type");
-	}
-	
-	if (template.Obstruction)
-	{
-		ret.obstruction = {
-			"active": ("" + template.Obstruction.Active == "true"),
-			"blockMovement": ("" + template.Obstruction.BlockMovement == "true"),
-			"blockPathfinding": ("" + template.Obstruction.BlockPathfinding == "true"),
-			"blockFoundation": ("" + template.Obstruction.BlockFoundation == "true"),
-			"blockConstruction": ("" + template.Obstruction.BlockConstruction == "true"),
-			"disableBlockMovement": ("" + template.Obstruction.DisableBlockMovement == "true"),
-			"disableBlockPathfinding": ("" + template.Obstruction.DisableBlockPathfinding == "true"),
-			"shape": {}
-		};
-		
-		if (template.Obstruction.Static)
-		{
-			ret.obstruction.shape.type = "static";
-			ret.obstruction.shape.width = +template.Obstruction.Static["@width"];
-			ret.obstruction.shape.depth = +template.Obstruction.Static["@depth"];
-		}
-		else if (template.Obstruction.Unit)
-		{
-			ret.obstruction.shape.type = "unit";
-			ret.obstruction.shape.radius = +template.Obstruction.Unit["@radius"];
-		}
-		else
-		{
-			ret.obstruction.shape.type = "cluster";
-		}
-	}
-
-	if (template.Pack)
-	{
-		ret.pack = {
-			"state": template.Pack.State,
-			"time": ApplyValueModificationsToTemplate("Pack/Time", +template.Pack.Time, player, template),
-		};
-	}
-
-	if (template.Health)
-	{
-		ret.health = Math.round(ApplyValueModificationsToTemplate("Health/Max", +template.Health.Max, player, template));
-	}
-
-	if (template.Identity)
-	{
-		ret.selectionGroupName = template.Identity.SelectionGroupName;
-		ret.name = {
-			"specific": (template.Identity.SpecificName || template.Identity.GenericName),
-			"generic": template.Identity.GenericName
-		};
-		ret.icon = template.Identity.Icon;
-		ret.tooltip =  template.Identity.Tooltip;
-		ret.gateConversionTooltip =  template.Identity.GateConversionTooltip;
-		ret.requiredTechnology = template.Identity.RequiredTechnology;
-		ret.visibleIdentityClasses = GetVisibleIdentityClasses(template.Identity);
-	}
-
-	if (template.UnitMotion)
-	{
-		ret.speed = {
-			"walk": ApplyValueModificationsToTemplate("UnitMotion/WalkSpeed", +template.UnitMotion.WalkSpeed, player, template),
-		};
-		if (template.UnitMotion.Run) ret.speed.run = ApplyValueModificationsToTemplate("UnitMotion/Run/Speed", +template.UnitMotion.Run.Speed, player, template);
-	}
-
-	if (template.Trader)
-		ret.trader = template.Trader;
-
-	if (template.WallSet)
-	{
-		ret.wallSet = {
-			"templates": {
-				"tower": template.WallSet.Templates.Tower,
-				"gate": template.WallSet.Templates.Gate,
-				"long": template.WallSet.Templates.WallLong,
-				"medium": template.WallSet.Templates.WallMedium,
-				"short": template.WallSet.Templates.WallShort,
-			},
-			"maxTowerOverlap": +template.WallSet.MaxTowerOverlap,
-			"minTowerOverlap": +template.WallSet.MinTowerOverlap,
-		};
-	}
-	
-	if (template.WallPiece)
-	{
-		ret.wallPiece = {"length": +template.WallPiece.Length};
-	}
-
-	return ret;
+	return GetTemplateDataHelper(template, player);
 };
 
 GuiInterface.prototype.GetTechnologyData = function(player, name)
 {
 	var cmpTechTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager);
 	var template = cmpTechTempMan.GetTemplate(name);
-	
+
 	if (!template)
 	{
 		warn("Tried to get data for invalid technology: " + name);
 		return null;
 	}
-	
-	var ret = {};
-	
-	// Get specific name for this civ or else the generic specific name 
+
 	var cmpPlayer = QueryPlayerIDInterface(player, IID_Player);
-	var specific = undefined;
-	if (template.specificName)
-	{
-		if (template.specificName[cmpPlayer.GetCiv()])
-			specific = template.specificName[cmpPlayer.GetCiv()];
-		else
-			specific = template.specificName['generic'];
-	}
-	
-	ret.name = {
-		"specific": specific,
-		"generic": template.genericName,
-	};
-	if (template.icon)
-		ret.icon = "technologies/" + template.icon;
-	else
-		ret.icon = null;
-	ret.cost = {
-		"food": template.cost ? (+template.cost.food) : 0,
-		"wood": template.cost ? (+template.cost.wood) : 0,
-		"metal": template.cost ? (+template.cost.metal) : 0,
-		"stone": template.cost ? (+template.cost.stone) : 0,
-		"time": template.researchTime ? (+template.researchTime) : 0,
-	}
-	ret.tooltip = template.tooltip;
-	
-	if (template.requirementsTooltip)
-		ret.requirementsTooltip = template.requirementsTooltip;
-	else
-		ret.requirementsTooltip = "";
-	
-	if (template.requirements.class)
-		ret.classRequirements = {"class": template.requirements.class, "number": template.requirements.number};
-	
-	ret.description = template.description;
-	
-	return ret;
+	return GetTechnologyDataHelper(template, cmpPlayer.GetCiv());
 };
 
 GuiInterface.prototype.IsTechnologyResearched = function(player, tech)
@@ -812,7 +619,7 @@ GuiInterface.prototype.IsTechnologyResearched = function(player, tech)
 	
 	if (!cmpTechnologyManager)
 		return false;
-	
+
 	return cmpTechnologyManager.IsTechnologyResearched(tech);
 };
 
@@ -820,10 +627,10 @@ GuiInterface.prototype.IsTechnologyResearched = function(player, tech)
 GuiInterface.prototype.CheckTechnologyRequirements = function(player, tech)
 {
 	var cmpTechnologyManager = QueryPlayerIDInterface(player, IID_TechnologyManager);
-	
+
 	if (!cmpTechnologyManager)
 		return false;
-	
+
 	return cmpTechnologyManager.CanResearch(tech);
 };
 
@@ -852,7 +659,10 @@ GuiInterface.prototype.GetStartedResearch = function(player)
 GuiInterface.prototype.GetBattleState = function(player)
 {
 	var cmpBattleDetection = QueryPlayerIDInterface(player, IID_BattleDetection);
-	return cmpBattleDetection.GetState();
+	if (cmpBattleDetection)
+		return cmpBattleDetection.GetState();
+	else
+		return false;
 };
 
 // Returns a list of ongoing attacks against the player.
@@ -986,6 +796,22 @@ GuiInterface.prototype.IsStanceSelected = function(player, data)
 	return false;
 };
 
+GuiInterface.prototype.GetAllBuildableEntities = function(player, cmd)
+{
+	var buildableEnts = [];
+	for each (var ent in cmd.entities)
+	{
+		var cmpBuilder = Engine.QueryInterface(ent, IID_Builder);
+		if (!cmpBuilder)
+			continue;
+
+		for (var building of cmpBuilder.GetEntitiesList())
+			if (buildableEnts.indexOf(building) == -1)
+				buildableEnts.push(building);
+	}
+	return buildableEnts;
+};
+
 GuiInterface.prototype.SetSelectionHighlight = function(player, cmd)
 {
 	var cmpPlayerMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
@@ -1618,10 +1444,8 @@ GuiInterface.prototype.SetWallPlacementPreview = function(player, cmd)
 		cmpOwnership.SetOwner(player);
 		
 		// Check whether it's in a visible or fogged region
-		//  tell GetLosVisibility to force RetainInFog because preview entities set this to false,
-		//	which would show them as hidden instead of fogged
 		// TODO: should definitely reuse SetBuildingPlacementPreview, this is just straight up copy/pasta
-		var visible = (cmpRangeManager.GetLosVisibility(ent, player, true) != "hidden");
+		var visible = (cmpRangeManager.GetLosVisibility(ent, player) != "hidden");
 		if (visible)
 		{
 			var cmpBuildRestrictions = Engine.QueryInterface(ent, IID_BuildRestrictions);
@@ -2006,6 +1830,7 @@ var exposedFunctions = {
 	"IsStanceSelected": 1,
 
 	"SetSelectionHighlight": 1,
+	"GetAllBuildableEntities": 1,
 	"SetStatusBars": 1,
 	"GetPlayerEntities": 1,
 	"DisplayRallyPoint": 1,
