@@ -43,6 +43,8 @@ Player.prototype.Init = function()
 		"stone": markForTranslation("Stone"),
 	}
 	this.disabledTemplates = {};
+	this.disabledTechnologies = {};
+	this.startingTechnologies = {};
 };
 
 Player.prototype.SetPlayerID = function(id)
@@ -568,6 +570,24 @@ Player.prototype.IsNeutral = function(id)
 	return this.diplomacy[id] == 0;
 };
 
+/**
+ * Do some map dependant initializations
+ */
+Player.prototype.OnGlobalInitGame = function(msg)
+{
+	let cmpTechnologyManager = Engine.QueryInterface(this.entity, IID_TechnologyManager);
+	if (cmpTechnologyManager)
+		for (let tech in this.startingTechnologies)
+			cmpTechnologyManager.ResearchTechnology(tech);
+
+	// Replace the "{civ}" code with this civ ID
+	let disabledTemplates = this.disabledTemplates;
+	this.disabledTemplates = {};
+	for (let template in disabledTemplates)
+		if (disabledTemplates[template])
+			this.disabledTemplates[template.replace(/\{civ\}/g, this.civ)] = true;
+};
+
 /**
  * Keep track of population effects of all entities that
  * become owned or unowned by this player
@@ -698,6 +718,8 @@ Player.prototype.TributeResource = function(player, amounts)
 	var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	if (cmpGUIInterface)
 		cmpGUIInterface.PushNotification(notification);
+
+	Engine.BroadcastMessage(MT_TributeExchanged, {"to": player, "from": this.playerID, "amounts": amounts});
 };
 
 Player.prototype.AddDisabledTemplate = function(template)
@@ -724,9 +746,42 @@ Player.prototype.SetDisabledTemplates = function(templates)
 	cmpGuiInterface.PushNotification({"type": "resetselectionpannel", "players": [this.GetPlayerID()]});
 };
 
-Player.prototype.GetDisabledTemplates = function(templates)
+Player.prototype.GetDisabledTemplates = function()
 {
 	return this.disabledTemplates;
 };
 
+Player.prototype.AddDisabledTechnology = function(tech)
+{
+	this.disabledTechnologies[tech] = true;
+	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, {});
+};
+
+Player.prototype.RemoveDisabledTechnology = function(tech)
+{
+	this.disabledTechnologies[tech] = false;
+	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, {});
+};
+
+Player.prototype.SetDisabledTechnologies = function(techs)
+{
+	this.disabledTechnologies = techs;
+	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, {});
+};
+
+Player.prototype.GetDisabledTechnologies = function()
+{
+	return this.disabledTechnologies;
+};
+
+Player.prototype.AddStartingTechnology = function(tech)
+{
+	this.startingTechnologies[tech] = true;
+};
+
+Player.prototype.SetStartingTechnologies = function(techs)
+{
+	this.startingTechnologies = techs;
+};
+
 Engine.RegisterComponentType(IID_Player, "Player", Player);
