@@ -103,7 +103,15 @@ ProductionQueue.prototype.CalculateEntitiesList = function()
 		string = string.replace(/\{civ\}/g, cmpIdentity.GetCiv());
 	
 	var entitiesList = string.split(/\s+/);
-
+	
+	// Remove disabled entities
+	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+	var disabledEntities = cmpPlayer.GetDisabledTemplates();
+	
+	for (var i = entitiesList.length - 1; i >= 0; --i)
+		if (disabledEntities[entitiesList[i]])
+			entitiesList.splice(i, 1);
+	
 	// check if some templates need to show their advanced or elite version
 	var upgradeTemplate = function(templateName)
 	{
@@ -120,7 +128,7 @@ ProductionQueue.prototype.CalculateEntitiesList = function()
 	};
 
 	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	var playerID = QueryOwnerInterface(this.entity, IID_Player).GetPlayerID();
+	var playerID = cmpPlayer.GetPlayerID();
 	for each (var templateName in entitiesList)
 		this.entitiesList.push(upgradeTemplate(templateName));
 	for each (var item in this.queue)
@@ -148,13 +156,19 @@ ProductionQueue.prototype.GetTechnologiesList = function()
 	
 	var techs = string.split(/\s+/);
 	var techList = [];
-	var superseded = {}; // Stores the tech which supersedes the key 
+	var superseded = {}; // Stores the tech which supersedes the key
+
+	var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
+	if (cmpPlayer)
+		var disabledTechnologies = cmpPlayer.GetDisabledTechnologies();
 	
 	// Add any top level technologies to an array which corresponds to the displayed icons
 	// Also store what a technology is superceded by in the superceded object {"tech1":"techWhichSupercedesTech1", ...}
 	for (var i in techs)
 	{
 		var tech = techs[i];
+		if (disabledTechnologies && disabledTechnologies[tech])
+			continue;
 		var template = cmpTechnologyManager.GetTechnologyTemplate(tech);
 		if (!template.supersedes || techs.indexOf(template.supersedes) === -1)
 			techList.push(tech);
@@ -288,6 +302,10 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 				"timeTotal": time*1000,
 				"timeRemaining": time*1000,
 			});
+			
+			// Call the related trigger event 
+			var cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);
+			cmpTrigger.CallEvent("TrainingQueued", {"playerid": cmpPlayer.GetPlayerID(), "unitTemplate": templateName, "count": count, "metadata": metadata, "trainerEntity": this.entity});
 		}
 		else if (type == "technology")
 		{
@@ -324,6 +342,10 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 				"timeTotal": time*1000,
 				"timeRemaining": time*1000,
 			});
+			
+			// Call the related trigger event 
+			var cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);
+			cmpTrigger.CallEvent("ResearchQueued", {"playerid": cmpPlayer.GetPlayerID(), "technologyTemplate": templateName, "researcherEntity": this.entity});
 		}
 		else
 		{
@@ -342,7 +364,7 @@ ProductionQueue.prototype.AddBatch = function(templateName, type, count, metadat
 	}
 	else
 	{
-		var notification = {"player": cmpPlayer.GetPlayerID(), "message": markForTranslation("The production queue is full."), "translateMessage": true };
+		var notification = {"players": [cmpPlayer.GetPlayerID()], "message": markForTranslation("The production queue is full."), "translateMessage": true };
 		var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 		cmpGUIInterface.PushNotification(notification);
 	}
@@ -432,6 +454,7 @@ ProductionQueue.prototype.GetQueue = function()
 			"count": item.count,
 			"neededSlots": item.neededSlots,
 			"progress": 1 - ( item.timeRemaining / (item.timeTotal || 1) ),
+			"timeRemaining": item.timeRemaining,
 			"metadata": item.metadata,
 		});
 	}
@@ -504,7 +527,6 @@ ProductionQueue.prototype.OnDestroy = function()
 ProductionQueue.prototype.SpawnUnits = function(templateName, count, metadata)
 {
 	var cmpFootprint = Engine.QueryInterface(this.entity, IID_Footprint);
-	var cmpPosition = Engine.QueryInterface(this.entity, IID_Position);
 	var cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
 	var cmpRallyPoint = Engine.QueryInterface(this.entity, IID_RallyPoint);
 	
@@ -550,7 +572,7 @@ ProductionQueue.prototype.SpawnUnits = function(templateName, count, metadata)
 		if (cmpAutoGarrison && cmpAutoGarrison.PerformGarrison(ent))
 		{
 			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
-			cmpUnitAI.Autogarrison();
+			cmpUnitAI.Autogarrison(this.entity);
 		}
 		else
 		{
@@ -664,6 +686,8 @@ ProductionQueue.prototype.ProgressTimeout = function(data)
 			}
 
 			item.productionStarted = true;
+			if (item.unitTemplate)
+				Engine.PostMessage(this.entity, MT_TrainingStarted, {"entity": this.entity});
 		}
 
 		// If we won't finish the batch now, just update its timer
@@ -706,7 +730,7 @@ ProductionQueue.prototype.ProgressTimeout = function(data)
 				if (!this.spawnNotified)
 				{
 					var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
-					var notification = {"player": cmpPlayer.GetPlayerID(), "message": "Can't find free space to spawn trained units" };
+					var notification = {"players": [cmpPlayer.GetPlayerID()], "message": "Can't find free space to spawn trained units" };
 					var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 					cmpGUIInterface.PushNotification(notification);
 					this.spawnNotified = true;
@@ -774,4 +798,11 @@ ProductionQueue.prototype.OnValueModification = function(msg)
 		this.CalculateEntitiesList();
 };
 
+ProductionQueue.prototype.OnDisabledTemplatesChanged = function(msg)
+{
+	// if the disabled templates of the player is changed,
+	// update the entities list so that this is reflected there
+	this.CalculateEntitiesList();
+};
+
 Engine.RegisterComponentType(IID_ProductionQueue, "ProductionQueue", ProductionQueue);
