@@ -121,6 +121,22 @@ StatisticsTracker.prototype.Init = function()
 	this.treasuresCollected = 0;
 };
 
+/**
+ * Returns a subset of statistics that will be added to the simulation state,
+ * thus called each turn. Basic statistics should not contain data that would 
+ * be expensive to compute.
+ *
+ * Note: as of now, nothing in the game needs that, but some AIs developed by
+ * modders need it in the API.
+ */
+StatisticsTracker.prototype.GetBasicStatistics = function()
+{
+	return {
+		"resourcesGathered": this.resourcesGathered,
+		"percentMapExplored": this.GetPercentMapExplored()
+	};
+};
+
 StatisticsTracker.prototype.GetStatistics = function()
 {
 	return {
@@ -142,7 +158,8 @@ StatisticsTracker.prototype.GetStatistics = function()
 		"tributesReceived": this.tributesReceived,
 		"tradeIncome": this.tradeIncome,
 		"treasuresCollected": this.treasuresCollected,
-		"percentMapExplored": this.GetPercentMapExplored()
+		"percentMapExplored": this.GetPercentMapExplored(),
+		"teamPercentMapExplored": this.GetTeamPercentMapExplored()
 	};
 };
 
@@ -336,4 +353,33 @@ StatisticsTracker.prototype.GetPercentMapExplored = function()
 	return cmpRangeManager.GetPercentMapExplored(cmpPlayer.GetPlayerID());
 };
 
+/**
+ * Note: cmpRangeManager.GetUnionPercentMapExplored computes statistics from scratch!
+ * As a consequence, this function should not be called too often.
+ */
+StatisticsTracker.prototype.GetTeamPercentMapExplored = function()
+{
+	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+
+	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+	var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+	if (!cmpPlayer)
+		return 0;
+
+	var team = cmpPlayer.GetTeam();
+	// If teams are not locked, this statistic won't be displayed, so don't bother computing
+	if (team == -1 || !cmpPlayer.GetLockTeams())
+		return cmpRangeManager.GetPercentMapExplored(cmpPlayer.GetPlayerID());
+
+	var teamPlayers = [];
+	for (var i = 1; i < cmpPlayerManager.GetNumPlayers(); ++i)
+	{
+		let cmpOtherPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(i), IID_Player);
+		if (cmpOtherPlayer && cmpOtherPlayer.GetTeam() == team)
+			teamPlayers.push(i);
+	}
+
+	return cmpRangeManager.GetUnionPercentMapExplored(teamPlayers);
+};
+
 Engine.RegisterComponentType(IID_StatisticsTracker, "StatisticsTracker", StatisticsTracker);
