@@ -15,9 +15,6 @@ Trader.prototype.Schema =
 		"<MaxDistance>2.0</MaxDistance>" +
 		"<GainMultiplier>1.0</GainMultiplier>" +
 	"</a:example>" +
-	"<element name='MaxDistance' a:help='Max distance from market when performing deal'>" +
-		"<ref name='positiveDecimal'/>" +
-	"</element>" +
 	"<element name='GainMultiplier' a:help='Additional gain multiplier'>" +
 		"<ref name='positiveDecimal'/>" +
 	"</element>";
@@ -37,6 +34,8 @@ Trader.prototype.Init = function()
 Trader.prototype.CalculateGain = function(firstMarket, secondMarket)
 {
 	var gain = CalculateTraderGain(firstMarket, secondMarket, this.template, this.entity);
+	if (!gain)	// One of our markets must have been destroyed
+		return null;
 
 	// For ship increase gain for each garrisoned trader
 	// Calculate this here to save passing unnecessary stuff into the CalculateTraderGain function
@@ -267,7 +266,11 @@ Trader.prototype.StopTrading = function()
 // to be able to trade with it.
 Trader.prototype.GetRange = function()
 {
-	return { "min": 0, "max": +this.template.MaxDistance };
+	var cmpObstruction = Engine.QueryInterface(this.entity, IID_Obstruction);
+	var max = 1;
+	if (cmpObstruction)
+		max += cmpObstruction.GetUnitRadius()*1.5;
+	return { "min": 0, "max": max};
 };
 
 Trader.prototype.OnGarrisonedUnitsChanged = function()
