@@ -1,8 +1,10 @@
 Engine.LoadComponentScript("interfaces/Attack.js");
 Engine.LoadComponentScript("interfaces/AlertRaiser.js");
+Engine.LoadComponentScript("interfaces/Auras.js");
 Engine.LoadComponentScript("interfaces/Barter.js");
 Engine.LoadComponentScript("interfaces/Builder.js");
 Engine.LoadComponentScript("interfaces/DamageReceiver.js");
+Engine.LoadComponentScript("interfaces/EndGameManager.js");
 Engine.LoadComponentScript("interfaces/EntityLimits.js");
 Engine.LoadComponentScript("interfaces/Foundation.js");
 Engine.LoadComponentScript("interfaces/GarrisonHolder.js");
@@ -35,6 +37,10 @@ AddMock(SYSTEM_ENTITY, IID_Barter, {
 	}},
 });
 
+AddMock(SYSTEM_ENTITY, IID_EndGameManager, {
+	GetGameType: function() { return "conquest"; }
+});
+
 AddMock(SYSTEM_ENTITY, IID_PlayerManager, {
 	GetNumPlayers: function() { return 2; },
 	GetPlayerByID: function(id) { TS_ASSERT(id === 0 || id === 1); return 100+id; },
@@ -75,6 +81,7 @@ AddMock(100, IID_Player, {
 	IsMutualAlly: function() { return false; },
 	IsNeutral: function() { return false; },
 	IsEnemy: function() { return true; },
+	GetDisabledTemplates: function() { return {}; },
 });
 
 AddMock(100, IID_EntityLimits, {
@@ -94,6 +101,18 @@ AddMock(100, IID_TechnologyManager, {
 });
 
 AddMock(100, IID_StatisticsTracker, {
+	GetBasicStatistics: function() { 
+		return {
+			"resourcesGathered": {
+				"food": 100,	
+				"wood": 0,	
+				"metal": 0,	
+				"stone": 0,
+				"vegetarianFood": 0, 
+			},
+			"percentMapExplored": 10
+		};
+	},
 	GetStatistics: function() { 
 		return {
 			"unitsTrained": 10,
@@ -110,6 +129,7 @@ AddMock(100, IID_StatisticsTracker, {
 			},
 			"treasuresCollected": 0,
 			"percentMapExplored": 10,
+			"teamPercentMapExplored": 10
 		};
 	},
 	IncreaseTrainedUnitsCounter: function() { return 1; },
@@ -137,6 +157,7 @@ AddMock(101, IID_Player, {
 	IsMutualAlly: function() {return false; },
 	IsNeutral: function() { return false; },
 	IsEnemy: function() { return false; },
+	GetDisabledTemplates: function() { return {}; },
 });
 
 AddMock(101, IID_EntityLimits, {
@@ -156,6 +177,18 @@ AddMock(101, IID_TechnologyManager, {
 });
 
 AddMock(101, IID_StatisticsTracker, {
+	GetBasicStatistics: function() { 
+		return {
+			"resourcesGathered": {
+				"food": 100,	
+				"wood": 0,	
+				"metal": 0,	
+				"stone": 0,
+				"vegetarianFood": 0, 
+			},
+			"percentMapExplored": 10
+		};
+	},
 	GetStatistics: function() { 
 		return {
 			"unitsTrained": 10,
@@ -172,6 +205,7 @@ AddMock(101, IID_StatisticsTracker, {
 			},
 			"treasuresCollected": 0,
 			"percentMapExplored": 10,
+			"teamPercentMapExplored": 10
 		};
 	},
 	IncreaseTrainedUnitsCounter: function() { return 1; },
@@ -198,6 +232,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			team: -1,
 			teamsLocked: false,
 			cheatsEnabled: false,
+			disabledTemplates: {},
 			phase: "village",
 			isAlly: [false, false],
 			isMutualAlly: [false, false],
@@ -211,6 +246,16 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			researchedTechs: {},
 			classCounts: {},
 			typeCountsByClass: {},
+			statistics: {
+				resourcesGathered: {
+					food: 100,
+					wood: 0,
+					metal: 0,
+					stone: 0,
+					vegetarianFood: 0, 
+				},
+				percentMapExplored: 10
+			},
 		},
 		{
 			name: "Player 2",
@@ -226,6 +271,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			team: -1,
 			teamsLocked: false,
 			cheatsEnabled: false,
+			disabledTemplates: {},
 			phase: "village",
 			isAlly: [true, true],
 			isMutualAlly: [false, false],
@@ -239,10 +285,22 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			researchedTechs: {},
 			classCounts: {},
 			typeCountsByClass: {},
+			statistics: {
+				resourcesGathered: {
+					food: 100,
+					wood: 0,
+					metal: 0,
+					stone: 0,
+					vegetarianFood: 0, 
+				},
+				percentMapExplored: 10
+			},
 		}
 	],
 	circularMap: false,
 	timeElapsed: 0,
+	gameType: "conquest",
+	barterPrices: {buy: {food: 150}, sell: {food: 25}}
 });
 
 TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
@@ -261,6 +319,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 			team: -1,
 			teamsLocked: false,
 			cheatsEnabled: false,
+			disabledTemplates: {},
 			phase: "village",
 			isAlly: [false, false],
 			isMutualAlly: [false, false],
@@ -289,6 +348,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 				},
 				treasuresCollected: 0,
 				percentMapExplored: 10,
+				teamPercentMapExplored: 10
 			},
 		},
 		{
@@ -305,6 +365,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 			team: -1,
 			teamsLocked: false,
 			cheatsEnabled: false,
+			disabledTemplates: {},
 			phase: "village",
 			isAlly: [true, true],
 			isMutualAlly: [false, false],
@@ -333,11 +394,13 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
 				},
 				treasuresCollected: 0,
 				percentMapExplored: 10,
+				teamPercentMapExplored: 10
 			},
 		}
 	],
 	circularMap: false,
 	timeElapsed: 0,
+	gameType: "conquest",
 	barterPrices: {buy: {food: 150}, sell: {food: 25}}
 });
 
@@ -357,12 +420,14 @@ AddMock(10, IID_Health, {
 
 AddMock(10, IID_Identity, {
 	GetClassesList: function() { return ["class1", "class2"]; },
+	GetVisibleClassesList: function() { return ["class3", "class4"]; },
 	GetRank: function() { return "foo"; },
 	GetSelectionGroupName: function() { return "Selection Group Name"; },
 	HasClass: function() { return true; },
 });
 
 AddMock(10, IID_Position, {
+	GetTurretParent: function() {return INVALID_ENTITY;},
 	GetPosition: function() {
 		return {x:1, y:2, z:3};
 	},
@@ -381,21 +446,25 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetEntityState(-1, 10), {
 	id: 10,
 	template: "example",
 	alertRaiser: null,
-	buildEntities: ["test1", "test2"],
+	builder: true,
 	identity: {
 		rank: "foo",
 		classes: ["class1", "class2"],
+		visibleClasses: ["class3", "class4"],
 		selectionGroupName: "Selection Group Name",
 	},
+	fogging: null,
 	foundation: null,
 	garrisonHolder: null,
 	gate: null,
 	guard: null,
+	mirage: null,
 	pack: null,
 	player: -1,
 	position: {x:1, y:2, z:3},
 	production: null,
 	rallyPoint: null,
+	resourceCarrying: null,
 	rotation: {x:4, y:5, z:6},
 	trader: null,
 	unitAI: null,
@@ -415,8 +484,8 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedEntityState(-1, 10), {
 	buildingAI: null,
 	healer: null,
 	obstruction: null,
+	turretParent: null,
 	promotion: null,
-	resourceCarrying: null,
 	resourceDropsite: null,
 	resourceGatherRates: null,
 	resourceSupply: null,
