@@ -74,6 +74,9 @@ var g_AssignedCount = 0;
 // tick handler
 var g_LoadingState = 0; // 0 = not started, 1 = loading, 2 = loaded
 
+// Filled by scripts in victory_conditions/
+var g_VictoryConditions = {};
+
 ////////////////////////////////////////////////////////////////////////////////////////////////
 
 function init(attribs)
@@ -401,9 +404,13 @@ function initMain()
 		// to allow easy keyboard selection of maps
 		Engine.GetGUIObjectByName("mapSelection").focus();
 	}
-	// Sync g_GameAttributes to everyone.
+
 	if (g_IsController)
+	{
+		loadGameAttributes();
+		// Sync g_GameAttributes to everyone.
 		updateGameAttributes();
+	}
 }
 
 function handleNetMessage(message)
@@ -676,7 +683,7 @@ function loadMapData(name)
 				// To be defined later.
 				g_MapData[name] = { settings: { "Name": "", "Description": "" } };
 			else
-				g_MapData[name] = parseJSONData(name+".json");
+				g_MapData[name] = Engine.ReadJSONFile(name+".json");
 			break;
 
 		default:
@@ -688,11 +695,105 @@ function loadMapData(name)
 	return g_MapData[name];
 }
 
+const FILEPATH_MATCHSETTINGS_SP = "config/matchsettings.json";
+const FILEPATH_MATCHSETTINGS_MP = "config/matchsettings.mp.json";
+function loadGameAttributes()
+{
+	if (Engine.ConfigDB_GetValue("user", "persistmatchsettings") != "true")
+		return;
+
+	var settingsFile = g_IsNetworked ? FILEPATH_MATCHSETTINGS_MP : FILEPATH_MATCHSETTINGS_SP;
+	if (!Engine.FileExists(settingsFile))
+		return;
+
+	var attrs = Engine.ReadJSONFile(settingsFile);
+	if (!attrs || !attrs.settings)
+		return;
+
+	g_IsInGuiUpdate = true;
+
+	var mapName = attrs.map || "";
+	var mapSettings = attrs.settings;
+
+	g_GameAttributes = attrs;
+
+	// Assign new seeds and match id
+	g_GameAttributes.matchID = Engine.GetMatchID();
+	mapSettings.Seed = Math.floor(Math.random() * 65536);
+	mapSettings.AISeed = Math.floor(Math.random() * 65536);
+
+	// Ensure that cheats are enabled in singleplayer
+	if (!g_IsNetworked) 
+		mapSettings.CheatsEnabled = true; 
+
+	var aiCodes = [ ai.id for each (ai in g_AIs) ];
+	var civListCodes = [ civ.Code for each (civ in g_CivData) if (civ.SelectableInGameSetup !== false) ];
+	civListCodes.push("random");
+
+	var playerData = mapSettings.PlayerData;
+
+	// Validate player civs
+	if (playerData)
+	{
+		for (var i = 0; i < playerData.length; ++i)
+		{
+			if (civListCodes.indexOf(playerData[i].Civ) < 0)
+				playerData[i].Civ = "random";
+		}
+	}
+
+	// Refresh probably obsoleted/incomplete map data.
+	var newMapData = loadMapData(mapName);
+	if (newMapData && newMapData.settings)
+	{
+		for (var prop in newMapData.settings)
+			mapSettings[prop] = newMapData.settings[prop];
+
+		// Set player data
+		if (playerData)
+			mapSettings.PlayerData = playerData;
+	}
+
+	var mapFilterSelection = Engine.GetGUIObjectByName("mapFilterSelection");
+	mapFilterSelection.selected = mapFilterSelection.list_data.indexOf(attrs.mapFilter);
+
+	var mapTypeSelection = Engine.GetGUIObjectByName("mapTypeSelection");
+	mapTypeSelection.selected = mapTypeSelection.list_data.indexOf(attrs.mapType);
+
+	initMapNameList();
+
+	var mapSelectionBox = Engine.GetGUIObjectByName("mapSelection");
+	mapSelectionBox.selected = mapSelectionBox.list_data.indexOf(mapName);
+
+	if (mapSettings.PopulationCap)
+	{
+		var populationCapBox = Engine.GetGUIObjectByName("populationCap");
+		populationCapBox.selected = populationCapBox.list_data.indexOf(mapSettings.PopulationCap);
+	}
+	if (mapSettings.StartingResources)
+	{
+		var startingResourcesBox = Engine.GetGUIObjectByName("startingResources");
+		startingResourcesBox.selected = startingResourcesBox.list_data.indexOf(mapSettings.StartingResources);
+	}
+
+	g_IsInGuiUpdate = false;
+
+	onGameAttributesChange();
+}
+
+function saveGameAttributes()
+{
+	var attributes = Engine.ConfigDB_GetValue("user", "persistmatchsettings") == "true" ? g_GameAttributes : {};
+	Engine.WriteJSONFile(g_IsNetworked ? FILEPATH_MATCHSETTINGS_MP : FILEPATH_MATCHSETTINGS_SP, attributes);
+}
 ////////////////////////////////////////////////////////////////////////////////////////////////
 // GUI event handlers
 
 function cancelSetup()
 {
+	if (g_IsController)
+		saveGameAttributes();
+
 	Engine.DisconnectNetworkGame();
 
 	if (Engine.HasXmppClient())
@@ -803,9 +904,9 @@ function selectMapType(type)
 	{
 	case "scenario":
 		// Set a default map
-		// TODO: This should be remembered from the last session
 		g_GameAttributes.mapPath = "maps/scenarios/";
 		g_GameAttributes.map = g_GameAttributes.mapPath + (g_IsNetworked ? DEFAULT_NETWORKED_MAP : DEFAULT_OFFLINE_MAP);
+		g_GameAttributes.settings.AISeed = Math.floor(Math.random() * 65536);
 		break;
 
 	case "skirmish":
@@ -813,6 +914,7 @@ function selectMapType(type)
 		g_GameAttributes.settings = {
 			PlayerData: g_DefaultPlayerData.slice(0, 4),
 			Seed: Math.floor(Math.random() * 65536),
+			AISeed: Math.floor(Math.random() * 65536),
 			CheatsEnabled: g_GameAttributes.settings.CheatsEnabled
 		};
 		break;
@@ -822,6 +924,7 @@ function selectMapType(type)
 		g_GameAttributes.settings = {
 			PlayerData: g_DefaultPlayerData.slice(0, 4),
 			Seed: Math.floor(Math.random() * 65536),
+			AISeed: Math.floor(Math.random() * 65536),
 			CheatsEnabled: g_GameAttributes.settings.CheatsEnabled
 		};
 		break;
@@ -869,12 +972,19 @@ function selectMap(name)
 		return;
 
 	// reset some map specific properties which are not necessarily redefined on each map
-	g_GameAttributes.settings.TriggerScripts = undefined;
-	g_GameAttributes.settings.CircularMap = undefined;
+	for (let prop of ["TriggerScripts", "CircularMap", "Garrison"])
+		if (g_GameAttributes.settings[prop] !== undefined)
+			g_GameAttributes.settings[prop] = undefined;
 
 	var mapData = loadMapData(name);
 	var mapSettings = (mapData && mapData.settings ? deepcopy(mapData.settings) : {});
 
+	// Reset victory conditions
+	var victories = getVictoryConditions();
+	var victoryIdx = (mapSettings.GameType !== undefined && victories.data.indexOf(mapSettings.GameType) != -1 ? victories.data.indexOf(mapSettings.GameType) : VICTORY_DEFAULTIDX);
+	g_GameAttributes.settings.GameType = victories.data[victoryIdx];
+	g_GameAttributes.settings.VictoryScripts = victories.scripts[victoryIdx];
+
 	// Copy any new settings
 	g_GameAttributes.map = name;
 	g_GameAttributes.script = mapSettings.Script;
@@ -882,6 +992,10 @@ function selectMap(name)
 		for (var prop in mapSettings)
 			g_GameAttributes.settings[prop] = mapSettings[prop];
 
+	// Ignore gaia
+	if (g_GameAttributes.settings.PlayerData.length && !g_GameAttributes.settings.PlayerData[0])
+		g_GameAttributes.settings.PlayerData.shift();
+
 	// Use default AI if the map doesn't specify any explicitly
 	for (var i = 0; i < g_GameAttributes.settings.PlayerData.length; ++i)
 	{
@@ -924,6 +1038,8 @@ function launchGame()
 	if (!g_GameAttributes.map)
 		return;
 
+	saveGameAttributes();
+
 	if (g_GameAttributes.map == "random")
 		selectMap(Engine.GetGUIObjectByName("mapSelection").list_data[Math.floor(Math.random() *
 			(Engine.GetGUIObjectByName("mapSelection").list.length - 1)) + 1]);
@@ -1264,8 +1380,12 @@ function onGameAttributesChange()
 	// Load the description from the map file, if there is one
 	var description = mapSettings.Description ? translate(mapSettings.Description) : translate("Sorry, no description available.");
 
-	// Describe the number of players
-	var playerString = sprintf(translatePlural("%(number)s player. %(description)s", "%(number)s players. %(description)s", numPlayers), { number: numPlayers, description: description });
+	// Describe the number of players and the victory conditions
+	var playerString = sprintf(translatePlural("%(number)s player. ", "%(number)s players. ", numPlayers), { number: numPlayers });
+	let victory = translate(victories.text[victoryIdx]);
+	if (victoryIdx != VICTORY_DEFAULTIDX)
+		victory = "[color=\"orange\"]" + victory + "[/color]";
+	playerString += translate("Victory Condition:") + " " + victory + ".\n\n" + description;
 
 	for (var i = 0; i < MAX_PLAYERS; ++i)
 	{
@@ -1290,7 +1410,7 @@ function onGameAttributesChange()
 		var pDefs = g_DefaultPlayerData ? g_DefaultPlayerData[i] : {};
 
 		// Common to all game types
-		var color = iColorToString(getSetting(pData, pDefs, "Colour"));
+		var color = rgbToGuiColor(getSetting(pData, pDefs, "Colour"));
 		pColor.sprite = "colour:" + color + " 100";
 		pName.caption = translate(getSetting(pData, pDefs, "Name"));
 
@@ -1599,7 +1719,7 @@ function addChatMessage(msg)
 		var pData = mapSettings.PlayerData ? mapSettings.PlayerData[player] : {};
 		var pDefs = g_DefaultPlayerData ? g_DefaultPlayerData[player] : {};
 
-		color = iColorToString(getSetting(pData, pDefs, "Colour"));
+		color = rgbToGuiColor(getSetting(pData, pDefs, "Colour"));
 	}
 
 	var formatted;
@@ -1829,7 +1949,7 @@ function sendRegisterGameStanza()
 
 	// Map sizes only apply to random maps.
 	if (g_GameAttributes.mapType == "random")
-		var mapSize = Engine.GetGUIObjectByName("mapSize").list[selectedMapSize];
+		var mapSize = Engine.GetGUIObjectByName("mapSize").list_data[selectedMapSize];
 	else
 		var mapSize = "Default";
 
