@@ -55,7 +55,10 @@ var commands = {
 	"aichat": function(player, cmd, data)
 	{
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGuiInterface.PushNotification({"type": cmd.type, "players": [player], "message": cmd.message});
+		var notification = { "players": [player] };
+		for (var key in cmd)
+			notification[key] = cmd[key];
+		cmpGuiInterface.PushNotification(notification);
 	},
 
 	"cheat": function(player, cmd, data)
@@ -545,7 +548,7 @@ var commands = {
 		for each (var ent in data.entities)
 		{
 			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
-			if (cmpUnitAI)
+			if (cmpUnitAI && !cmpUnitAI.IsTurret())
 				cmpUnitAI.SwitchToStance(cmd.name);
 		}
 	},
@@ -912,8 +915,7 @@ function TryConstructBuilding(player, cmpPlayer, controlAllUnits, cmd)
 	
 	var cmpTechnologyManager = QueryPlayerIDInterface(player, IID_TechnologyManager);
 	
-	// TODO: Enable this check once the AI gets technology support 
-	if (!cmpTechnologyManager.CanProduce(cmd.template) && !cmpPlayer.IsAI()) 
+	if (!cmpTechnologyManager.CanProduce(cmd.template))
 	{
 		if (g_DebugCommands)
 		{
@@ -973,11 +975,6 @@ function TryConstructBuilding(player, cmpPlayer, controlAllUnits, cmd)
 			"queued": cmd.queued
 		});
 	}
-	
-	// Load a mirage for the owner of this new entity
-	var cmpFogging = Engine.QueryInterface(ent, IID_Fogging)
-	if (cmpFogging)
-		cmpFogging.LoadMirage(player);
 
 	return ent;
 }
@@ -1273,10 +1270,15 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 		// TODO: We only check if the formation is usable by some units
 		// if we move them to it. We should check if we can use formations
 		// for the other cases.
-		if (cmpIdentity && cmpIdentity.CanUseFormation(formationTemplate || "formations/line_closed"))
+		var nullFormation = (formationTemplate || cmpUnitAI.GetLastFormationTemplate()) == "formations/null";
+		if (!nullFormation && cmpIdentity && cmpIdentity.CanUseFormation(formationTemplate || "formations/line_closed"))
 			formedEnts.push(ent);
 		else
+		{
+			if (nullFormation)
+				cmpUnitAI.SetLastFormationTemplate("formations/null");
 			nonformedUnitAIs.push(cmpUnitAI);
+		}
 	}
 
 	if (formedEnts.length == 0)
@@ -1317,6 +1319,7 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 			if (cmpFormation)
 				cmpFormation.RemoveMembers(formation.members[fid]);
 		}
+
 		// TODO replace the fixed 60 with something sensible, based on vision range f.e.
 		var formationSeparation = 60;
 		var clusters = ClusterEntities(formation.entities, formationSeparation);
@@ -1356,7 +1359,7 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 			formationUnitAIs.push(Engine.QueryInterface(formationEnt, IID_UnitAI));
 			cmpFormation.SetFormationSeparation(formationSeparation);
 			cmpFormation.SetMembers(cluster);
-			
+
 			for each (var ent in formationEnts)
 				cmpFormation.RegisterTwinFormation(ent);
 
