@@ -4,8 +4,9 @@
  * all other initialization must be done after loading map (terrain/entities).
  * DO NOT use other components here, as they may fail unpredictably.
  * settings is the object containing settings for this map.
- * newPlayers if true will remove any old player entities and add new ones
- *	(used when loading a map or when Atlas changes the number of players).
+ * newPlayers if true will remove old player entities or add new ones until
+ * the new number of player entities is obtained
+ * (used when loading a map or when Atlas changes the number of players).
  */
 function LoadPlayerSettings(settings, newPlayers)
 {
@@ -18,129 +19,140 @@ function LoadPlayerSettings(settings, newPlayers)
 	if (!(rawData && rawData.PlayerData))
 		throw("Player.js: Error reading player_defaults.json");
 
-	var playerDefaults = rawData.PlayerData;
+	// Add gaia to simplify iteration
+	if (settings.PlayerData && settings.PlayerData[0])
+		settings.PlayerData.unshift(null);
 
-	// default number of players
-	var numPlayers = 8;
+	var playerDefaults = rawData.PlayerData;
+	var playerData = settings.PlayerData;
 
 	// Get player manager
 	var cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+	var numPlayers = cmpPlayerManager.GetNumPlayers();
 
-	// Remove existing players and add new ones
+	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+
+	// Remove existing players or add new ones
 	if (newPlayers)
 	{
-		cmpPlayerManager.RemoveAllPlayers();
+		var settingsNumPlayers = 9; // default 8 players + gaia
 
-		if (settings.PlayerData)
-		{
-			// Get number of players including gaia
-			numPlayers = settings.PlayerData.length + 1;
-		}
+		if (playerData)
+			settingsNumPlayers = playerData.length; // includes gaia (see above)
 		else
-		{
 			warn("Player.js: Setup has no player data - using defaults");
-		}
 
-		for (var i = 0; i < numPlayers; ++i)
+		while (settingsNumPlayers > numPlayers)
 		{
 			// Add player entity to engine
-			// TODO: Get player template name from civ data
-			var entID = Engine.AddEntity("special/player");
+			var civ = getSetting(playerData, playerDefaults, numPlayers, "Civ");
+			var template = cmpTemplateManager.TemplateExists("special/player_"+civ) ? "special/player_"+civ : "special/player";
+			var entID = Engine.AddEntity(template);
 			var cmpPlayer = Engine.QueryInterface(entID, IID_Player);
 			if (!cmpPlayer)
-				throw("Player.js: Error creating player entity "+i);
+				throw("Player.js: Error creating player entity " + numPlayers);
 
-			cmpPlayer.SetPlayerID(i);
-
-			// Add player to player manager
 			cmpPlayerManager.AddPlayer(entID);
+			++numPlayers;
+		}
 
-			// Properly autoresearch techs on init.
-			var cmpTechManager = Engine.QueryInterface(entID, IID_TechnologyManager);
-			if (cmpTechManager !== undefined)
-				cmpTechManager.UpdateAutoResearch();
+		while (settingsNumPlayers < numPlayers)
+		{
+			cmpPlayerManager.RemoveLastPlayer();
+			--numPlayers;
 		}
 	}
 
-	numPlayers = cmpPlayerManager.GetNumPlayers();
+	// Even when no new player, we must check the template compatibility as player template may be civ dependent
+	for (var i = 0; i < numPlayers; ++i)
+	{
+		var civ = getSetting(playerData, playerDefaults, i, "Civ");
+		var template = cmpTemplateManager.TemplateExists("special/player_"+civ) ? "special/player_"+civ : "special/player";
+		var entID = cmpPlayerManager.GetPlayerByID(i);
+		if (cmpTemplateManager.GetCurrentTemplateName(entID) === template)
+			continue;
+		// We need to recreate this player to have the right template
+		entID = Engine.AddEntity(template);
+		cmpPlayerManager.ReplacePlayer(i, entID);
+	}
 
 	// Initialize the player data
 	for (var i = 0; i < numPlayers; ++i)
 	{
 		var cmpPlayer = Engine.QueryInterface(cmpPlayerManager.GetPlayerByID(i), IID_Player);
-		var pDefs = playerDefaults ? playerDefaults[i] : {};
+		cmpPlayer.SetName(getSetting(playerData, playerDefaults, i, "Name"));
+		cmpPlayer.SetCiv(getSetting(playerData, playerDefaults, i, "Civ"));
+		var colour = getSetting(playerData, playerDefaults, i, "Colour");
+		cmpPlayer.SetColour(colour.r, colour.g, colour.b);
 
-		// Skip gaia
-		if (i > 0)
+		// Special case for gaia
+		if (i == 0)
 		{
-			var pData = settings.PlayerData ? settings.PlayerData[i-1] : {};
-
-			cmpPlayer.SetName(getSetting(pData, pDefs, "Name"));
-			cmpPlayer.SetCiv(getSetting(pData, pDefs, "Civ"));
-			var colour = getSetting(pData, pDefs, "Colour");
-			cmpPlayer.SetColour(colour.r, colour.g, colour.b);
-
-			// Note: this is not yet implemented but I leave it commented to highlight it's easy
-			// If anyone ever adds handicap.
-			//if (getSetting(pData, pDefs, "GatherRateMultiplier") !== undefined)
-			//	cmpPlayer.SetGatherRateMultiplier(getSetting(pData, pDefs, "GatherRateMultiplier"));
-
-			if (getSetting(pData, pDefs, "PopulationLimit") !== undefined)
-				cmpPlayer.SetMaxPopulation(getSetting(pData, pDefs, "PopulationLimit"));
-
-			if (getSetting(pData, pDefs, "Resources") !== undefined)
-				cmpPlayer.SetResourceCounts(getSetting(pData, pDefs, "Resources"));
-
-			// If diplomacy explicitly defined, use that; otherwise use teams
-			if (getSetting(pData, pDefs, "Diplomacy") !== undefined)
-				cmpPlayer.SetDiplomacy(getSetting(pData, pDefs, "Diplomacy"));
-			else
-			{
-				// Init diplomacy
-				var myTeam = getSetting(pData, pDefs, "Team");
-
-				// Set all but self as enemies as SetTeam takes care of allies
-				for (var j = 0; j < numPlayers; ++j)
-				{
-					if (i == j)
-						cmpPlayer.SetAlly(j);
-					else
-						cmpPlayer.SetEnemy(j);
-				}
-				cmpPlayer.SetTeam((myTeam !== undefined) ? myTeam : -1);
-			}
-
-			// If formations explicitly defined, use that; otherwise use civ defaults
-			var formations = getSetting(pData, pDefs, "Formations");
-			if (formations !== undefined)
-				cmpPlayer.SetFormations(formations);
-			else
-			{
-				var rawFormations = Engine.ReadCivJSONFile(cmpPlayer.GetCiv()+".json");
-				if (!(rawFormations && rawFormations.Formations))
-					throw("Player.js: Error reading "+cmpPlayer.GetCiv()+".json");
-
-				cmpPlayer.SetFormations(rawFormations.Formations);
-			}
-
-			var startCam = getSetting(pData, pDefs, "StartingCamera");
-			if (startCam !== undefined)
-				cmpPlayer.SetStartingCamera(startCam.Position, startCam.Rotation);
-		}
-		else
-		{
-			// Copy gaia data from defaults
-			cmpPlayer.SetName(pDefs.Name);
-			cmpPlayer.SetCiv(pDefs.Civ);
-			cmpPlayer.SetColour(pDefs.Colour.r, pDefs.Colour.g, pDefs.Colour.b);
-
 			// Gaia should be its own ally.
 			cmpPlayer.SetAlly(0);
 
 			// Gaia is everyone's enemy
 			for (var j = 1; j < numPlayers; ++j)
 				cmpPlayer.SetEnemy(j);
+
+			continue;
 		}
+
+		// Note: this is not yet implemented but I leave it commented to highlight it's easy
+		// If anyone ever adds handicap.
+		//if (getSetting(playerData, playerDefaults, i, "GatherRateMultiplier") !== undefined)
+		//	cmpPlayer.SetGatherRateMultiplier(getSetting(playerData, playerDefaults, i, "GatherRateMultiplier"));
+
+		if (getSetting(playerData, playerDefaults, i, "PopulationLimit") !== undefined)
+			cmpPlayer.SetMaxPopulation(getSetting(playerData, playerDefaults, i, "PopulationLimit"));
+
+		if (getSetting(playerData, playerDefaults, i, "Resources") !== undefined)
+			cmpPlayer.SetResourceCounts(getSetting(playerData, playerDefaults, i, "Resources"));
+
+		if (getSetting(playerData, playerDefaults, i, "StartingTechnologies") !== undefined)
+			cmpPlayer.SetStartingTechnologies(getSetting(playerData, playerDefaults, i, "StartingTechnologies"));
+
+		if (getSetting(playerData, playerDefaults, i, "DisabledTechnologies") !== undefined)
+			cmpPlayer.SetDisabledTechnologies(getSetting(playerData, playerDefaults, i, "DisabledTechnologies"));
+
+		if (getSetting(playerData, playerDefaults, i, "DisabledTemplates") !== undefined)
+			cmpPlayer.SetDisabledTemplates(getSetting(playerData, playerDefaults, i, "DisabledTemplates"));
+
+		// If diplomacy explicitly defined, use that; otherwise use teams
+		if (getSetting(playerData, playerDefaults, i, "Diplomacy") !== undefined)
+			cmpPlayer.SetDiplomacy(getSetting(playerData, playerDefaults, i, "Diplomacy"));
+		else
+		{
+			// Init diplomacy
+			var myTeam = getSetting(playerData, playerDefaults, i, "Team");
+
+			// Set all but self as enemies as SetTeam takes care of allies
+			for (var j = 0; j < numPlayers; ++j)
+			{
+				if (i == j)
+					cmpPlayer.SetAlly(j);
+				else
+					cmpPlayer.SetEnemy(j);
+			}
+			cmpPlayer.SetTeam((myTeam !== undefined) ? myTeam : -1);
+		}
+
+		// If formations explicitly defined, use that; otherwise use civ defaults
+		var formations = getSetting(playerData, playerDefaults, i, "Formations");
+		if (formations !== undefined)
+			cmpPlayer.SetFormations(formations);
+		else
+		{
+			var rawFormations = Engine.ReadCivJSONFile(cmpPlayer.GetCiv()+".json");
+			if (!(rawFormations && rawFormations.Formations))
+				throw("Player.js: Error reading "+cmpPlayer.GetCiv()+".json");
+
+			cmpPlayer.SetFormations(rawFormations.Formations);
+		}
+
+		var startCam = getSetting(playerData, playerDefaults, i, "StartingCamera");
+		if (startCam !== undefined)
+			cmpPlayer.SetStartingCamera(startCam.Position, startCam.Rotation);
 	}
 
 	// NOTE: We need to do the team locking here, as otherwise
@@ -154,14 +166,14 @@ function LoadPlayerSettings(settings, newPlayers)
 }
 
 // Get a setting if it exists or return default
-function getSetting(settings, defaults, property)
+function getSetting(settings, defaults, idx, property)
 {
-	if (settings && (property in settings))
-		return settings[property];
+	if (settings && settings[idx] && (property in settings[idx]))
+		return settings[idx][property];
 
 	// Use defaults
-	if (defaults && (property in defaults))
-		return defaults[property];
+	if (defaults && defaults[idx] && (property in defaults[idx]))
+		return defaults[idx][property];
 
 	return undefined;
 }
