@@ -6,6 +6,8 @@ var g_specialKey = Math.random();
 var g_spamMonitor = {};
 var g_timestamp = Engine.ConfigDB_GetValue("user", "lobby.chattimestamp") == "true";
 var g_mapSizes = {};
+const g_mapTypesText = [translateWithContext("map", "Skirmish"), translateWithContext("map", "Random"), translate("Scenario")];
+const g_mapTypes = ["skirmish", "random", "scenario"];
 var g_userRating = ""; // Rating of user, defaults to Unrated
 var g_modPrefix = "@";
 // Block spammers for 30 seconds.
@@ -34,8 +36,8 @@ function init(attribs)
 	playersNumberFilter.list_data = ["",2,3,4,5,6,7,8];
 
 	var mapTypeFilter = Engine.GetGUIObjectByName("mapTypeFilter");
-	mapTypeFilter.list = [translateWithContext("map", "Any"), translateWithContext("map", "Skirmish"), translateWithContext("map", "Random"), translate("Scenario")];
-	mapTypeFilter.list_data = ["", "skirmish", "random", "scenario"];
+	mapTypeFilter.list = [translateWithContext("map", "Any")].concat(g_mapTypesText);
+	mapTypeFilter.list_data = [""].concat(g_mapTypes);
 
 	Engine.LobbySetPlayerPresence("available");
 	Engine.SendGetGameList();
@@ -77,7 +79,7 @@ function resetFilters()
 	Engine.GetGUIObjectByName("mapSizeFilter").selected = 0
 	Engine.GetGUIObjectByName("playersNumberFilter").selected = 0;
 	Engine.GetGUIObjectByName("mapTypeFilter").selected = 0;
-	Engine.GetGUIObjectByName("showFullFilter").checked = true;
+	Engine.GetGUIObjectByName("showFullFilter").checked = false;
 
 	// Update the list of games
 	updateGameList();
@@ -131,7 +133,7 @@ function updateSubject(newSubject)
 	var subjectBox = Engine.GetGUIObjectByName("subjectBox");
 	var logo = Engine.GetGUIObjectByName("logo");
 	// Load new subject and un-escape newlines.
-	subject.caption = newSubject.replace("\\n", "\n", "g");
+	subject.caption = newSubject;
 	// If the subject is only whitespace, hide it and reposition the logo.
 	if (subject.caption.match(/^([\s\t\r\n]*)$/g))
 	{
@@ -153,7 +155,10 @@ function updateSubject(newSubject)
 function updatePlayerList()
 {
 	var playersBox = Engine.GetGUIObjectByName("playersBox");
-	[playerList, presenceList, nickList, ratingList] = [[],[],[],[]];
+	var playerList = [];
+	var presenceList = [];
+	var nickList = [];
+	var ratingList = [];
 	var cleanPlayerList = Engine.GetPlayerList();
 	// Sort the player list, ignoring case.
 	cleanPlayerList.sort(function(a,b) 
@@ -387,7 +392,7 @@ function updateGameList()
 		if(!filterGame(g))
 		{
 			// 'waiting' games are highlighted in orange, 'running' in red, and 'init' in green.
-			var name;
+			let name;
 			if (g.state == 'init')
 				name = '[color="0 125 0"]' + g.name + '[/color]';
 			else if (g.state == 'waiting')
@@ -397,8 +402,9 @@ function updateGameList()
 			list_name.push(name);
 			list_ip.push(g.ip);
 			list_mapName.push(translate(g.niceMapName));
-			list_mapSize.push(g.mapSize.split("(")[0]);
-			list_mapType.push(translate(toTitleCase(g.mapType)));
+			list_mapSize.push(translatedMapSize(g.mapSize));
+			let idx = g_mapTypes.indexOf(g.mapType);
+			list_mapType.push(idx != -1 ? g_mapTypesText[idx] : "");
 			list_nPlayers.push(g.nbp + "/" +g.tnbp);
 			list.push(g.name);
 			list_data.push(c);
@@ -407,7 +413,6 @@ function updateGameList()
 	}
 
 	gamesBox.list_name = list_name;
-	//gamesBox.list_ip = list_ip;
 	gamesBox.list_mapName = list_mapName;
 	gamesBox.list_mapSize = list_mapSize;
 	gamesBox.list_mapType = list_mapType;
@@ -470,14 +475,22 @@ function formatPlayerListEntry(nickname, presence, rating)
 		nickname = g_modPrefix + nickname;
 	var formattedName = colorPlayerName(nickname);
 
-	// Give moderators special formatting.
-	if (role == "moderator")
-		formattedName = formattedName; //TODO
-
 	// Push this player's name and status onto the list
 	return [formattedName, formattedStatus, formattedRating];
 }
 
+/**
+ * Given a map size, returns that map size translated into the current
+ * language.
+ */
+function translatedMapSize(mapSize)
+{
+	if (+mapSize !== +mapSize) // NaN
+		return translate(mapSize);
+	else
+		return g_mapSizes.shortNames[g_mapSizes.tiles.indexOf(+mapSize)];
+}
+
 /**
  * Populate the game info area with information on the current game selection.
  */
@@ -500,7 +513,7 @@ function updateGameSelection()
 	if (g_GameList[g].mapType == "random" && g_GameList[g].mapName == "random")
 		mapData = {"settings": {"Description": translate("A randomly selected map.")}};
 	else if (g_GameList[g].mapType == "random" && Engine.FileExists(g_GameList[g].mapName + ".json"))
-		mapData = parseJSONData(g_GameList[g].mapName + ".json");
+		mapData = Engine.ReadJSONFile(g_GameList[g].mapName + ".json");
 	else if (Engine.FileExists(g_GameList[g].mapName + ".xml"))
 		mapData = Engine.LoadMapSettings(g_GameList[g].mapName + ".xml");
 	else
@@ -516,8 +529,9 @@ function updateGameSelection()
 	Engine.GetGUIObjectByName("sgMapName").caption = translate(g_GameList[g].niceMapName);
 	Engine.GetGUIObjectByName("sgNbPlayers").caption = g_GameList[g].nbp + "/" + g_GameList[g].tnbp;
 	Engine.GetGUIObjectByName("sgPlayersNames").caption = g_GameList[g].players;
-	Engine.GetGUIObjectByName("sgMapSize").caption = g_GameList[g].mapSize.split("(")[0];
-	Engine.GetGUIObjectByName("sgMapType").caption = translate(toTitleCase(g_GameList[g].mapType));
+	Engine.GetGUIObjectByName("sgMapSize").caption = translatedMapSize(g_GameList[g].mapSize);
+	let idx = g_mapTypes.indexOf(g_GameList[g].mapType);
+	Engine.GetGUIObjectByName("sgMapType").caption = idx != -1 ? g_mapTypesText[idx] : "";
 
 	// Display map description if it exists, otherwise display a placeholder.
 	if (mapData && mapData.settings.Description)
@@ -729,7 +743,7 @@ function onTick()
 function submitChatInput()
 {
 	var input = Engine.GetGUIObjectByName("chatInput");
-	var text = escapeText(input.caption);
+	var text = input.caption;
 	if (text.length)
 	{
 		if (!handleSpecialCommand(text) && !isSpam(text, g_Name))
@@ -738,35 +752,6 @@ function submitChatInput()
 	}
 }
 
-function completeNick()
-{
-	var input = Engine.GetGUIObjectByName("chatInput");
-	var text = escapeText(input.caption);
-	if (text.length)
-	{
-		var matched = false;
-		for each (var playerObj in Engine.GetPlayerList())
-		{
-			var player = playerObj.name;
-			var breaks = text.match(/(\s+)/g) || [];
-			text.split(/\s+/g).reduceRight(function (wordsSoFar, word, index)
-			{
-				if (matched)
-					return null;
-				var matchCandidate = word + (breaks[index - 1] || "") + wordsSoFar;
-				if (player.toUpperCase().indexOf(matchCandidate.toUpperCase().trim()) == 0)
-				{
-					input.caption = text.replace(matchCandidate.trim(), player);
-					matched = true;
-				}
-				return matchCandidate;
-			}, "");
-		if (matched)
-			break;
-		}
-	}
-}
-
 function isValidNick(nick)
 {
 	var prohibitedNicks = ["system"];
@@ -936,7 +921,7 @@ function ircFormat(text, from, color, key)
 		var timeString = Engine.FormatMillisecondsIntoDateString(time.getTime(), translate("HH:mm"));
 
 		// Translation: Time prefix as shown in the multiplayer lobby (when you enable it in the options page).
-		var timePrefixString = '[font="sans-bold-13"]' + sprintf(translate("[%(time)s]"), { time: timeString }) + '[/font]';
+		var timePrefixString = '[font="sans-bold-13"]' + sprintf(translate("\\[%(time)s]"), { time: timeString }) + '[/font]';
 
 		// Translation: IRC message format when there is a time prefix.
 		return sprintf(translate("%(time)s %(message)s"), { time: timePrefixString, message: formattedMessage });
