@@ -394,7 +394,8 @@ RMS.SetProgress(85);
 // create straggler trees
 var types = [oTree1, oTree2, oTree4, oTree3];	// some variation
 createStragglerTrees(types, avoidClasses(clWater, 5, clForest, 7, clHill, 1, clPlayer, 12, clMetal, 1, clRock, 1));
-setWaterWaviness(10);
+setWaterWaviness(4.0);
+setWaterType("lake");
 
 // Export map data
 ExportMap();
