@@ -32,11 +32,11 @@ const INPUT_MASSTRIBUTING = 10;
 
 var inputState = INPUT_NORMAL;
 
+const INVALID_ENTITY = 0;
+
 var mouseX = 0;
 var mouseY = 0;
 var mouseIsOverObject = false;
-// Distance to search for a selatable entity in. Bigger numbers are slower.
-var SELECTION_SEARCH_RADIUS = 100;
 
 // Number of pixels the mouse can move before the action is considered a drag
 var maxDragDelta = 4;
@@ -139,14 +139,14 @@ function updateBuildingPlacementPreview()
 				return false;
 			}
 
-			if (placementSupport.attack)
+			if (placementSupport.attack && placementSupport.attack.Ranged)
 			{
 				// building can be placed here, and has an attack
 				// show the range advantage in the tooltip
 				var cmd = {x: placementSupport.position.x, 
 				    z: placementSupport.position.z,
-				    range: placementSupport.attack.maxRange,
-				    elevationBonus: placementSupport.attack.elevationBonus,
+				    range: placementSupport.attack.Ranged.maxRange,
+				    elevationBonus: placementSupport.attack.Ranged.elevationBonus,
 				};
 				var averageRange = Engine.GuiInterfaceCall("GetAverageRangeForBuildings",cmd);
 				placementSupport.tooltipMessage = sprintf(translate("Basic range: %(range)s"), { range: Math.round(cmd.range/4) }) + "\n" + sprintf(translate("Average bonus range: %(range)s"), { range: Math.round((averageRange - cmd.range)/4) });
@@ -278,13 +278,13 @@ function determineAction(x, y, fromMinimap)
 	if (!g_DevSettings.controlAll && !allOwnedByPlayer)
 		return undefined;
 
-	var targets = [];
 	var target = undefined;
 	if (!fromMinimap)
-		targets = Engine.PickEntitiesAtPoint(x, y, SELECTION_SEARCH_RADIUS);
-
-	if (targets.length)
-		target = targets[0];
+	{
+		var ent = Engine.PickEntityAtPoint(x, y);
+		if (ent != INVALID_ENTITY)
+			target = ent;
+	}
 
 	// decide between the following ordered actions
 	// if two actions are possible, the first one is taken
@@ -437,44 +437,76 @@ function tryPlaceWall(queued)
 	return true;
 }
 
-// Limits bandboxed selections to certain types of entities based on priority
-function getPreferredEntities(ents)
+// Updates the bandbox object with new positions and visibility.
+// The coordinates [x0, y0, x1, y1] are returned for further use.
+function updateBandbox(bandbox, ev, hidden)
 {
-	var entStateList = [];
-	var preferredEnts = [];
+	var x0 = dragStart[0];
+	var y0 = dragStart[1];
+	var x1 = ev.x;
+	var y1 = ev.y;
+	// normalize the orientation of the rectangle
+	if (x0 > x1) { let t = x0; x0 = x1; x1 = t; }
+	if (y0 > y1) { let t = y0; y0 = y1; y1 = t; }
 
-	// Check if there are units in the selection and get a list of entity states
-	for each (var ent in ents)
-	{
-		var entState = GetEntityState(ent);
-		if (!entState)
-			continue;
-		if (hasClass(entState, "Unit"))
-			preferredEnts.push(ent);
+	bandbox.size = [x0, y0, x1, y1].join(" ");
+	bandbox.hidden = hidden;
 
-		entStateList.push(entState);
-	}
-
-	// If there are no units, check if there are defensive entities in the selection
-	if (!preferredEnts.length)
-		for (var i = 0; i < ents.length; i++)
-			if (hasClass(entStateList[i], "Defensive"))
-				preferredEnts.push(ents[i]);
-
-	return preferredEnts;
+	return [x0, y0, x1, y1];
 }
 
-// Removes any support units from the passed list of entities
-function getMilitaryEntities(ents)
-{
-	var militaryEnts = [];
-	for each (var ent in ents)
-	{
-		var entState = GetEntityState(ent);
-		if (!hasClass(entState, "Support"))
-			militaryEnts.push(ent);
+// Define some useful unit filters for getPreferredEntities
+var unitFilters = {
+	"isUnit": function (entity) {
+		var entState = GetEntityState(entity);
+		if (!entState)
+			return false;
+		return hasClass(entState, "Unit");
+	},
+	"isDefensive": function (entity) {
+		var entState = GetEntityState(entity);
+		if (!entState)
+			return false;
+		return hasClass(entState, "Defensive");
+	},
+	"isNotSupport": function (entity) {
+		var entState = GetEntityState(entity);
+		if (!entState)
+			return false;
+		return hasClass(entState, "Unit") && !hasClass(entState, "Support") && !hasClass(entState, "Domestic");
+	},
+	"isIdle": function (entity) {
+		var entState = GetEntityState(entity);
+		if (!entState)
+			return false;
+		return hasClass(entState, "Unit") && entState.unitAI.isIdle;
+	},
+	"isAnything": function (entity) {
+		return true;
 	}
-	return militaryEnts;
+};
+
+// Choose, inside a list of entities, which ones will be selected.
+// We may use several entity filters, until one returns at least one element.
+function getPreferredEntities(ents)
+{
+	// Default filters
+	var filters = [unitFilters.isUnit, unitFilters.isDefensive, unitFilters.isAnything];
+
+	// Handle hotkeys
+	if (Engine.HotkeyIsPressed("selection.milonly"))
+		filters = [unitFilters.isNotSupport];
+	if (Engine.HotkeyIsPressed("selection.idleonly"))
+		filters = [unitFilters.isIdle];
+
+	var preferredEnts = [];
+	for (var i = 0; i < filters.length; ++i)
+	{
+		preferredEnts = ents.filter(filters[i]);
+		if (preferredEnts.length > 0) 
+			break;
+	}
+	return preferredEnts;
 }
 
 function handleInputBeforeGui(ev, hoveredObject)
@@ -511,55 +543,25 @@ function handleInputBeforeGui(ev, hoveredObject)
 	switch (inputState)
 	{
 	case INPUT_BANDBOXING:
+		var bandbox = Engine.GetGUIObjectByName("bandbox");
 		switch (ev.type)
 		{
 		case "mousemotion":
-			var x0 = dragStart[0];
-			var y0 = dragStart[1];
-			var x1 = ev.x;
-			var y1 = ev.y;
-			if (x0 > x1) { var t = x0; x0 = x1; x1 = t; }
-			if (y0 > y1) { var t = y0; y0 = y1; y1 = t; }
+			var rect = updateBandbox(bandbox, ev, false);
 
-			var bandbox = Engine.GetGUIObjectByName("bandbox");
-			bandbox.size = [x0, y0, x1, y1].join(" ");
-			bandbox.hidden = false;
-
-			// TODO: Should we handle "control all units" here as well?
-			var ents = Engine.PickFriendlyEntitiesInRect(x0, y0, x1, y1, Engine.GetPlayerID());
-			g_Selection.setHighlightList(ents);
+			var ents = Engine.PickFriendlyEntitiesInRect(rect[0], rect[1], rect[2], rect[3], Engine.GetPlayerID());
+			var preferredEntities = getPreferredEntities(ents);
+			g_Selection.setHighlightList(preferredEntities);
 
 			return false;
 
 		case "mousebuttonup":
 			if (ev.button == SDL_BUTTON_LEFT)
 			{
-				var x0 = dragStart[0];
-				var y0 = dragStart[1];
-				var x1 = ev.x;
-				var y1 = ev.y;
-				if (x0 > x1) { var t = x0; x0 = x1; x1 = t; }
-				if (y0 > y1) { var t = y0; y0 = y1; y1 = t; }
-
-				var bandbox = Engine.GetGUIObjectByName("bandbox");
-				bandbox.hidden = true;
+				var rect = updateBandbox(bandbox, ev, true);
 
 				// Get list of entities limited to preferred entities
-				// TODO: Should we handle "control all units" here as well?
-				var ents = Engine.PickFriendlyEntitiesInRect(x0, y0, x1, y1, Engine.GetPlayerID());
-				var preferredEntities = getPreferredEntities(ents)
-
-				if (preferredEntities.length)
-				{
- 					ents = preferredEntities;
-
-					if (Engine.HotkeyIsPressed("selection.milonly"))
-					{
-						var militaryEntities = getMilitaryEntities(ents);
-						if (militaryEntities.length)
-							ents = militaryEntities;
-					}
-				}
+				var ents = getPreferredEntities(Engine.PickFriendlyEntitiesInRect(rect[0], rect[1], rect[2], rect[3], Engine.GetPlayerID()));
 
 				// Remove the bandbox hover highlighting
 				g_Selection.setHighlightList([]);
@@ -585,7 +587,6 @@ function handleInputBeforeGui(ev, hoveredObject)
 			else if (ev.button == SDL_BUTTON_RIGHT)
 			{
 				// Cancel selection
-				var bandbox = Engine.GetGUIObjectByName("bandbox");
 				bandbox.hidden = true;
 
 				g_Selection.setHighlightList([]);
@@ -840,11 +841,11 @@ function handleInputAfterGui(ev)
 	// Handle the time-warp testing features, restricted to single-player
 	if (!g_IsNetworked && Engine.GetGUIObjectByName("devTimeWarp").checked)
 	{
-		if (ev.type == "hotkeydown" && ev.hotkey == "timewarp.fastforward")
+		if (ev.type == "hotkeydown" && ev.hotkey == "session.timewarp.fastforward")
 			Engine.SetSimRate(20.0);
-		else if (ev.type == "hotkeyup" && ev.hotkey == "timewarp.fastforward")
+		else if (ev.type == "hotkeyup" && ev.hotkey == "session.timewarp.fastforward")
 			Engine.SetSimRate(1.0);
-		else if (ev.type == "hotkeyup" && ev.hotkey == "timewarp.rewind")
+		else if (ev.type == "hotkeyup" && ev.hotkey == "session.timewarp.rewind")
 			Engine.RewindTimeWarp();
 	}
 
@@ -852,15 +853,13 @@ function handleInputAfterGui(ev)
 	{
 		g_ShowAllStatusBars = (ev.type == "hotkeydown");
 		recalculateStatusBarDisplay();
-	}
-
-	if (ev.hotkey == "session.highlightguarding")
+	} 
+	else if (ev.hotkey == "session.highlightguarding")
 	{
 		g_ShowGuarding = (ev.type == "hotkeydown");
 		updateAdditionalHighlight();
 	}
-
-	if (ev.hotkey == "session.highlightguarded")
+	else if (ev.hotkey == "session.highlightguarded")
 	{
 		g_ShowGuarded = (ev.type == "hotkeydown");
 		updateAdditionalHighlight();
@@ -875,9 +874,9 @@ function handleInputAfterGui(ev)
 		{
 		case "mousemotion":
 			// Highlight the first hovered entity (if any)
-			var ents = Engine.PickEntitiesAtPoint(ev.x, ev.y, SELECTION_SEARCH_RADIUS);
-			if (ents.length)
-				g_Selection.setHighlightList([ents[0]]);
+			var ent = Engine.PickEntityAtPoint(ev.x, ev.y);
+			if (ent != INVALID_ENTITY)
+				g_Selection.setHighlightList([ent]);
 			else
 				g_Selection.setHighlightList([]);
 
@@ -929,9 +928,9 @@ function handleInputAfterGui(ev)
 		{
 		case "mousemotion":
 			// Highlight the first hovered entity (if any)
-			var ents = Engine.PickEntitiesAtPoint(ev.x, ev.y, SELECTION_SEARCH_RADIUS);
-			if (ents.length)
-				g_Selection.setHighlightList([ents[0]]);
+			var ent = Engine.PickEntityAtPoint(ev.x, ev.y);
+			if (ent != INVALID_ENTITY)
+				g_Selection.setHighlightList([ent]);
 			else
 				g_Selection.setHighlightList([]);
 
@@ -979,15 +978,19 @@ function handleInputAfterGui(ev)
 				return false;
 			}
 
-			var ents = Engine.PickEntitiesAtPoint(ev.x, ev.y, SELECTION_SEARCH_RADIUS);
-			g_Selection.setHighlightList(ents);
+			var ent = Engine.PickEntityAtPoint(ev.x, ev.y);
+			if (ent != INVALID_ENTITY)
+				g_Selection.setHighlightList([ent]);
+			else
+				g_Selection.setHighlightList([]);
 			return false;
 
 		case "mousebuttonup":
 			if (ev.button == SDL_BUTTON_LEFT)
 			{
-				var ents = Engine.PickEntitiesAtPoint(ev.x, ev.y, SELECTION_SEARCH_RADIUS);
-				if (!ents.length)
+				var ents = [];
+				var selectedEntity = Engine.PickEntityAtPoint(ev.x, ev.y);
+				if (selectedEntity == INVALID_ENTITY)
 				{
 					if (!Engine.HotkeyIsPressed("selection.add") && !Engine.HotkeyIsPressed("selection.remove"))
 					{
@@ -998,7 +1001,6 @@ function handleInputAfterGui(ev)
 					return true;
 				}
 
-				var selectedEntity = ents[0];
 				var now = new Date();
 
 				// If camera following and we select different unit, stop
@@ -1051,7 +1053,7 @@ function handleInputAfterGui(ev)
 					prevClickedEntity = selectedEntity;
 
 					// We only want to include the first picked unit in the selection
-					ents = [ents[0]];
+					ents = [selectedEntity];
 				}
 
 				// Update the list of selected units
@@ -1092,6 +1094,14 @@ function handleInputAfterGui(ev)
 			}
 			else
 			{
+				// cancel if not enough resources
+				if (placementSupport.template && Engine.GuiInterfaceCall("GetNeededResources", GetTemplateData(placementSupport.template).cost))
+				{
+					placementSupport.Reset();
+					inputState = INPUT_NORMAL;
+					return true;
+				}
+
 				var snapData = Engine.GuiInterfaceCall("GetFoundationSnapData", {
 					"template": placementSupport.template,
 					"x": placementSupport.position.x,
@@ -1229,7 +1239,7 @@ function startBuildingPlacement(buildTemplate, playerState)
 	    templateData.attack.Ranged.maxRange) 
 	{
 		// add attack information to display a good tooltip
-		placementSupport.attack = templateData.attack.Ranged;
+		placementSupport.attack = templateData.attack;
 	}
 }
 
@@ -1248,16 +1258,18 @@ function exchangeResources(command)
 // Camera jumping: when the user presses a hotkey the current camera location is marked.
 // When they press another hotkey the camera jumps back to that position. If the camera is already roughly at that location,
 // jump back to where it was previously.
-var jumpCameraPositions = [], jumpCameraLast;
+var jumpCameraPositions = [];
+var jumpCameraLast;
+var jumpCameraDistanceThreshold = Engine.ConfigDB_GetValue("user", "gui.session.camerajump.threshold");
 
 function jumpCamera(index)
 {
-	var position = jumpCameraPositions[index], distanceThreshold = Engine.ConfigDB_GetValue("user", "camerajump.threshold");
+	var position = jumpCameraPositions[index];
 	if (position)
 	{
 		if (jumpCameraLast &&
-				Math.abs(Engine.CameraGetX() - position.x) < distanceThreshold &&
-				Math.abs(Engine.CameraGetZ() - position.z) < distanceThreshold)
+				Math.abs(Engine.CameraGetX() - position.x) < jumpCameraDistanceThreshold &&
+				Math.abs(Engine.CameraGetZ() - position.z) < jumpCameraDistanceThreshold)
 			Engine.CameraMoveTo(jumpCameraLast.x, jumpCameraLast.z);
 		else
 		{
@@ -1557,7 +1569,12 @@ function performGroup(action, groupId)
 		g_Selection.addList(toSelect);
 
 		if (action == "snap" && toSelect.length)
-			Engine.CameraFollow(toSelect[0]);
+		{
+			let entState = GetEntityState(toSelect[0]);
+			let position = entState.position;
+			if (position && entState.visibility != "hidden")
+				Engine.CameraMoveTo(position.x, position.z);
+		}
 		break;
 	case "save":
 	case "breakUp":
