@@ -105,11 +105,11 @@ function toggleMenu()
 
 // Menu buttons
 // =============================================================================
-function settingsMenuButton()
+function optionsMenuButton()
 {
 	closeMenu();
 	closeOpenDialogs();
-	openSettings();
+	openOptions();
 }
 
 function chatMenuButton()
@@ -148,20 +148,20 @@ function exitMenuButton()
 	pauseGame();
 	if (g_IsNetworked && g_IsController)
 	{
-		var btCode = [leaveGame, resumeGame];
+		var btCode = [resumeGame, leaveGame];
 		var message = translate("Are you sure you want to quit? Leaving will disconnect all other players.");
 	}
 	else if (g_IsNetworked && !g_GameEnded && !g_IsObserver)
 	{
-		var btCode = [networkReturnQuestion, resumeGame];
+		var btCode = [resumeGame, networkReturnQuestion];
 		var message = translate("Are you sure you want to quit?");
 	}
 	else
 	{
-		var btCode = [leaveGame, resumeGame];
+		var btCode = [resumeGame, leaveGame];
 		var message = translate("Are you sure you want to quit?");
 	}
-	messageBox(400, 200, message, translate("Confirmation"), 0, [translate("Yes"), translate("No")], btCode);
+	messageBox(400, 200, message, translate("Confirmation"), 0, [translate("No"), translate("Yes")], btCode);
 }
 
 function networkReturnQuestion()
@@ -201,7 +201,7 @@ function openSave()
 	Engine.PushGuiPage("page_savegame.xml", {"savedGameData":savedGameData, "callback":"resumeGame"});
 }
 
-function openSettings()
+function openOptions()
 {
 	pauseGame();
 	Engine.PushGuiPage("page_options.xml", {"callback":"resumeGame"});
@@ -209,6 +209,8 @@ function openSettings()
 
 function openChat()
 {
+	updateTeamCheckbox(false);
+
 	Engine.GetGUIObjectByName("chatInput").focus(); // Grant focus to the input area
 	Engine.GetGUIObjectByName("chatDialogPanel").hidden = false;
 }
@@ -220,6 +222,14 @@ function closeChat()
 	Engine.GetGUIObjectByName("chatDialogPanel").hidden = true;
 }
 
+function updateTeamCheckbox(check)
+{
+	Engine.GetGUIObjectByName("toggleTeamChatLabel").hidden = g_IsObserver;
+	let toggleTeamChat = Engine.GetGUIObjectByName("toggleTeamChat");
+	toggleTeamChat.hidden = g_IsObserver;
+	toggleTeamChat.checked = !g_IsObserver && check;
+}
+
 function toggleChatWindow(teamChat)
 {
 	var chatWindow = Engine.GetGUIObjectByName("chatDialogPanel");
@@ -237,7 +247,7 @@ function toggleChatWindow(teamChat)
 		chatInput.caption = ""; // Clear chat input
 	}
 
-	Engine.GetGUIObjectByName("toggleTeamChat").checked = teamChat;
+	updateTeamCheckbox(teamChat);
 	chatWindow.hidden = !chatWindow.hidden;
 }
 
@@ -257,113 +267,92 @@ function openDiplomacy()
 		closeTrade();
 	isDiplomacyOpen = true;
 
-	var we = Engine.GetPlayerID();
-	var players = getPlayerData(g_PlayerAssignments);
+	let we = Engine.GetPlayerID();
 
 	// Get offset for one line
-	var onesize = Engine.GetGUIObjectByName("diplomacyPlayer[0]").size;
-	var rowsize = onesize.bottom - onesize.top;
+	let onesize = Engine.GetGUIObjectByName("diplomacyPlayer[0]").size;
+	let rowsize = onesize.bottom - onesize.top;
 
 	// We don't include gaia
-	for (var i = 1; i < players.length; i++)
+	for (let i = 1; i < g_Players.length; ++i)
 	{
 		// Apply offset
-		var row = Engine.GetGUIObjectByName("diplomacyPlayer["+(i-1)+"]");
-		var size = row.size;
+		let row = Engine.GetGUIObjectByName("diplomacyPlayer["+(i-1)+"]");
+		let size = row.size;
 		size.top = rowsize*(i-1);
 		size.bottom = rowsize*i;
 		row.size = size;
 
 		// Set background colour
-		var playerColor = players[i].color.r+" "+players[i].color.g+" "+players[i].color.b;
+		let playerColor = rgbToGuiColor(g_Players[i].color);
 		row.sprite = "colour: "+playerColor + " 32";
 
-		Engine.GetGUIObjectByName("diplomacyPlayerName["+(i-1)+"]").caption = "[color=\"" + playerColor + "\"]" + players[i].name + "[/color]";
-		Engine.GetGUIObjectByName("diplomacyPlayerCiv["+(i-1)+"]").caption = g_CivData[players[i].civ].Name;
+		Engine.GetGUIObjectByName("diplomacyPlayerName["+(i-1)+"]").caption = "[color=\"" + playerColor + "\"]" + g_Players[i].name + "[/color]";
+		Engine.GetGUIObjectByName("diplomacyPlayerCiv["+(i-1)+"]").caption = g_CivData[g_Players[i].civ].Name;
 
-		Engine.GetGUIObjectByName("diplomacyPlayerTeam["+(i-1)+"]").caption = (players[i].team < 0) ? translateWithContext("team", "None") : players[i].team+1;
+		Engine.GetGUIObjectByName("diplomacyPlayerTeam["+(i-1)+"]").caption = (g_Players[i].team < 0) ? translateWithContext("team", "None") : g_Players[i].team+1;
 
 		if (i != we)
-			Engine.GetGUIObjectByName("diplomacyPlayerTheirs["+(i-1)+"]").caption = (players[i].isAlly[we] ? translate("Ally") : (players[i].isNeutral[we] ? translate("Neutral") : translate("Enemy")));
+			Engine.GetGUIObjectByName("diplomacyPlayerTheirs["+(i-1)+"]").caption = (g_Players[i].isAlly[we] ? translate("Ally") : (g_Players[i].isNeutral[we] ? translate("Neutral") : translate("Enemy")));
 
 		// Don't display the options for ourself, or if we or the other player aren't active anymore
-		if (i == we || players[we].state != "active" || players[i].state != "active")
+		if (i == we || g_Players[we].state != "active" || g_Players[i].state != "active")
 		{
 			// Hide the unused/unselectable options
-			for each (var a in ["TributeFood", "TributeWood", "TributeStone", "TributeMetal", "Ally", "Neutral", "Enemy"])
+			for each (let a in ["TributeFood", "TributeWood", "TributeStone", "TributeMetal", "Ally", "Neutral", "Enemy"])
 				Engine.GetGUIObjectByName("diplomacyPlayer"+a+"["+(i-1)+"]").hidden = true;
 			continue;
 		}
 
 		// Tribute
-		for each (var resource in ["food", "wood", "stone", "metal"])
+		for each (let resource in ["food", "wood", "stone", "metal"])
 		{
-			var button = Engine.GetGUIObjectByName("diplomacyPlayerTribute"+toTitleCase(resource)+"["+(i-1)+"]");
+			let button = Engine.GetGUIObjectByName("diplomacyPlayerTribute"+resource[0].toUpperCase()+resource.substring(1)+"["+(i-1)+"]");
 			button.onpress = (function(player, resource, button){
 				// Implement something like how unit batch training works. Shift+click to send 500, shift+click+click to send 1000, etc.
 				// Also see input.js (searching for "INPUT_MASSTRIBUTING" should get all the relevant parts).
-				var multiplier = 1;
+				let multiplier = 1;
 				return function() {
-					var isBatchTrainPressed = Engine.HotkeyIsPressed("session.masstribute");
+					let isBatchTrainPressed = Engine.HotkeyIsPressed("session.masstribute");
 					if (isBatchTrainPressed)
 					{
 						inputState = INPUT_MASSTRIBUTING;
 						multiplier += multiplier == 1 ? 4 : 5;
 					}
-					var amounts = {
+					let amounts = {
 						"food": (resource == "food" ? 100 : 0) * multiplier,
 						"wood": (resource == "wood" ? 100 : 0) * multiplier,
 						"stone": (resource == "stone" ? 100 : 0) * multiplier,
 						"metal": (resource == "metal" ? 100 : 0) * multiplier,
 					};
-					button.tooltip = formatTributeTooltip(players[player], resource, amounts[resource]);
+					button.tooltip = formatTributeTooltip(g_Players[player], resource, amounts[resource]);
 					// This is in a closure so that we have access to `player`, `amounts`, and `multiplier` without some
 					// evil global variable hackery.
 					flushTributing = function() {
 						tributeResource({"player": player, "amounts": amounts});
 						multiplier = 1;
-						button.tooltip = formatTributeTooltip(players[player], resource, 100);
+						button.tooltip = formatTributeTooltip(g_Players[player], resource, 100);
 					};
 					if (!isBatchTrainPressed)
 						flushTributing();
 				};
 			})(i, resource, button);
 			button.hidden = false;
-			button.tooltip = formatTributeTooltip(players[i], resource, 100);
+			button.tooltip = formatTributeTooltip(g_Players[i], resource, 100);
 		}
 
 		// Skip our own teams on teams locked
-		if (players[we].teamsLocked && players[we].team != -1 && players[we].team == players[i].team)
+		if (g_Players[we].teamsLocked && g_Players[we].team != -1 && g_Players[we].team == g_Players[i].team)
 			continue;
 
 		// Diplomacy settings
 		// Set up the buttons
-		for each (var setting in ["ally", "neutral", "enemy"])
+		for each (let setting in ["Ally", "Neutral", "Enemy"])
 		{
-			var button = Engine.GetGUIObjectByName("diplomacyPlayer"+toTitleCase(setting)+"["+(i-1)+"]");
+			let button = Engine.GetGUIObjectByName("diplomacyPlayer"+setting+"["+(i-1)+"]");
 
-			if (setting == "ally")
-			{
-				if (players[we].isAlly[i])
-					button.caption = translate("x");
-				else
-					button.caption = "";
-			}
-			else if (setting == "neutral")
-			{
-				if (players[we].isNeutral[i])
-					button.caption = translate("x");
-				else
-					button.caption = "";
-			}
-			else // "enemy"
-			{
-				if (players[we].isEnemy[i])
-					button.caption = translate("x");
-				else
-					button.caption = "";
-			}
-			button.onpress = (function(e){ return function() { setDiplomacy(e) } })({"player": i, "to": setting});
+			button.caption = g_Players[we]["is"+setting][i] ? translate("x") : "";
+			button.onpress = (function(e){ return function() { setDiplomacy(e) } })({"player": i, "to": setting.toLowerCase()});
 			button.hidden = false;
 		}
 	}
@@ -692,7 +681,7 @@ function closeOpenDialogs()
 
 function formatTributeTooltip(player, resource, amount)
 {
-	var playerColor = player.color.r + " " + player.color.g + " " + player.color.b;
+	let playerColor = rgbToGuiColor(player.color);
 	return sprintf(translate("Tribute %(resourceAmount)s %(resourceType)s to %(playerName)s. Shift-click to tribute %(greaterAmount)s."), {
 		resourceAmount: amount,
 		resourceType: getLocalizedResourceName(resource, "withinSentence"),
