@@ -17,7 +17,9 @@ function getCheatsData()
 	var cheatFileList = getJSONFileList("simulation/data/cheats/");
 	for each (var fileName in cheatFileList)
 	{
-		var currentCheat = parseJSONData("simulation/data/cheats/"+fileName+".json");
+		var currentCheat = Engine.ReadJSONFile("simulation/data/cheats/"+fileName+".json");
+		if (!currentCheat)
+			continue;
 		if (Object.keys(cheats).indexOf(currentCheat.Name) !== -1)
 			warn("Cheat name '" + currentCheat.Name + "' is already present");
 		else
@@ -50,8 +52,12 @@ var g_NotificationsTypes =
 			"type": "message",
 			"text": notification.message
 		}
-		if (notification.type == "aichat")
-			message["translate"] = true;
+		message["translate"] = true;
+		if ("translateParameters" in notification)
+		{
+			message["translateParameters"] = notification["translateParameters"];
+			message["parameters"] = notification["parameters"];
+		}
 		var guid = findGuidForPlayerID(g_PlayerAssignments, player);
 		if (guid == undefined)
 		{
@@ -70,9 +76,7 @@ var g_NotificationsTypes =
 			"player": player
 		});
 
-		// If the diplomacy panel is open refresh it.
-		if (isDiplomacyOpen)
-			openDiplomacy();
+		updateDiplomacy();
 	},
 	"diplomacy": function(notification, player)
 	{
@@ -83,9 +87,7 @@ var g_NotificationsTypes =
 			"status": notification.status
 		});
 
-		// If the diplomacy panel is open refresh it.
-		if (isDiplomacyOpen)
-			openDiplomacy();
+		updateDiplomacy();
 	},
 	"quit": function(notification, player)
 	{
@@ -153,6 +155,15 @@ function handleNotifications()
 		action(notification, player);
 }
 
+function updateDiplomacy()
+{
+	g_Players = getPlayerData(g_PlayerAssignments);
+
+	// If the diplomacy panel is open refresh it.
+	if (isDiplomacyOpen)
+		openDiplomacy();
+}
+
 function updateTimeNotifications()
 {
 	var notifications =  Engine.GuiInterfaceCall("GetTimeNotifications");
@@ -265,7 +276,7 @@ function handleNetMessage(message)
 
 		g_PlayerAssignments = message.hosts;
 
-		if (g_IsController)
+		if (g_IsController && Engine.HasXmppClient())
 		{
 			var players = [ assignment.name for each (assignment in g_PlayerAssignments) ]
 			Engine.SendChangeStateGame(Object.keys(g_PlayerAssignments).length, players.join(", "));
@@ -356,7 +367,8 @@ function submitChatInput()
 			}
 		}
 
-		if (!isCheat)
+		// Observers should only send messages to "/all"
+		if (!isCheat && (!g_IsObserver || text.indexOf("/") == -1 || text.indexOf("/all ") == 0))
 		{
 			if (Engine.GetGUIObjectByName("toggleTeamChat").checked)
 				text = "/team " + text;
@@ -413,9 +425,9 @@ function addChatMessage(msg, playerAssignments)
 		playerColor = "255 255 255";
 		username = translate("Unknown player");
 	}
-	
+
 	var formatted;
-	
+
 	switch (msg.type)
 	{
 	case "connect":
@@ -463,13 +475,16 @@ function addChatMessage(msg, playerAssignments)
 		[username, playerColor] = getUsernameAndColor(msg.player1);
 
 		// Format the amounts to proper English: 200 food, 100 wood, and 300 metal; 100 food; 400 wood and 200 stone
-		var amounts = Object.keys(msg.amounts)
+		let amounts = Object.keys(msg.amounts)
 			.filter(function (type) { return msg.amounts[type] > 0; })
-			.map(function (type) { return msg.amounts[type] + " " + type; });
+			.map(function (type) { return sprintf(translate("%(amount)s %(resourceType)s"), {
+				"amount": msg.amounts[type],
+				"resourceType": getLocalizedResourceName(type, "withinSentence")});
+			});
 
 		if (amounts.length > 1)
 		{
-			var lastAmount = amounts.pop();
+			let lastAmount = amounts.pop();
 			amounts = sprintf(translate("%(previousAmounts)s and %(lastAmount)s"), {
 				previousAmounts: amounts.join(translate(", ")),
 				lastAmount: lastAmount
@@ -495,9 +510,17 @@ function addChatMessage(msg, playerAssignments)
 
 		var message;
 		if ("translate" in msg && msg.translate)
-			message = translate(msg.text); // No need to escape, not a use message.
+		{
+			message = translate(msg.text); // No need to escape, not a user message.
+			if ("translateParameters" in msg && msg.translateParameters)
+			{
+				var parameters = msg.parameters || {};
+				translateObjectKeys(parameters, msg.translateParameters);
+				message = sprintf(message, parameters);
+			}
+		}
 		else
-			message = escapeText(msg.text)
+			message = escapeText(msg.text);
 
 		if (msg.action)
 		{
@@ -569,7 +592,9 @@ function parseChatCommands(msg, playerAssignments)
 		sender = playerAssignments[msg.guid].player;
 	else
 		sender = msg.player;
-	
+
+	// TODO: It would be nice to display multiple different contexts.
+	// It should be made clear that only players matching the union of those receive the message.
 	var recurse = false;
 	var split = msg.text.split(/\s/);
 
@@ -595,15 +620,24 @@ function parseChatCommands(msg, playerAssignments)
 			msg.hide = true;
 		recurse = true;
 		break;
+	case "/ally":
+	case "/allies":
+		// Check if we sent the message, or are the sender's (mutual) ally
+		if (Engine.GetPlayerID() == sender || (g_Players[sender] && g_Players[sender].isMutualAlly[Engine.GetPlayerID()]))
+			msg.context = translate("Ally");
+		else
+			msg.hide = true;
+
+		recurse = true;
+		break;
 	case "/enemy":
-		// Check if we are in a team.
-		if (g_Players[Engine.GetPlayerID()] && g_Players[Engine.GetPlayerID()].team != -1)
-		{
-			if (g_Players[Engine.GetPlayerID()].team == g_Players[sender].team && sender != Engine.GetPlayerID())
-				msg.hide = true;
-			else
-				msg.context = translate("Enemy");
-		}
+	case "/enemies":
+		// Check if we sent the message, or are the sender's enemy
+		if (Engine.GetPlayerID() == sender || (g_Players[sender] && g_Players[sender].isEnemy[Engine.GetPlayerID()]))
+			msg.context = translate("Enemy");
+		else
+			msg.hide = true;
+
 		recurse = true;
 		break;
 	case "/me":
