@@ -251,7 +251,9 @@ EntitySelection.prototype.update = function()
 		}
 
 		// Remove non-visible units (e.g. moved back into fog-of-war)
-		if (entState.visibility == "hidden")
+		// At the next update, mirages will be renamed to the real 
+		// entity they replace, so just ignore them now
+		if (entState.visibility == "hidden" && !entState.mirage)
 		{
 			// Disable any highlighting of the disappeared unit
 			_setHighlight([ent], 0, false);
@@ -319,7 +321,7 @@ EntitySelection.prototype.addList = function(ents, quiet)
 	{
 		// Only add entities we own to our selection
 		var entState = GetEntityState(ent);
-		if (!this.selected[ent] && (selection.length + i) <= MAX_SELECTION_SIZE && (allowUnownedSelect || (entState && entState.player == playerID)))
+		if (!this.selected[ent] && (selection.length + i) <= MAX_SELECTION_SIZE && entState && (allowUnownedSelect || entState.player == playerID))
 		{
 			added.push(ent);
 			this.selected[ent] = ent;
