@@ -41,7 +41,7 @@ function displaySingle(entState, template)
 	// Indicate disconnected players by prefixing their name
 	if (g_Players[entState.player].offline)
 	{
-		playerName = sprintf(translate("[OFFLINE] %(player)s"), { player: playerName });
+		playerName = sprintf(translate("\\[OFFLINE] %(player)s"), { player: playerName });
 	}
 
 	// Rank
@@ -56,7 +56,7 @@ function displaySingle(entState, template)
 		Engine.GetGUIObjectByName("rankIcon").hidden = true;
 		Engine.GetGUIObjectByName("rankIcon").tooltip = "";
 	}
-								
+
 	// Hitpoints
 	if (entState.hitpoints)
 	{
@@ -75,7 +75,7 @@ function displaySingle(entState, template)
 	{
 		Engine.GetGUIObjectByName("healthSection").hidden = true;
 	}
-	
+
 	// TODO: Stamina
 	var player = Engine.GetPlayerID();
 	if (entState.stamina && (entState.player == player || g_DevSettings.controlAll))
@@ -164,7 +164,7 @@ function displaySingle(entState, template)
 		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = sprintf(translate("Gain: %(amount)s"), { amount: getTradingTooltip(entState.trader.goods.amount) });
 	}
 	// And for number of workers
-	else if (entState.foundation && !entState.mirage)
+	else if (entState.foundation && entState.visibility == "visible")
 	{
 		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
 		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
@@ -172,7 +172,7 @@ function displaySingle(entState, template)
 		Engine.GetGUIObjectByName("resourceCarryingText").caption = entState.foundation.numBuilders + "    ";
 		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = sprintf(translate("Number of builders.\nTasking another to this foundation would speed construction up by %(numb)s%%"), { numb : Math.round((Math.pow((entState.foundation.numBuilders+1)/entState.foundation.numBuilders, 0.7) - 1.0)*100) });
 	}
-	else if (entState.resourceSupply && (!entState.resourceSupply.killBeforeGather || !entState.hitpoints) && !entState.mirage)
+	else if (entState.resourceSupply && (!entState.resourceSupply.killBeforeGather || !entState.hitpoints) && entState.visibility == "visible")
 	{
 		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
 		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
@@ -223,78 +223,13 @@ function displaySingle(entState, template)
 		Engine.GetGUIObjectByName("icon").sprite = "bkFillBlack";
 	}
 
-	var armorLabel = "[font=\"sans-bold-13\"]" + translate("Armor:") + "[/font]"
-	var armorString = sprintf(translate("%(label)s %(details)s"), { label: armorLabel, details: armorTypeDetails(entState.armour) });
+	var armorString = getArmorTooltip(entState.armour);
 
 	// Attack and Armor
 	if ("attack" in entState && entState.attack)
-	{
-		// Rate
-		if (entState.buildingAI)
-			var rateLabel = "[font=\"sans-bold-13\"]" + translate("Interval:") + "[/font]";
-		else
-			var rateLabel = "[font=\"sans-bold-13\"]" + translate("Rate:") + "[/font]";
-
-		var rate = sprintf(translate("%(label)s %(details)s"), {
-			label: rateLabel,
-			details: attackRateDetails(entState)
-		});
-
-		var attack;
-		var label = "[font=\"sans-bold-13\"]" + getAttackTypeLabel(entState.attack.type) + "[/font]"
-		if (entState.attack.type == "Ranged")
-		{
-			var realRange = entState.attack.elevationAdaptedRange;
-			var range =  entState.attack.maxRange;
-			var rangeLabel = "[font=\"sans-bold-13\"]" + translate("Range:") + "[/font]"
-			var relativeRange = Math.round((realRange - range));
-			var meters = "[font=\"sans-10\"][color=\"orange\"]" + translate("meters") + "[/color][/font]";
-
-			if (relativeRange > 0)
-				attack = sprintf(translate("%(label)s %(details)s, %(rangeLabel)s %(range)s %(meters)s (%(relative)s), %(rate)s"), {
-					label: label,
-					details: damageTypeDetails(entState.attack),
-					rangeLabel: rangeLabel,
-					range: Math.round(range),
-					meters: meters,
-					relative: "+" + relativeRange,
-					rate: rate
-				});
-			else if (relativeRange < 0)
-				attack = sprintf(translate("%(label)s %(details)s, %(rangeLabel)s %(range)s %(meters)s (%(relative)s), %(rate)s"), {
-					label: label,
-					details: damageTypeDetails(entState.attack),
-					rangeLabel: rangeLabel,
-					range: Math.round(range),
-					meters: meters,
-					relative: relativeRange,
-					rate: rate
-				});
-			else // don't show when it's 0
-				attack = sprintf(translate("%(label)s %(details)s, %(rangeLabel)s %(range)s %(meters)s, %(rate)s"), {
-					label: label,
-					details: damageTypeDetails(entState.attack),
-					rangeLabel: rangeLabel,
-					range: Math.round(range),
-					meters: meters,
-					rate: rate
-				});
-		}
-		else
-		{
-			attack = sprintf(translate("%(label)s %(details)s, %(rate)s"), {
-				label: label,
-				details: damageTypeDetails(entState.attack),
-				rate: rate
-			});
-		}
-
-		Engine.GetGUIObjectByName("attackAndArmorStats").tooltip = attack + "\n" + armorString;	
-	}
+		Engine.GetGUIObjectByName("attackAndArmorStats").tooltip = getAttackTooltip(entState) + "\n" + armorString;
 	else
-	{
 		Engine.GetGUIObjectByName("attackAndArmorStats").tooltip = armorString;
-	}
 
 	// Icon Tooltip
 	var iconTooltip = "";
