@@ -141,10 +141,7 @@ g_SelectionPanels.Command = {
 	},
 	"setTooltip": function(data)
 	{
-		if (data.item.tooltip)
-			data.button.tooltip = data.item.tooltip
-		else
-			data.button.tooltip = toTitleCase(data.item.name);
+		data.button.tooltip = data.item.tooltip;
 	},
 	"setAction": function(data)
 	{
@@ -820,6 +817,23 @@ g_SelectionPanels.Selection = {
 				else
 					data.carried[carrying.type] = carrying.amount;
 			}
+
+			if (state.trader && state.trader.goods && state.trader.goods.amount)
+			{
+				if (!data.carried)
+					data.carried = {};
+				var amount = state.trader.goods.amount;
+				var type = state.trader.goods.type;
+				var totalGain = amount.traderGain;
+				if (amount.market1Gain)
+					totalGain += amount.market1Gain;
+				if (amount.market2Gain)
+					totalGain += amount.market2Gain;
+				if (data.carried[type])
+					data.carried[type] += totalGain;
+				else
+					data.carried[type] = totalGain;
+			}
 		}
 		return true;
 	},
@@ -943,7 +957,7 @@ g_SelectionPanels.Training = {
 		var tooltip = "";
 		var key = Engine.ConfigDB_GetValue("user", "hotkey.session.queueunit." + (data.i + 1));
 		if (key)
-			tooltip += "[color=\"255 251 131\"][font=\"sans-bold-16\"][" + key + "][/font][/color] ";
+			tooltip += "[color=\"255 251 131\"][font=\"sans-bold-16\"]\\[" + key + "][/font][/color] ";
 
 		tooltip += getEntityNamesFormatted(data.template);
 		tooltip += getVisibleEntityClassesFormatted(data.template);
@@ -971,11 +985,11 @@ g_SelectionPanels.Training = {
 			if (data.template.health)
 				tooltip += "\n[font=\"sans-bold-13\"]" + translate("Health:") + "[/font] " + data.template.health;
 			if (data.template.attack)
-				tooltip += "\n" + getEntityAttack(data.template);
+				tooltip += "\n" + getAttackTooltip(data.template);
 			if (data.template.armour)
-				tooltip += "\n[font=\"sans-bold-13\"]" + translate("Armor:") + "[/font] " + armorTypesToText(data.template.armour);
+				tooltip += "\n" + getArmorTooltip(data.template.armour);
 			if (data.template.speed)
-				tooltip += "\n" + getEntitySpeed(data.template);
+				tooltip += "\n" + getSpeedTooltip(data.template);
 		}
 		tooltip += "[color=\"255 251 131\"]" + formatBatchTrainingString(data.buildingsCountToTrainFullBatch, data.fullBatchSize, data.remainderBatch) + "[/color]";
 		if (!data.technologyEnabled)
